/* $Id: AddPointTool.cc,v 1.6 2004/07/21 06:31:20 pklaus Exp $
 *
 * AddPointTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/add_point.xpm"
#include "../cursors/crosshair.xbm"
#include "../cursors/crosshair_mask.xbm"
#include "AddPointTool.hh"
#include "GDDocument.hh"
#include "BezierPath.hh"
#include "BezierNode.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"
#include "GDApplication.hh"
#include "AddAction.hh"

AddPointTool::AddPointTool() {
	pixmapData = add_point_xpm;
	toolName = "AddPointTool";
	moving = false;
	CursorFactory::createCursorFromData(crosshairCursor,
					crosshair_bits, crosshair_mask_bits,
					crosshair_width, crosshair_height,
					crosshair_x_hot, crosshair_y_hot);
}

void AddPointTool::activate() {
	Tool::activate();
}

void AddPointTool::deactivate() {
	Tool::deactivate();
}

gint AddPointTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;

	if (ev->button == 1) {
		doc = docwin->doc;
		doc->deSelectAll();
		gdouble x = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble y = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		SelectableItem *node = doc->checkCurves(x, y, 
				doc->application->preferences.catchingRange / docwin->osBuffer.scaleFactor);
		if (node && ! strcmp(node->identify(), "BezierNode")) {
			BezierNode *bnode = (BezierNode *) node;
			AddAction *newAction = new AddAction(bnode);
			doc->setAction(newAction);
			BezierNode *newNode = bnode->subDivide(bnode->curveDivision);
			
			doc->redraw();
		}
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint AddPointTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK) {
		GDDocument *doc = docwin->doc;
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint AddPointTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void AddPointTool::setCursor(guint state, DocWindow *docwin) {
	docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
}
