/* $Id: BezierTool.cc,v 1.7 2004/06/19 16:27:26 pklaus Exp $
 *
 * BezierTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/draw.xpm"
#include "../cursors/bezier_common.xbm"
#include "../cursors/bezier_common_mask.xbm"
#include "../cursors/bezier_append.xbm"
#include "../cursors/bezier_append_mask.xbm"
#include "../cursors/bezier_new.xbm"
#include "../cursors/bezier_new_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "../cursors/bezier_change_handle.xbm"
#include "../cursors/bezier_change_handle_mask.xbm"
#include "config.h"
#include "BezierTool.hh"
#include "GDDocument.hh"
#include "CursorFactory.hh"
#include "BezierNode.hh"
#include "BezierPath.hh"
#include "DocWindow.hh"
#include "DocumentMenu.hh"
#include "GDApplication.hh"
#include "AddAction.hh"

extern GDApplication *theApplication;

BezierTool::BezierTool() {
	downButton = -1;
	pixmapData = draw_xpm;
	toolName = "BezierTool";
	moving = false;
	nodeCaught = false;
	CursorFactory::createCursorFromData(bezierCommonCursor,
					bezier_common_bits, bezier_common_mask_bits,
					bezier_common_width, bezier_common_height,
					bezier_common_x_hot, bezier_common_y_hot);
	CursorFactory::createCursorFromData(bezierNewCursor,
					bezier_new_bits, bezier_new_mask_bits,
					bezier_new_width, bezier_new_height,
					bezier_new_x_hot, bezier_new_y_hot);
	CursorFactory::createCursorFromData(bezierAppendCursor,
					bezier_append_bits, bezier_append_mask_bits,
					bezier_append_width, bezier_append_height,
					bezier_append_x_hot, bezier_append_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
	CursorFactory::createCursorFromData(changeHandleCursor,
					bezier_change_handle_bits, bezier_change_handle_mask_bits,
					bezier_change_handle_width, bezier_change_handle_height,
					bezier_change_handle_x_hot, bezier_change_handle_y_hot);
}

void BezierTool::activate() {
	Tool::activate();
}

void BezierTool::deactivate() {
	//g_printerr("BezierTool::deactivate\n");
	theApplication->docMenu->menuDocWindow->doc->currPath = NULL;
	theApplication->docMenu->menuDocWindow->doc->currNode = NULL;
	Tool::deactivate();
}

gint BezierTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {

	GDDocument	*doc;
	
	downButton = ev->button;
	closingPath = false;
	moveNextHandle = true;
	
	if (downButton == 1) {
		doc = docwin->doc;
		doc->context.set_function(GDK_INVERT);
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		SelectableItem *sitem = doc->checkPoints(sx, sy, doc->application->preferences.catchingRange / docwin->osBuffer.scaleFactor, true);
		BezierNode *bnode;
		if ((bnode = (BezierNode *)sitem) && 
					! strcmp(sitem->identify(), "BezierNode") && 
					(!bnode->next || !bnode->prev)) {
			// connect two paths
			if (! bnode->next) {
				// node is end of path
				if (bnode->path == doc->currPath) {
					// caught the end of the current path (which is nothing special)
					if (ev->state & GDK_CONTROL_MASK) {
						bnode->straightHandles = false;
						doc->currNode = bnode;
					} else {
						sx = bnode->onCurve.x;
						sy = bnode->onCurve.y;
						newBezierNode(docwin);
					}
				} else {
					// caught the end of another path
					if (doc->currPath) {
						// attach all nodes to this path
						// and remove the obsolete old path
						closingPath = true;
						bnode->path->reverseOrder();
						takeOverNode(bnode, doc);
					} else {
						doc->currPath = bnode->path;
						doc->activeLayer = doc->findObjectLayer(doc->currPath);
						doc->currNode = bnode;
					}
				}
			} else {
				// node is beginning of path
				if (bnode->path == doc->currPath) {
					// closing the current path
					closingPath = true;
					bnode->prev = doc->currNode;
					doc->currNode->next = bnode;
					doc->currNode = bnode;
					if (ev->state & GDK_CONTROL_MASK) {
						bnode->straightHandles = false;
						moveNextHandle = false;
					}
				} else {
					if (doc->currPath) {
						// another path
						closingPath = true;
						takeOverNode(bnode, doc);
					} else {
						// continue a path at the beginning side
						bnode->path->reverseOrder();
						doc->currPath = bnode->path;
						doc->activeLayer = doc->findObjectLayer(doc->currPath);
						doc->currNode = bnode;
					}
				}
			}
		} else {
			if (sitem) {
				// as magnet()
				sitem->getCoordinates(sx, sy);
			}
			checkAxisConstraint(doc, ev->state);
			newBezierNode(docwin);
		}
		/* set the coordinates of the node point on the curve */
		doc->currPath->deSelectAll();
		doc->currNode->select();
		
		gdouble px = sx + corrX;
		gdouble py = sy + corrY;
		if (doc->currNode->straightHandles || ! moveNextHandle) {
			doc->currNode->setPreviousHandle(px, py);
		}
		doc->currNode->setOnCurvePoint(px, py);
		if (moveNextHandle) {
			doc->currNode->setNextHandle(px, py);
		}
		doc->currNode->drawPreviousHandle(docwin, doc->context);
		doc->currNode->drawNextHandle(docwin, doc->context);
		doc->currNode->drawSegment(docwin, doc->context);
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

void BezierTool::checkAxisConstraint(GDDocument *doc, guint state) {
	corrX = 0;
	corrY = 0;
	if (state & GDK_SHIFT_MASK) {
		// single axis/ equal axis movement
		gdouble lx = doc->currNode->onCurve.x;
		gdouble ly = doc->currNode->onCurve.y;
		gdouble deltax = fabs(sx - lx);
		gdouble deltay = fabs(sy - ly);
		if (deltax > 0 && deltay > 0 && deltax / deltay > 0.5 && deltax / deltay < 2) {
			gdouble delta = deltax > deltay ? deltax : deltay;
			corrX = lx + copysign(delta, sx - lx) - sx;
			corrY = ly + copysign(delta, sy - ly) - sy;
		} else if (deltax > deltay) {
			corrY = ly - sy;
		} else {
			corrX = lx - sx;
		}
	}
}

void BezierTool::newBezierNode(DocWindow *docwin) {
	/* create a new curve element */
	if (! docwin->doc->currPath) {
		BezierPath *newPath = new BezierPath(docwin->doc);
		docwin->doc->addPath(newPath);
		docwin->doc->deSelectAll();
	}
	docwin->doc->currPath->deSelectAll();
	docwin->doc->currNode = new BezierNode(docwin->doc->currPath);
	docwin->doc->currNode->select();
	/* add the element to the path */
	docwin->doc->currPath->addNode(docwin->doc->currNode);
}

void BezierTool::takeOverNode(BezierNode *node, GDDocument *doc) {
	// change path of all nodes
	BezierPath *oldpath = node->path;
	BezierNode *list = node->path->nodes;
	while (list) { // assuming the path is not already closed
		list->path = doc->currPath;
		list = list->next;
	}
	doc->currPath->addNode(node);
	doc->currNode = node;
	// dispose off the obsolete old path
	oldpath->nodes = NULL;
	doc->removeObject(oldpath);
	delete oldpath;
}

gint BezierTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	GdkDrawable	*drawable;
	GdkPoint	gdkpts[SUBDIVISIONS+2];
	gint32		numpts;
	
	GDDocument *doc = docwin->doc;
	if (ev->state & GDK_BUTTON1_MASK) {
		doc->context.set_function(GDK_INVERT);
		doc->currNode->drawPreviousHandle(docwin, doc->context);
		doc->currNode->drawNextHandle(docwin, doc->context);
		doc->currNode->drawSegment(docwin, doc->context);
		gdouble x = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble y = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		nodeCaught = magnet(x, y, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// single axis/ equal axis movement
			gdouble deltax = fabs(x - sx);
			gdouble deltay = fabs(y - sy);
			if (deltax > 0 && deltay > 0 && deltax / deltay > 0.5 && deltax / deltay < 2) {
				gdouble delta = deltax > deltay ? deltax : deltay;
				x = sx + copysign(delta, x - sx);
				y = sy + copysign(delta, y - sy);
			} else if (deltax > deltay) {
				y = sy;
			} else {
				x = sx;
			}
		}
		x += corrX;
		y += corrY;
		if (moveNextHandle) {
			doc->currNode->setNextHandle(x, y);
			doc->currNode->drawNextHandle(docwin, doc->context);
		}
		if (doc->currNode->straightHandles || ! moveNextHandle) {
			doc->currNode->setPreviousHandle(2*doc->currNode->onCurve.x - x, 
											  2*doc->currNode->onCurve.y - y);
			doc->currNode->drawPreviousHandle(docwin, doc->context);
		}
		doc->currNode->drawSegment(docwin, doc->context);
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		gdouble x = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble y = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		SelectableItem *magnetItem = docwin->doc->checkPoints(x, y, 
								docwin->doc->application->preferences.catchingRange / 
								docwin->osBuffer.scaleFactor, true);
		nodeCaught = (magnetItem && 
						! strcmp(magnetItem->identify(), "BezierNode") &&
						( ((BezierNode *) magnetItem)->prev == NULL || 
						  ((BezierNode *) magnetItem)->next == NULL ) );
	}
	setCursor(ev->state, docwin);
	
  return 0;
}

gint BezierTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		// create action event
		AddAction *newAction = new AddAction(doc->currPath, doc->currNode);
		doc->setAction(newAction);
		if (closingPath) {
			doc->currNode = NULL;
			doc->currPath = NULL;
		}
		doc->context.set_function(GDK_SET);
		doc->redraw();
	}
	moving = false;
	setCursor(ev->state, docwin);
	downButton = 0;
	return 0;
}

void BezierTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (nodeCaught) {
			docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
		}
	} else {
		if (nodeCaught) {
			if (state & GDK_CONTROL_MASK) {
				docwin->DocWinDrawingArea->get_window().set_cursor(changeHandleCursor);
			} else {
				docwin->DocWinDrawingArea->get_window().set_cursor(bezierAppendCursor);
			}
		} else if (docwin->doc->currPath) {
			docwin->DocWinDrawingArea->get_window().set_cursor(bezierCommonCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(bezierNewCursor);
		}
	}
}
