/* $Id: DLList.hh,v 1.5 2003/06/24 09:40:53 pklaus Exp $
 *
 * DLList.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _DLLIST_HH
#  define _DLLIST_HH


template<class Item>
class DLList {

public:
	Item *next;
	Item *prev;
	
	DLList() {
		next = NULL;
		prev = NULL;
	};
	
	Item *insert(Item *after, Item *item) {
		if (after == NULL) {
			Item *list = (Item *) this;
			while (list->next) {
				list = list->next;
			}
			list->next = item;
			item->prev = list;
			return (Item *)this;
		} else if (after == (Item *) (-1)) {
			item->prev = NULL;
			this->prev = item;
			item->next = (Item *) this;
			return item;
		} else {
			item->next = after->next;
			item->prev = after;
			item->next->prev = item;
			after->next = item;
			return (Item *) this;
		}
	};
	
	Item *remove(Item *which) {
		Item *item = (Item *)this;
		while (item) {
			if (item == which) {
				if (item->prev) {
					item->prev->next = item->next;
				}
				if (item->next) {
					item->next->prev = item->prev;
				}
				if (which == this) {
					return this->next;
				} else {
					return (Item *) this;
				}
			}
			item = item->next;
		}
		return (Item *) this;
	}
	
	Item *last() {
		Item *item = (Item *)this;
		while (item->next) {
			item = item->next;
		}
		return item;
	}
	
	
	int size() {
		int size = 0;
		Item *item = (Item *)this;
		while (item) {
			size++;
			item = item->next;
		}
		return size;
	};
	
	Item *operator[](unsigned int index) {
		Item *item = (Item *) this;
		while (item && item->next && index--) {
			item = item->next;
		}
		return item;
	}
	
	Item *moveUp(Item *which) {
		Item * previous = which->prev;
		if (previous) {
			if (previous->prev) {
				previous->prev->next = which;
			}
			if (which->next) {
				which->next->prev = previous;
			}
			previous->next = which->next;
			which->next = previous;
			which->prev = previous->prev;
			previous->prev = which;
		}
		if (previous == (Item *) this) {
			return which;
		} else {
			return (Item *) this;
		}
	}
	
	Item *moveDown(Item *which) {
		Item *next = which->next;
		if (next) {
			if (next->next) {
				next->next->prev = which;
			}
			if (which->prev) {
				which->prev->next = next;
			}
			which->next = next->next;
			next->prev = which->prev;
			next->next = which;
			which->prev = next;
		}
		if (which == (Item *) this) {
			return next;
		} else {
			return (Item *) this;
		}
	}
};

#endif
