/* $Id: DocWindow.cc,v 1.24 2004/10/15 07:26:09 pklaus Exp $
 *
 * DocWindow.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include "config.h"
#include "DocWindow.hh"
#include "GDDocument.hh"
#include "DocumentMenu.hh"
#include "Tool.hh"
#include "Properties.hh"
#include "TextProperties.hh"
#include "GDApplication.hh"
#include "Layers.hh"
#include "GradientListOrgDialog.hh"
#include "PatternListOrgDialog.hh"
#include "AlignDistribute.hh"
#include <gtk--/adjustment.h>
#include <glib.h>
#include <libart_lgpl/art_config.h>
#include <libart_lgpl/art_rgb.h>
#include <math.h>

DocWindow::DocWindow(GDDocument *myDoc) {
	doc = myDoc;
	set_data("document", myDoc);
	releaseTool = false;
	doScrollRedraw = true;
	displayWireFrame = false;
	add_accel_group(*(doc->application->docMenu->DocumentMenu_accgrp));
	set_default_size(640, 480);
	osBuffer.origin.x = sizeX / 2 - 320;
	osBuffer.origin.y = sizeY / 2 - 240;
	osBuffer.scaleFactor = 1;
	rulerUnitFactor = doc->application->preferences.rulerUnitFactor;
	updateScrollbarRanges();
	//DocWinDrawingArea->size(sizeX, sizeY);
	DocWinDrawingArea->motion_notify_event.connect(VRuler->motion_notify_event.slot());
	DocWinDrawingArea->motion_notify_event.connect(HRuler->motion_notify_event.slot());
	DocHScrollbar->get_adjustment()->value_changed.connect(SigC::slot(this, &DocWindow::scrollUpdate));
	DocVScrollbar->get_adjustment()->value_changed.connect(SigC::slot(this, &DocWindow::scrollUpdate));
	DocWinDrawingArea->configure_event.connect(SigC::slot(this, &DocWindow::drawingConfigureEvent));
	HRuler->set_metric(GTK_PIXELS);
	VRuler->set_metric(GTK_PIXELS);
}

void DocWindow::setWindowTitle(string title) {
	set_title(title);
}

RectangleArea DocWindow::getContentRectangle() {
	RectangleArea rect;
	rect.left = osBuffer.origin.x;
	rect.top = osBuffer.origin.y;
	rect.right = rect.left + getWidth() / osBuffer.scaleFactor;
	rect.bottom = rect.top + getHeight() / osBuffer.scaleFactor;
	return rect;
}


gint DocWindow::buttonDown(GdkEventButton *ev) {
	motionPosition.x = ev->x;
	motionPosition.y = ev->y;
	if (doc->application->currentTool) {
		doc->application->currentTool->buttonDown(ev,this);
	}
	return 0;
}

gint DocWindow::motionEvent(GdkEventMotion *ev) {
	motionPosition.x = ev->x;
	motionPosition.y = ev->y;
	if (doc->application->currentTool) {
		doc->application->currentTool->motion(ev, this);
	}
	return 0;
}

gint DocWindow::button_up(GdkEventButton *ev) {
	if (doc->application->currentTool) {
		doc->application->currentTool->buttonUp(ev, this);
	}
	if (releaseTool && doc->application->tempOutTool) {
		if (doc->application->currentTool) {
			doc->application->currentTool->deactivate();
		}
		doc->application->currentTool = doc->application->tempOutTool;
		doc->application->currentTool->activate();
	}
	return 0;
}

gint DocWindow::keyPressEvent(GdkEventKey *ev) {
	
	//g_printerr("DocWindow::keyPressEvent state: %d, key: %X\n", ev->state, ev->keyval);
	guint state = ev->state;
	if (ev->keyval == GDK_Shift_L || ev->keyval == GDK_Shift_R) {
		state |= GDK_SHIFT_MASK;
	} else if (ev->keyval == GDK_Control_L || ev->keyval == GDK_Control_R) {
		state |= GDK_CONTROL_MASK;
	} else if (ev->keyval == GDK_Alt_L || ev->keyval == GDK_Alt_R) {
		state |= GDK_MOD1_MASK;
	} else if (ev->keyval == GDK_Meta_L || ev->keyval == GDK_Meta_R) {
		state |= GDK_MOD4_MASK;
	}
	if (doc->application->currentTool && ! (state & GDK_CONTROL_MASK)) { 
		if (doc->application->currentTool->keyPressed(ev, this) != 0) {
			return 0;
		}
	}
	if (! doc->application->currentTool->moving) {
		if (ev->keyval == GDK_Meta_L || ev->keyval == GDK_Meta_R) {
			if (! doc->application->tempOutTool) {
				doc->application->tempOutTool = doc->application->currentTool;
			}
			if (doc->application->currentTool) {
				doc->application->currentTool->deactivate();
			}
			doc->application->currentTool = doc->application->lastSelectionTool;
			doc->application->currentTool->activate();
		} else if (ev->keyval == GDK_Mode_switch) { // Alt Gr key pressed
			if (doc->application->currentTool == doc->application->zoomTool) {
				return 0;
			}
			//g_printerr("DocWindow::keyPressEvent: got Alt Gr key, tool = %s\n", doc->toolWindow->currentTool->toolName);
			if (! doc->application->tempOutTool) {
				doc->application->tempOutTool = doc->application->currentTool;
			}
			if (doc->application->currentTool) {
				doc->application->currentTool->deactivate();
			}
			doc->application->currentTool = doc->application->zoomTool;
			doc->application->currentTool->activate();
		} else if (ev->keyval == GDK_BackSpace || ev->keyval == GDK_Delete) {
			// shortcut for the clear function
			doc->deleteSelectionItems();
		}
	}
	//g_printerr("KeyPressEvent state: %d, key: %X, length: %d\n", state, ev->keyval, ev->length);
	if (doc->application->currentTool) {
		GdkEventMotion mevent;
		mevent.x = motionPosition.x;
		mevent.y = motionPosition.y;
		mevent.state = state;
		doc->application->currentTool->motion(&mevent, this);
	}
	return 0;
}

gint DocWindow::keyReleaseEvent(GdkEventKey *ev) {
	//g_printerr("DocWindow::keyPressEvent state: %d, key: %X\n", ev->state, ev->keyval);
	guint state = ev->state;
	if (ev->keyval == GDK_Shift_L || ev->keyval == GDK_Shift_R) {
		state &= ~GDK_SHIFT_MASK;
	} else if (ev->keyval == GDK_Control_L || ev->keyval == GDK_Control_R) {
		state &= ~GDK_CONTROL_MASK;
	} else if (ev->keyval == GDK_Alt_L || ev->keyval == GDK_Alt_R) {
		state &= ~GDK_MOD1_MASK;
	} else if (ev->keyval == GDK_Meta_L || ev->keyval == GDK_Meta_R) {
		state &= ~GDK_MOD4_MASK;
	}
	if ((ev->keyval == GDK_Meta_L || ev->keyval == GDK_Meta_R ||
					ev->keyval == GDK_Mode_switch) && doc->application->tempOutTool) {
		if (! doc->application->currentTool->moving) {
			//g_printerr("Temporary tool not moving, restore original tool\n");
			if (doc->application->currentTool) {
				doc->application->currentTool->deactivate();
			}
			doc->application->currentTool = doc->application->tempOutTool;
			doc->application->currentTool->activate();
		} else {
			//g_printerr("Temporary tool moving, set flag\n");
			releaseTool = true;
		}
	}
	//g_printerr("KeyReleaseEvent state: %d, key: %X, length: %d\n", state, ev->keyval, ev->length);
	if (doc->application->currentTool) {
		//g_printerr("DocWindow::KeyReleaseEvent Tool: %s\n", doc->toolWindow->currentTool->toolName);
		if (((state & GDK_CONTROL_MASK) == 0) && 
				(doc->application->currentTool->keyReleased(ev, this) == 0)) {
			GdkEventMotion mevent;
			mevent.x = motionPosition.x;
			mevent.y = motionPosition.y;
			mevent.state = state;
			doc->application->currentTool->motion(&mevent, this);
		}
	}
	return 0;
}

void DocWindow::scrollUpdate() {
	//g_printerr("DocWindow::scrollUpdate\n");
	//g_printerr("orx: %f, ory: %f\n", origin.x, origin.y);
	gdouble ox = DocHScrollbar->get_adjustment()->get_value();
	gdouble oy = DocVScrollbar->get_adjustment()->get_value();
	//if (origin.x != ox || origin.y != oy) {
		osBuffer.origin.x = ox;
		osBuffer.origin.y = oy;
		//g_printerr("after: orx: %f, ory: %f\n", origin.x, origin.y);
		updateRulerRanges();
		doc->redraw(this);
	//}
	doScrollRedraw = true;
}

void DocWindow::updateRulerRanges() {
	//g_printerr("Origin: %f,%f   Size: %d,%d\n", origin.x, origin.y, DocWinDrawingArea->width(), DocWinDrawingArea->height());
	gdouble npx = doc->nullPoint.x;
	gdouble npy = doc->nullPoint.y;
	HRuler->set_range(	(osBuffer.origin.x - npx) * rulerUnitFactor, 
						(osBuffer.origin.x - npx + DocWinDrawingArea->width() / osBuffer.scaleFactor) * rulerUnitFactor, 
						(osBuffer.origin.x - npx) * rulerUnitFactor, sizeX * rulerUnitFactor);
	VRuler->set_range(	(osBuffer.origin.y - npy) * rulerUnitFactor, 
						(osBuffer.origin.y - npy + DocWinDrawingArea->height() / osBuffer.scaleFactor) * rulerUnitFactor, 
						(osBuffer.origin.y - npy) * rulerUnitFactor, sizeY * rulerUnitFactor);
}

void DocWindow::updateScrollbarRanges() {
	//g_printerr("DocWindow::updateScrollbarRanges\n");
	gdouble x = osBuffer.origin.x;
	gdouble y = osBuffer.origin.y;
	Gtk::Adjustment *hsa = DocHScrollbar->get_adjustment();
	hsa->set_lower(0);
	hsa->set_upper(sizeX);
	hsa->set_value(x);
	hsa->set_step_increment(5 / osBuffer.scaleFactor);
	hsa->set_page_increment(DocWinDrawingArea->width() / osBuffer.scaleFactor);
	hsa->set_page_size(DocWinDrawingArea->width() / osBuffer.scaleFactor);
	Gtk::Adjustment *vsa = DocVScrollbar->get_adjustment();
	vsa->set_lower(0);
	vsa->set_upper(sizeY);
	vsa->set_value(y);
	vsa->set_step_increment(5 / osBuffer.scaleFactor);
	vsa->set_page_increment(DocWinDrawingArea->height() / osBuffer.scaleFactor);
	vsa->set_page_size(DocWinDrawingArea->height() / osBuffer.scaleFactor);
}

void DocWindow::updateScaling() {
	updateScrollbarRanges();
	updateRulerRanges();
}

gint DocWindow::drawingConfigureEvent(GdkEventConfigure *ev) {
	//g_printerr("drawingConfigureEvent\n");
	updateScrollbarRanges();
	updateRulerRanges();
	doc->redraw(this);
	return 0;
}

gint DocWindow::resizedEvent(GdkEventConfigure *ev) {
	//g_printerr("resizedEvent\n");
	//doc->draw(this);
	return 0;
}

gint DocWindow::onExposeEvent(GdkEventExpose *ev) {
	//g_printerr("DocWindow::onExposeEvent needRedraw:%d area:%d,%d,%d,%d\n", needRedraw, ev->area.x, ev->area.y, ev->area.width, ev->area.height);
	//g_printerr("DocWindow::onExposeEvent width: %d, height: %d, rowstride: %d\n", DocWinDrawingArea->width(), DocWinDrawingArea->height(), pxbRowstride);
	//doc->draw(this);
	RectangleArea rect;
	rect.left = osBuffer.origin.x + (gdouble) ev->area.x / osBuffer.scaleFactor;
	rect.top = osBuffer.origin.y + (gdouble) ev->area.y / osBuffer.scaleFactor;
	rect.right = rect.left + (gdouble) ev->area.width / osBuffer.scaleFactor;
	rect.bottom = rect.top + (gdouble) ev->area.height / osBuffer.scaleFactor;
	if (displayWireFrame) {
		doc->draw(this, rect);
	} else {
		if (needRedraw) {
			doc->draw(this, rect);
			needRedraw = false;
		} else {
			drawPixmap(&(ev->area));
			doc->drawWireFramePageRectangle(this, rect);
		}
	}
	return 0;
}

gint DocWindow::deleteEvent(GdkEventAny *ev) {
	gint ret = doc->closeWindow(this);
	if (! ret) {
		delete this;
	}
	return ret;
}

gint DocWindow::onVRulerButtonPress(GdkEventButton *ev) {
	//g_printerr("DocWindow::onVRulerButtonPress button=%d doc=%lX\n", ev->button, (long) doc);
	rulerUnitSelectConnection = doc->application->preferences.unitPopupMenu->selection_done.connect(slot(this, &DocWindow::rulerUnitsChanged));
	rulerUnitCancelConnection = doc->application->preferences.unitPopupMenu->cancel.connect(slot(this, &DocWindow::rulerUnitsCancel));
	int sel = 0;
	for (vector<gdouble>::iterator it = doc->application->preferences.factors.begin(); it != doc->application->preferences.factors.end(); it++, sel++) {
		//g_printerr("DocWindow::onVRulerButtonPress curr=%f check=%f\n", rulerUnitFactor, *it);
		if (fabs(*it - rulerUnitFactor) < 0.000001) {
			//g_printerr("DocWindow::onVRulerButtonPress selected is %d\n", sel);
			doc->application->preferences.unitPopupMenu->set_active(sel);
			doc->application->preferences.unitPopupMenu->get_active()->set_state(GTK_STATE_SELECTED);
			((Gtk::RadioMenuItem *) doc->application->preferences.unitPopupMenu->get_active())->set_active();
			break;
		}
	}
	doc->application->preferences.unitPopupMenu->popup(ev->button, ev->time);
	return 1;
}

void DocWindow::rulerUnitsChanged() {
	//g_printerr("DocWindow::rulerUnitsChanged oldfac=%f doc=%lX\n", rulerUnitFactor, (long) doc);
	rulerUnitSelectConnection.disconnect();
	rulerUnitCancelConnection.disconnect();
	rulerUnitFactor = doc->application->preferences.factors[(int) doc->application->preferences.unitPopupMenu->get_active()->get_user_data()];
	//g_printerr("DocWindow::rulerUnitsChanged newfac=%f doc=%lX\n", rulerUnitFactor, (long) doc);
	updateRulerRanges();
}

void DocWindow::rulerUnitsCancel() {
	//g_printerr("DocWindow::rulerUnitsCancel doc=%lX\n", (long) doc);
	rulerUnitSelectConnection.disconnect();
	rulerUnitCancelConnection.disconnect();
}


gint DocWindow::onHRulerButtonPress(GdkEventButton *ev) {
	return onVRulerButtonPress(ev);
}

gint DocWindow::onDocWindowMenuPopup(GdkEventButton *ev) {
	GDDocument *doc = (GDDocument *) get_data("document");
	//menu->show();
	doc->application->docMenu->setWindow(this);
	doc->application->docMenu->popup(ev->button, ev->time);
	
	return 1;
}
gint DocWindow::onDocWindowFocus(GdkEventFocus *ev) {
	//g_printerr("DocWindow::focus (docwin: %lX, doc: %lX)\n", this, doc);
	doc->application->docMenu->setWindow(this);
	doc->application->properties->setDocument(doc);
	doc->application->textProperties->setDocument(doc);
	doc->application->layers->setDocument(doc);
	doc->application->gradients->setDocument(doc);
	doc->application->patterns->setDocument(doc);
	//doc->application->customColors->setDocument(doc);
	doc->application->alignDistribute->setDocument(doc);
	return 1;
}

gint DocWindow::onDocWindowFocusOut(GdkEventFocus *ev) {
	//g_printerr("DocWindow::onDocWindowFocusOut\n");
	return 1;
}

void DocWindow::drawPixmap() {
	drawPixmap(NULL);
}

void DocWindow::drawPixmap(GdkRectangle *area) {
	if (osBuffer.pixbuffer) {
		gint x = 0;
		gint y = 0;
		guint width = osBuffer.pxbWidth;
		guint height = osBuffer.pxbHeight;
		guchar *pixbuf = osBuffer.pixbuffer;
		if (area) {
			x = area->x;
			y = area->y;
			pixbuf += (y * width + x) * 3;
			width = area->width;
			height = area->height;
		}
		gdk_draw_rgb_image (DocWinDrawingArea->get_window(), 
						DocWinDrawingArea->get_style()->get_white_gc(), 
						x, y, width, height, GDK_RGB_DITHER_NONE, pixbuf,
						osBuffer.pxbRowstride);
	}
}

void DocWindow::whiteBackground() {
	osBuffer.whiteBackground();
}

void DocWindow::allocPixBuffer() {
	//g_printerr("DocWindow::allocPixBuffer w: %d, h: %d, pxbSize: %d\n", DocWinDrawingArea->width(),  DocWinDrawingArea->height(), pxbSize);
	osBuffer.allocBuffer(DocWinDrawingArea->width() * 3, DocWinDrawingArea->width(), DocWinDrawingArea->height());
}

void OffscreenBuffer::whiteBackground() {
	if (pixbuffer) {
		art_rgb_fill_run(pixbuffer, 0xFF, 0xFF, 0xFF, pxbSize / 3);
	}
}

void OffscreenBuffer::allocBuffer(uint rowstride, uint width, uint height) {
	size_t newSize = rowstride * height;
	if (newSize > pxbSize) {
		//g_printerr("DocWindow::allocPixBuffer: need to alloc new buffer: %d > %d\n", newSize, pxbSize);
		if (pixbuffer) {
			g_free(pixbuffer);
		}
		pixbuffer = (guchar *) g_malloc(newSize);
		pxbSize = newSize;
	}
	pxbRowstride = rowstride;
	pxbHeight = height;
	pxbWidth = width;
}
