/* $Id: DocumentMenu.cc,v 1.31 2004/07/21 06:31:20 pklaus Exp $
 *
 * DocumentMenu.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "config.h"
#include <libgnome/libgnome.h>
#include <gdk/gdkkeysyms.h>
#include <gtk--/accelgroup.h>
#include <gtk--/list.h>
#include <gtk--/checkmenuitem.h>
#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include "DocumentMenu.hh"
#include "DocWindow.hh"
#include "GDDocument.hh"
#include "BezierNode.hh"
#include "BezierPath.hh"
#include "SelectionItemList.hh"
#include "CompoundPath.hh"
#include "ObjectGroup.hh"
#include "ClipGroup.hh"
#include "Layer.hh"
#include "GDApplication.hh"
#include "ClipBoard.hh"
#include "GroupAction.hh"
#include "OrderAction.hh"
#include "AverageAction.hh"
#include "ReverseAction.hh"
#include "ConvertAction.hh"
#include "AttributeAction.hh"
#include "TextAction.hh"
#include "Undo.hh"
#include "TextObject.hh"
#include "PathMingler.hh"
#include "PathMinglerAction.hh"

#include "pixmaps/BooleObjects/unite.xpm"
#include "pixmaps/BooleObjects/intersect.xpm"
#include "pixmaps/BooleObjects/exclude.xpm"
#include "pixmaps/BooleObjects/minus_front.xpm"
#include "pixmaps/BooleObjects/minus_back.xpm"
#include "pixmaps/BooleObjects/trim.xpm"
#include "pixmaps/BooleObjects/divide.xpm"
#include "pixmaps/BooleObjects/outline.xpm"
#include "pixmaps/BooleObjects/crop.xpm"

extern GDApplication *theApplication;


DocumentMenu::DocumentMenu() : Gtk::Menu() {
	menuDocWindow = NULL;
	DocumentMenu_accgrp = Gtk::AccelGroup::create();
	
	Gtk::MenuItem *NewDocument;
	Gtk::MenuItem *OpenDocument;
	Gtk::MenuItem *SaveDocumentAs;
	Gtk::MenuItem *RevertDocument;
	Gtk::MenuItem *separator12;
	Gtk::MenuItem *Place;
	Gtk::MenuItem *separator24;
	Gtk::MenuItem *PrintSetup;
	Gtk::MenuItem *PrintDocument;
	Gtk::MenuItem *separator13;
	Gtk::MenuItem *CloseDocument;
	Gtk::MenuItem *Exit;
	Gtk::MenuItem *separator14;
	Gtk::MenuItem *SelectAll;
	Gtk::MenuItem *DeselectAll;
	Gtk::MenuItem *separator15;
	Gtk::MenuItem *separator16;
	Gtk::MenuItem *separator17;
	Gtk::MenuItem *separator18;
	Gtk::CheckMenuItem *DisplayToggleTemplate;
	Gtk::CheckMenuItem *ToggleGuides;
	Gtk::CheckMenuItem *TogglePageFrame;
	Gtk::CheckMenuItem *ToggleAncors;
	Gtk::MenuItem *Rendered;
	Gtk::MenuItem *Wireframe;
	Gtk::MenuItem *separator19;
	Gtk::MenuItem *Display;
	Gtk::MenuItem *ZoomIn;
	Gtk::MenuItem *ZoomOut;
	Gtk::MenuItem *NoScale;
	Gtk::MenuItem *FitInWindow;
	Gtk::MenuItem *separator20;
	Gtk::MenuItem *NewView;
	Gtk::MenuItem *separator23;
	Gtk::MenuItem *ObjectAttributes;
	Gtk::MenuItem *separator21;
	Gtk::MenuItem *separator22;
	Gtk::MenuItem *Guides;
	Gtk::MenuItem *Mask;
	Gtk::MenuItem *Combine;
	Gtk::MenuItem *separator26;
	Gtk::MenuItem *separator25;
	Gtk::MenuItem *separator27;
	Gtk::MenuItem *pathmingler1;
	Gtk::MenuItem *GraphicAttributesDialog;
	Gtk::MenuItem *TextAttributesDialog;
	Gtk::MenuItem *CustomColorsDialog;
	Gtk::MenuItem *PatternsDialog;
	Gtk::MenuItem *GradientsDialog;
	Gtk::MenuItem *LayersDialog;
	Gtk::MenuItem *aligndistribute;
	Gtk::MenuItem *FileSubMenu;
	Gtk::MenuItem *Edit;
	Gtk::MenuItem *Order;
	Gtk::MenuItem *View;
	Gtk::MenuItem *Object;
	Gtk::MenuItem *Dialogs;
	Gtk::MenuItem *docmenu1;
	{   static GnomeUIInfo FileSubMenu_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_New"), 0,  0, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,  GDK_N, GDK_CONTROL_MASK, 0 }, 
			GNOMEUIINFO_MENU_OPEN_ITEM(0, 0), 
			GNOMEUIINFO_MENU_SAVE_ITEM(0, 0), 
			GNOMEUIINFO_MENU_SAVE_AS_ITEM(0, 0), 
			GNOMEUIINFO_MENU_REVERT_ITEM(0, 0), 
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("Place"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_P, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			GNOMEUIINFO_SEPARATOR, 
			GNOMEUIINFO_MENU_PRINT_SETUP_ITEM(0, 0), 
			GNOMEUIINFO_MENU_PRINT_ITEM(0, 0), 
			GNOMEUIINFO_SEPARATOR, 
			GNOMEUIINFO_MENU_CLOSE_ITEM(0, 0), 
			GNOMEUIINFO_MENU_EXIT_ITEM(0, 0), 
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Edit_menu_uiinfo[] = 
		{	GNOMEUIINFO_MENU_UNDO_ITEM(0, 0), 
			GNOMEUIINFO_MENU_REDO_ITEM(0, 0), 
			GNOMEUIINFO_SEPARATOR, 
			GNOMEUIINFO_MENU_CUT_ITEM(0, 0), 
			GNOMEUIINFO_MENU_COPY_ITEM(0, 0), 
			GNOMEUIINFO_MENU_PASTE_ITEM(0, 0), 
			GNOMEUIINFO_MENU_CLEAR_ITEM(0, 0), 
	      { GNOME_APP_UI_ITEM, N_("_Select All"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_A, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("Deselect All"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_A, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Order_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Repeat Command"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_D, GDK_CONTROL_MASK, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("_Move"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_M, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("_Transform Separate Objects"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("Bring To _Front"), 0,  0, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TOP,  GDK_KP_Add, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("Ra_ise"), 0,  0, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UP,  GDK_KP_Add, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("Bring To _Back"), 0,  0, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOTTOM,  GDK_KP_Subtract, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("L_ower"), 0,  0, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_DOWN,  GDK_KP_Subtract, GDK_CONTROL_MASK, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("_Group"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_G, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Ungroup"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_G, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("_Lock"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_L, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("U_nlock"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_L, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK | GDK_MOD1_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("_Hide"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_4, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Show All"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_3, GDK_CONTROL_MASK, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Display_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_TOGGLEITEM, N_("Toggle _Template"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_TOGGLEITEM, N_("Toggle _Guides"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_TOGGLEITEM, N_("Toggle _Page Frame"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_TOGGLEITEM, N_("Toggle _Ancors"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo View_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Rendered"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_Y, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Wireframe"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_E, GDK_CONTROL_MASK, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_SUBTREE, N_("_Display"), 0,  Display_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Zoom _In"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_9, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("Zoom _Out"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_0, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_1:1"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_1, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Fit In Window"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("_New View"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_V, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo TextJustificationSubmenu_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("Left"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_L, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("Center"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_C, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("Right"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_R, (GdkModifierType)(GDK_CONTROL_MASK | GDK_SHIFT_MASK), 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Guides_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Make"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Release"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_TOGGLEITEM, N_("_Lock"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Mask_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Make"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Release"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Combine_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Make"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Release"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo pathmingler1_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("Unite"), N_("Makes One Object out of all selected"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, unite_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Intersect"), N_("Remove all paths but the intersection of all objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, intersect_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Exclude"), N_("Leaves all paths but the intersection of all objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, exclude_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Minus Front"), N_("Removes the area of the front-most object from all behind objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, minus_front_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Minus Back"), N_("Removes from the area of the front-most object all areas of the behind objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, minus_back_xpm,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("Trim"), N_("Removes all not viewable areas from the respective objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, trim_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Divide"), N_("Cut away objects on all path intersections"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, divide_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Outline"), N_("Cut away objects on all path intersections and convert all objects to outline only"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, outline_xpm,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("Crop"), N_("Does the same as Mask but by reshaping the objects"),  0, 0, 0, GNOME_APP_PIXMAP_DATA, crop_xpm,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Object_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("Object _Attributes"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_O, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("_Reverse Path Direction"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("_Join"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_J, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("A_verage"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_L, GDK_CONTROL_MASK, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_ITEM, N_("Create _Outlines"), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("Text Justification"), 0,  TextJustificationSubmenu_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_SEPARATOR, 
			{ GNOME_APP_UI_SUBTREE, N_("_Guides"), 0,  Guides_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("_Mask"), 0,  Mask_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("_Compound"), 0,  Combine_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("PathMingler"), 0,  pathmingler1_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo Dialogs_menu_uiinfo[] = 
		{	{ GNOME_APP_UI_ITEM, N_("_Graphic Attributes..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_I, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Text Attributes..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_T, GDK_CONTROL_MASK, 0 },
			{ GNOME_APP_UI_ITEM, N_("Custom _Colors..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Patterns..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("G_radients..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_ITEM, N_("_Layers..."), 0,  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_L, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			{ GNOME_APP_UI_ITEM, N_("_Align/Distribute"), N_("Show alignment/distribution window"),  0, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  GDK_A, (GdkModifierType)(GDK_CONTROL_MASK | GDK_MOD1_MASK), 0 },
			GNOMEUIINFO_END 
		};
		static GnomeUIInfo DocMenu_uiinfo[] = 
		{	GNOMEUIINFO_MENU_FILE_TREE(FileSubMenu_menu_uiinfo),
			GNOMEUIINFO_MENU_EDIT_TREE(Edit_menu_uiinfo),
			{ GNOME_APP_UI_SUBTREE, N_("O_rder"), 0,  Order_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("_View"), 0,  View_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("_Object"), 0,  Object_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			{ GNOME_APP_UI_SUBTREE, N_("_Dialogs"), 0,  Dialogs_menu_uiinfo, 0, 0, GNOME_APP_PIXMAP_NONE, 0,  0, (GdkModifierType)0, 0 },
			GNOMEUIINFO_END 
		};
		gnome_app_fill_menu(GTK_MENU_SHELL(this->gtkobj()), DocMenu_uiinfo, DocumentMenu_accgrp->gtkobj(), true, 0);
		NewDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[0].widget));
		OpenDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[1].widget));
		SaveDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[2].widget));
		SaveDocumentAs = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[3].widget));
		RevertDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[4].widget));
		separator12 = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[5].widget));
		Place = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[6].widget));
		separator24 = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[7].widget));
		PrintSetup = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[8].widget));
		PrintDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[9].widget));
		separator13 = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[10].widget));
		CloseDocument = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[11].widget));
		Exit = Gtk::wrap(GTK_MENU_ITEM(FileSubMenu_menu_uiinfo[12].widget));
		UndoItem = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[0].widget));
		Redo = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[1].widget));
		separator14 = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[2].widget));
		Cut = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[3].widget));
		Copy = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[4].widget));
		Paste = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[5].widget));
		Clear = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[6].widget));
		SelectAll = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[7].widget));
		DeselectAll = Gtk::wrap(GTK_MENU_ITEM(Edit_menu_uiinfo[8].widget));
		RepeatCommand = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[0].widget));
		separator15 = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[1].widget));
		MoveObjects = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[2].widget));
		TransformSeparateObjects = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[3].widget));
		separator16 = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[4].widget));
		BringToFront = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[5].widget));
		RaiseObjects = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[6].widget));
		BringToBack = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[7].widget));
		LowerObjects = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[8].widget));
		separator17 = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[9].widget));
		GroupObjects = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[10].widget));
		Ungroup = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[11].widget));
		separator18 = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[12].widget));
		Lock = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[13].widget));
		Unlock = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[14].widget));
		Hide = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[15].widget));
		ShowAll = Gtk::wrap(GTK_MENU_ITEM(Order_menu_uiinfo[16].widget));
		DisplayToggleTemplate = Gtk::wrap(GTK_CHECK_MENU_ITEM(Display_menu_uiinfo[0].widget));
		ToggleGuides = Gtk::wrap(GTK_CHECK_MENU_ITEM(Display_menu_uiinfo[1].widget));
		TogglePageFrame = Gtk::wrap(GTK_CHECK_MENU_ITEM(Display_menu_uiinfo[2].widget));
		ToggleAncors = Gtk::wrap(GTK_CHECK_MENU_ITEM(Display_menu_uiinfo[3].widget));
		Rendered = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[0].widget));
		Wireframe = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[1].widget));
		separator19 = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[2].widget));
		ZoomIn = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[4].widget));
		ZoomOut = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[5].widget));
		NoScale = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[6].widget));
		FitInWindow = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[7].widget));
		separator20 = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[8].widget));
		NewView = Gtk::wrap(GTK_MENU_ITEM(View_menu_uiinfo[9].widget));
		LeftJustify = Gtk::wrap(GTK_MENU_ITEM(TextJustificationSubmenu_menu_uiinfo[0].widget));
		CenterJustify = Gtk::wrap(GTK_MENU_ITEM(TextJustificationSubmenu_menu_uiinfo[1].widget));
		RightJustify = Gtk::wrap(GTK_MENU_ITEM(TextJustificationSubmenu_menu_uiinfo[2].widget));
		MakeGuide = Gtk::wrap(GTK_MENU_ITEM(Guides_menu_uiinfo[0].widget));
		ReleaseGuide = Gtk::wrap(GTK_MENU_ITEM(Guides_menu_uiinfo[1].widget));
		separator23 = Gtk::wrap(GTK_MENU_ITEM(Guides_menu_uiinfo[2].widget));
		LockGuides = Gtk::wrap(GTK_CHECK_MENU_ITEM(Guides_menu_uiinfo[3].widget));
		MakeMask = Gtk::wrap(GTK_MENU_ITEM(Mask_menu_uiinfo[0].widget));
		ReleaseMask = Gtk::wrap(GTK_MENU_ITEM(Mask_menu_uiinfo[1].widget));
		MakeCombine = Gtk::wrap(GTK_MENU_ITEM(Combine_menu_uiinfo[0].widget));
		ReleaseCombine = Gtk::wrap(GTK_MENU_ITEM(Combine_menu_uiinfo[1].widget));
		pathMingleUnite = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[0].widget));
		pathMingelIntersect = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[1].widget));
		pathMingleExclude = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[2].widget));
		pathMingleMinusFront = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[3].widget));
		pathMingleMinusBack = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[4].widget));
		separator27 = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[5].widget));
		pathMingleTrim = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[6].widget));
		pathMingleDivide = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[7].widget));
		pathMingleOutline = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[8].widget));
		pathMingleCrop = Gtk::wrap(GTK_MENU_ITEM(pathmingler1_menu_uiinfo[9].widget));
		ObjectAttributes = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[0].widget));
		ReversePathDirection = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[1].widget));
		separator21 = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[2].widget));
		JoinObjects = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[3].widget));
		Average = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[4].widget));
		separator26 = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[5].widget));
		CreateOutlines = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[6].widget));
		separator25 = Gtk::wrap(GTK_MENU_ITEM(Object_menu_uiinfo[7].widget));
		GraphicAttributesDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[0].widget));
		TextAttributesDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[1].widget));
		CustomColorsDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[2].widget));
		PatternsDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[3].widget));
		GradientsDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[4].widget));
		LayersDialog = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[5].widget));
		aligndistribute = Gtk::wrap(GTK_MENU_ITEM(Dialogs_menu_uiinfo[6].widget));
	}
	NewDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onNewDocument));
	OpenDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onOpenDocument));
	SaveDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onSaveDocument));
	SaveDocumentAs->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onSaveDocumentAs));
	RevertDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onRevertDocument));
	Place->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPlace));
	PrintSetup->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPrintSetup));
	PrintDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPrintDocument));
	CloseDocument->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onCloseDocument));
	Exit->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onExit));
	UndoItem->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onUndo));
	Redo->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onRedo));
	Cut->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onCut));
	Copy->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onCopy));
	Paste->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPaste));
	Clear->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onClear));
	SelectAll->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onSelectAll));
	DeselectAll->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onDeselectAll));
	RepeatCommand->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onRepeatCommand));
	MoveObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onMoveObjects));
	TransformSeparateObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onTransformSeparateObjects));
	BringToFront->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onBringToFront));
	RaiseObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onRaiseObjects));
	BringToBack->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onBringToBack));
	LowerObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onLowerObjects));
	GroupObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onGroupObjects));
	Ungroup->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onUngroup));
	Lock->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onLock));
	Unlock->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onUnlock));
	Hide->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onHide));
	ShowAll->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onShowAll));
	Rendered->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onRendered));
	Wireframe->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onWireframe));
	DisplayToggleTemplate->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onDisplayToggleTemplate));
	ToggleGuides->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onToggleGuides));
	TogglePageFrame->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onTogglePageFrame));
	ToggleAncors->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onToggleAncors));
	ZoomIn->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onZoomIn));
	ZoomOut->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onZoomOut));
	NoScale->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onNoScale));
	FitInWindow->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onFitInWindow));
	NewView->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onNewView));
	ObjectAttributes->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onObjectAttributes));
	ReversePathDirection->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onReversePathDirection));
	JoinObjects->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onJoinObjects));
	Average->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onAverage));
	CreateOutlines->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onCreateOutlines));
	LeftJustify->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onJustifyLeft));
	CenterJustify->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onJustifyCenter));
	RightJustify->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onJustifyRight));
	MakeGuide->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onMakeGuide));
	ReleaseGuide->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onReleaseGuide));
	LockGuides->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onLockGuides));
	MakeMask->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onMakeMask));
	ReleaseMask->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onReleaseMask));
	MakeCombine->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onMakeCombine));
	ReleaseCombine->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onReleaseCombine));
	pathMingleUnite->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleUnite));
	pathMingelIntersect->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleIntersect));
	pathMingleExclude->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleExclude));
	pathMingleMinusFront->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleMinusFront));
	pathMingleMinusBack->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleMinusBack));
	pathMingleTrim->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleTrim));
	pathMingleDivide->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleDivide));
	pathMingleOutline->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleOutline));
	pathMingleCrop->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPathMingleCrop));
	GraphicAttributesDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onGraphicAttributesDialog));
	TextAttributesDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onTextAttributesDialog));
	CustomColorsDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onCustomColorsDialog));
	PatternsDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onPatternsDialog));
	GradientsDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onGradientsDialog));
	LayersDialog->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onLayersDialog));
	aligndistribute->activate.connect(SigC::slot(static_cast<class DocumentMenu*>(this), &DocumentMenu::onAlignDistribute));
}

void DocumentMenu::updateItemSensitivity(GDDocument *doc) {
	if (!menuDocWindow) return;
	UndoItem->set_sensitive(doc->undo->undoPossible());
	Redo->set_sensitive(doc->undo->redoPossible());
	RepeatCommand->set_sensitive(doc->undo->repeatTransformPossible());
	Cut->set_sensitive(doc->selection->selectionNotEmpty());
	Copy->set_sensitive(doc->selection->selectionNotEmpty());
	Clear->set_sensitive(doc->selection->selectionNotEmpty());
	MoveObjects->set_sensitive(doc->selection->selectionNotEmpty());
	BringToFront->set_sensitive(doc->selection->selectionNotEmpty());
	BringToBack->set_sensitive(doc->selection->selectionNotEmpty());
	RaiseObjects->set_sensitive(doc->selection->selectionNotEmpty());
	LowerObjects->set_sensitive(doc->selection->selectionNotEmpty());
	GroupObjects->set_sensitive(doc->selection->selectionNotEmpty());
	Ungroup->set_sensitive(doc->selection->selectionNotEmpty());
	LowerObjects->set_sensitive(doc->selection->selectionNotEmpty());
	MakeMask->set_sensitive(doc->selection->selectionNotEmpty());
	ReleaseMask->set_sensitive(doc->selection->selectionNotEmpty());
	MakeCombine->set_sensitive(doc->selection->selectionNotEmpty());
	ReleaseCombine->set_sensitive(doc->selection->selectionNotEmpty());
	JoinObjects->set_sensitive(doc->selection->selectionNotEmpty());
	Average->set_sensitive(doc->selection->selectionNotEmpty());
	LeftJustify->set_sensitive(doc->selection->selectionNotEmpty());
	RightJustify->set_sensitive(doc->selection->selectionNotEmpty());
	CenterJustify->set_sensitive(doc->selection->selectionNotEmpty());
	MakeGuide->set_sensitive(doc->selection->selectionNotEmpty());
	ReleaseGuide->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleUnite->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingelIntersect->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleExclude->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleMinusFront->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleMinusBack->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleTrim->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleDivide->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleOutline->set_sensitive(doc->selection->selectionNotEmpty());
	pathMingleCrop->set_sensitive(doc->selection->selectionNotEmpty());

	SaveDocument->set_sensitive(true);
	Paste->set_sensitive(! doc->application->clipBoard->empty());
	//g_printerr("DocumentMenu::updateItemSensitivity: paste_sensitive: %d\n", ! doc->application->clipBoard->empty());
	LockGuides->set_active(doc->guidesLocked);
	//g_printerr("DocumentMenu::updateItemSensitivity: guidesLocked: %d doc=%lX\n", doc->guidesLocked, (long) doc);
	
	Lock->set_sensitive(false);
	Unlock->set_sensitive(false);
	Hide->set_sensitive(false);
	ShowAll->set_sensitive(false);
	TransformSeparateObjects->set_sensitive(false);
}

void DocumentMenu::setWindow(DocWindow *docwin) {
	if (selConnection.connected()) {
		selConnection.disconnect();
	}
	menuDocWindow = docwin;
	if (docwin) {
		selConnection = docwin->doc->selection->changed.connect(SigC::slot(this, &DocumentMenu::updateItemSensitivity));
		accelerate(*docwin);
		updateItemSensitivity(docwin->doc);
	}
}

void DocumentMenu::onNewDocument() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->toolWindow->on_new_document_activate();
	}
}
void DocumentMenu::onOpenDocument() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->toolWindow->on_open_document_activate();
	}
}
void DocumentMenu::onSaveDocument() {
	if (menuDocWindow) {
		menuDocWindow->doc->save();
	}
}
void DocumentMenu::onSaveDocumentAs() {
	if (menuDocWindow) {
		menuDocWindow->doc->onSaveDocumentAs();
	}
}
void DocumentMenu::onRevertDocument() {
}
void DocumentMenu::onPlace() {
	if (menuDocWindow) {
		menuDocWindow->doc->onPlaceImage();
	}
}
void DocumentMenu::onPrintSetup() {
}
void DocumentMenu::onPrintDocument() {
	if (menuDocWindow) {
		menuDocWindow->doc->showPrintDialog();
	}
}
void DocumentMenu::onCloseDocument() {
	if (menuDocWindow) {
		DocWindow *oldWin = menuDocWindow;
		if (! menuDocWindow->doc->closeWindow(menuDocWindow)) {
			//g_printerr("DocumentMenu::onCloseDocument: deleting window %lX\n", oldWin);
			delete oldWin;
		}
	}
}
void DocumentMenu::onExit() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->toolWindow->save_exit();
	}
}
void DocumentMenu::onUndo() {
	if (menuDocWindow) {
		menuDocWindow->doc->undo->undo();
	}
}
void DocumentMenu::onRedo() {
	if (menuDocWindow) {
		menuDocWindow->doc->undo->redo();
	}
}
void DocumentMenu::onCut() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->clipBoard->copy(menuDocWindow->doc);
		menuDocWindow->doc->deleteSelectionItems();
	}
}
void DocumentMenu::onCopy() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->clipBoard->copy(menuDocWindow->doc);
	}
}
void DocumentMenu::onPaste() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->clipBoard->paste(menuDocWindow->doc, menuDocWindow);
	}
}
void DocumentMenu::onClear() {
	if (menuDocWindow) {
		menuDocWindow->doc->deleteSelectionItems();
	}
}
void DocumentMenu::onSelectAll() {
	if (menuDocWindow) {
		//g_printerr("DocumentMenu::onSelectAll\n");
		menuDocWindow->doc->selectAll();
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onDeselectAll() {
	if (menuDocWindow) {
		//g_printerr("DocumentMenu::onDeselectAll\n");
		menuDocWindow->doc->deSelectAll();
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onRepeatCommand() {
	if (menuDocWindow) {
		menuDocWindow->doc->undo->repeatTransform();
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onMoveObjects() {
	if (menuDocWindow) {
		menuDocWindow->doc->onMoveObjects();
	}
}
void DocumentMenu::onTransformSeparateObjects() {
}
void DocumentMenu::onBringToFront() {
	if (menuDocWindow) {
		{
			OrderAction *newAction = new OrderAction(ORDERACTION_TYPE_FRONT);
			menuDocWindow->doc->setAction(newAction);
			SelectionItemList selList;
			menuDocWindow->doc->getSelection(selList);
			selList.changePosition(menuDocWindow->doc, GDPOSITION_TOP);
		}
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onRaiseObjects() {
	if (menuDocWindow) {
		{
			OrderAction *newAction = new OrderAction(ORDERACTION_TYPE_UP);
			menuDocWindow->doc->setAction(newAction);
			SelectionItemList selList;
			menuDocWindow->doc->getSelection(selList);
			selList.changePosition(menuDocWindow->doc, GDPOSITION_RAISE);
		}
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onBringToBack() {
	if (menuDocWindow) {
		{
			OrderAction *newAction = new OrderAction(ORDERACTION_TYPE_BACK);
			menuDocWindow->doc->setAction(newAction);
			SelectionItemList selList;
			menuDocWindow->doc->getSelection(selList);
			selList.changePosition(menuDocWindow->doc, GDPOSITION_BOTTOM);
		}
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onLowerObjects() {
	if (menuDocWindow) {
		{
			OrderAction *newAction = new OrderAction(ORDERACTION_TYPE_DOWN);
			menuDocWindow->doc->setAction(newAction);
			SelectionItemList selList;
			menuDocWindow->doc->getSelection(selList);
			selList.changePosition(menuDocWindow->doc, GDPOSITION_LOWER);
		}
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onGroupObjects() {
	if (menuDocWindow) {
		GroupAction *newAction = new GroupAction();
		menuDocWindow->doc->setAction(newAction);
		ObjectGroup *group = new ObjectGroup(menuDocWindow->doc);
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		GObject *lastobject = NULL;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			if (lastobject != object) {
				menuDocWindow->doc->removeObject(object);
				object->prev = NULL;
				object->next = NULL;
				group->addPart(object);
				lastobject = object;
			}
		}
		delete selection;
		if (group->parts) {
			group->selectAll();
			menuDocWindow->doc->activeLayer->addObject(group);
			menuDocWindow->doc->redraw();
		} else {
			delete group;
		}
	}
}
void DocumentMenu::onUngroup() {
	if (menuDocWindow) {
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			GObject *grpobject = NULL;
			while (object) {
				if (! strcmp(object->identify(), "ObjectGroup")) {
					grpobject = object;
				}
				object = object->parent;
			}
			if (grpobject) {
				GroupAction *newAction = new GroupAction((ObjectGroup *) grpobject);
				menuDocWindow->doc->setAction(newAction);
				//g_printerr("DocumentMenu::onUngroup group: %lX\n", grpobject);
				ObjectGroup *group = (ObjectGroup *) grpobject;
				menuDocWindow->doc->removeObject(grpobject);
				GObject *part = group->parts;
				while (part) {
					GObject *nextPart = part->next;
					group->removePart(part);
					part->next = NULL;
					part->prev = NULL;
					menuDocWindow->doc->activeLayer->addObject(part);
					part->selectAll();
					//g_printerr("DocumentMenu::onUngroup ungrouping %lX (%s)\n", part, part->identify());
					part = nextPart;
				}
				group->parts = NULL;
				delete group;
			}
		}
		//g_printerr("DocumentMenu::onUngroup ungrouped\n");
		delete selection;
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onLock() {
}
void DocumentMenu::onUnlock() {
}
void DocumentMenu::onHide() {
}
void DocumentMenu::onShowAll() {
}
void DocumentMenu::onRendered() {
	if (menuDocWindow && menuDocWindow->displayWireFrame) {
		menuDocWindow->displayWireFrame = false;
		menuDocWindow->doc->redraw(menuDocWindow);
	}
}
void DocumentMenu::onWireframe() {
	if (menuDocWindow && ! menuDocWindow->displayWireFrame) {
		menuDocWindow->displayWireFrame = true;
		menuDocWindow->doc->redraw(menuDocWindow);
	}
}
void DocumentMenu::onDisplayToggleTemplate() {
}
void DocumentMenu::onToggleGuides() {
}
void DocumentMenu::onTogglePageFrame() {
}
void DocumentMenu::onToggleAncors() {
}
void DocumentMenu::onZoomIn() {
	//g_printerr("DocumentMenu::onZoomIn\n");
	if (menuDocWindow && menuDocWindow->osBuffer.scaleFactor < 100) {
		menuDocWindow->osBuffer.scaleFactor *= 2;
		menuDocWindow->osBuffer.origin.x += menuDocWindow->DocWinDrawingArea->width() / 2 / menuDocWindow->osBuffer.scaleFactor;
		menuDocWindow->osBuffer.origin.y += menuDocWindow->DocWinDrawingArea->height() / 2 / menuDocWindow->osBuffer.scaleFactor;
		menuDocWindow->doc->redraw(menuDocWindow);
		menuDocWindow->updateScaling();
	}
}

void DocumentMenu::onZoomOut() {
	//g_printerr("DocumentMenu::onZoomOut\n");
	if (menuDocWindow && menuDocWindow->osBuffer.scaleFactor > 0.01) {
		menuDocWindow->osBuffer.scaleFactor /= 2;
		menuDocWindow->osBuffer.origin.x -= menuDocWindow->DocWinDrawingArea->width() / 4 / menuDocWindow->osBuffer.scaleFactor;
		menuDocWindow->osBuffer.origin.y -= menuDocWindow->DocWinDrawingArea->height() / 4 / menuDocWindow->osBuffer.scaleFactor;
		menuDocWindow->doc->redraw(menuDocWindow);
		menuDocWindow->updateScaling();
	}
}
void DocumentMenu::onNoScale() {
	if (menuDocWindow) {
		menuDocWindow->osBuffer.scaleFactor = 1;
		menuDocWindow->doc->redraw(menuDocWindow);
		menuDocWindow->updateScaling();
	}
}
void DocumentMenu::onFitInWindow() {
	if (menuDocWindow) {
		BoundingBox docBox = menuDocWindow->doc->getBoundingBox();
		gdouble facx = menuDocWindow->DocWinDrawingArea->width() / (docBox.right - docBox.left);
		gdouble facy = menuDocWindow->DocWinDrawingArea->height() / (docBox.bottom - docBox.top);
		gdouble bfac = facx > facy ? facx : facy;
		gdouble newfac = 64;
		while (newfac / 2 > bfac) {
			newfac /= 2;
		}
		menuDocWindow->osBuffer.scaleFactor = newfac;
		menuDocWindow->osBuffer.origin.x = docBox.left - (menuDocWindow->DocWinDrawingArea->width() / newfac - docBox.left + docBox.right) / 2;
		menuDocWindow->osBuffer.origin.y = docBox.top - (menuDocWindow->DocWinDrawingArea->height() / newfac - docBox.top + docBox.bottom) / 2;
		menuDocWindow->doc->redraw(menuDocWindow);
		menuDocWindow->updateScaling();
	}
}
void DocumentMenu::onNewView() {
	if (menuDocWindow) {
		menuDocWindow->doc->newDocWindow();
	}
}
void DocumentMenu::onObjectAttributes() {
}

void DocumentMenu::onReversePathDirection() {
	if (menuDocWindow) {
		ReverseAction *newAction = new ReverseAction();
		menuDocWindow->doc->setAction(newAction);
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		GObject *lastobject = NULL;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			if (lastobject != object) {
				object->reverseOrder();
				lastobject = object;
			}
		}
		delete selection;
	}
}

void DocumentMenu::onJoinObjects() {
	if (menuDocWindow) {
		menuDocWindow->doc->onJoinObjects();
	}
}
void DocumentMenu::onAverage() {
	if (menuDocWindow) {
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		if (selection->size() != 2) {
			theApplication->showError(_("To build an average exactly two node points must be selected"));
			return;
		}
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectableItem *item1 = (*it++)->orig;
		SelectableItem *item2 = (*it)->orig;
		if (strcmp(item1->identify(), "BezierNode") ||
				strcmp(item2->identify(), "BezierNode")) {
			theApplication->showError(_("To build an average exactly two node points must be selected"));
			return;
		}
		BezierNode *node1 = (BezierNode *) item1;
		BezierNode *node2 = (BezierNode *) item2;
		// send the Action signal
		AverageAction *newAction = new AverageAction(node1, node2);
		menuDocWindow->doc->setAction(newAction);
		gdouble x = (node1->onCurve.x + node2->onCurve.x) / 2;
		gdouble y = (node1->onCurve.y + node2->onCurve.y) / 2;
		gdouble dx = x - node1->onCurve.x;
		gdouble dy = y - node1->onCurve.y;
		node1->onCurve.x = x;
		node1->onCurve.y = y;
		node1->prevHandle.x += dx;
		node1->prevHandle.y += dy;
		node1->nextHandle.x += dx;
		node1->nextHandle.y += dy;
		node2->onCurve.x = x;
		node2->onCurve.y = y;
		node2->prevHandle.x -= dx;
		node2->prevHandle.y -= dy;
		node2->nextHandle.x -= dx;
		node2->nextHandle.y -= dy;
		delete selection;
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onMakeGuide() {
	if (! menuDocWindow) return;
	GDStyle guideStyle;
	guideStyle.isGuide = true;
	guideStyle.valueMask = GDSTYLE_GUIDE_MASK;
	if (menuDocWindow->doc) {
		AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_GUIDE, guideStyle);
		menuDocWindow->doc->setAction(newAction);
		menuDocWindow->doc->selection->setFillStyle(&guideStyle);
		menuDocWindow->doc->selection->setStrokeStyle(&guideStyle);
		if (menuDocWindow->doc->guidesLocked) {
			menuDocWindow->doc->deSelectAll();
		}
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onReleaseGuide() {
	if (! menuDocWindow) return;
	GDStyle guideStyle;
	guideStyle.isGuide = false;
	guideStyle.valueMask = GDSTYLE_GUIDE_MASK;
	if (menuDocWindow->doc) {
		AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_GUIDE, guideStyle);
		menuDocWindow->doc->setAction(newAction);
		menuDocWindow->doc->selection->setFillStyle(&guideStyle);
		menuDocWindow->doc->selection->setStrokeStyle(&guideStyle);
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onLockGuides() {
	if (menuDocWindow) {
		//g_printerr("Document::onLockGuides changing guidesLocked %d -> %d\n", menuDocWindow->doc->guidesLocked, LockGuides->get_active());
		menuDocWindow->doc->guidesLocked = LockGuides->get_active();
	}
}
void DocumentMenu::onMakeMask() {
	if (menuDocWindow) {
		GroupAction *newAction = new GroupAction(GROUPACTION_TYPE_CLIPGROUP);
		menuDocWindow->doc->setAction(newAction);
		ClipGroup *group = new ClipGroup(menuDocWindow->doc);
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		GObject *lastobject = NULL;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			if (lastobject != object) {
				if (lastobject) {
					menuDocWindow->doc->removeObject(lastobject);
					lastobject->prev = NULL;
					lastobject->next = NULL;
					group->addPart(lastobject);
				}
				lastobject = object;
			}
		}
		if (lastobject) {
			menuDocWindow->doc->removeObject(lastobject);
			lastobject->prev = NULL;
			lastobject->prev = NULL;
			group->addClipPart(lastobject);
		}
		delete selection;
		if (lastobject) {
			group->selectAll();
			menuDocWindow->doc->activeLayer->addObject(group);
			menuDocWindow->doc->redraw();
		} else {
			delete group;
		}
	}
}
void DocumentMenu::onReleaseMask() {
	if (menuDocWindow) {
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			GObject *grpobject = NULL;
			while (object) {
				if (! strcmp(object->identify(), "ClipGroup")) {
					grpobject = object;
				}
				object = object->parent;
			}
			if (grpobject) {
				ClipGroup *group = (ClipGroup *) grpobject;
				GroupAction *newAction = new GroupAction(group);
				menuDocWindow->doc->setAction(newAction);
				GObject *part = group->parts;
				while (part) {
					GObject *nextPart = part->next;
					group->removePart(part);
					part->next = NULL;
					part->prev = NULL;
					menuDocWindow->doc->activeLayer->addObject(part);
					part->selectAll();
					part = nextPart;
				}
				group->parts = NULL;
				part = group->clipparts;
				while (part) {
					GObject *nextPart = part->next;
					group->removePart(part);
					part->next = NULL;
					part->prev = NULL;
					menuDocWindow->doc->activeLayer->addObject(part);
					part->selectAll();
					part = nextPart;
				}
				group->clipparts = NULL;
				menuDocWindow->doc->removeObject(grpobject);
				delete group;
			}
		}
		delete selection;
		menuDocWindow->doc->redraw();
	}
}
void DocumentMenu::onMakeCombine() {
	//g_printerr("DocumentMenu::onMakeCombine\n");
	if (menuDocWindow) {
		GroupAction *newAction = new GroupAction(GROUPACTION_TYPE_COMPOUNDPATH);
		menuDocWindow->doc->setAction(newAction);
		CompoundPath *cpath = new CompoundPath(menuDocWindow->doc);
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		GObject *lastobject = NULL;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			if (object != lastobject && ! strcmp(object->identify(), "BezierPath")) {
				menuDocWindow->doc->removeObject(object);
				object->prev = NULL;
				object->next = NULL;
				cpath->addPart((BezierPath *) object);
				lastobject = object;
			}
		}
		delete selection;
		if (cpath->parts) {
			cpath->selectAll();
			menuDocWindow->doc->activeLayer->addObject(cpath);
			menuDocWindow->doc->redraw();
		} else {
			delete cpath;
		}
	}
	//g_printerr("DocumentMenu::onMakeCombine finished\n");
}
void DocumentMenu::onReleaseCombine() {
	if (menuDocWindow) {
		SelectionItemList *selection = new SelectionItemList();
		menuDocWindow->doc->getSelection(*selection);
		G_List_Iterator<SelectionItem *> it = selection->begin();
		gboolean actionSet = false;
		SelectionItem *item;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			GObject *object = sitem->getGObject();
			while (object && strcmp(object->identify(), "CompoundPath")) {
				object = object->parent;
			}
			if (object) {
				CompoundPath *cpath = (CompoundPath *) object;
				if (! actionSet) {
					GroupAction *newAction = new GroupAction(cpath);
					menuDocWindow->doc->setAction(newAction);
					actionSet = true;
				}
				menuDocWindow->doc->removeObject(object);
				cpath->deSelectAll();
				BezierPath *part = cpath->parts;
				while (part) {
					BezierPath *nextPath = (BezierPath *)part->next;
					cpath->removePart(part);
					part->next = NULL;
					part->prev = NULL;
					menuDocWindow->doc->activeLayer->addObject(part);
					part->selectAll();
					part = nextPath;
				}
				cpath->parts = NULL;
				delete cpath;
			}
		}
		delete selection;
		if (actionSet) {
			menuDocWindow->doc->redraw();
		} else {
			theApplication->showError(_("No compound path selected. Nothing done."));
		}
	}
}

void DocumentMenu::onPathMingleUnite() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_UNION);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleUnite(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleIntersect() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_INTERSECTION);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleIntersect(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleExclude() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_EXCLUSION);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleExclude(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleMinusFront() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_MINUS_FRONT);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleMinusFront(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleMinusBack() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_MINUS_BACK);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleMinusBack(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleTrim() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_TRIM);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleTrim(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleDivide() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_DIVIDE);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleDivide(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleOutline() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_OUTLINE);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleOutline(menuDocWindow->doc);
	}
}

void DocumentMenu::onPathMingleCrop() {
	if (menuDocWindow) {
		PathMinglerAction *newAction = new PathMinglerAction(PATHMINGLERACTION_TYPE_CROP);
		menuDocWindow->doc->setAction(newAction);
		PathMingler::pathMingleCrop(menuDocWindow->doc);
	}
}

void DocumentMenu::onCreateOutlines() {
	if (menuDocWindow) {
		GDDocument *doc = menuDocWindow->doc;
		SelectionItemList *selection = new SelectionItemList();
		doc->getSelection(*selection);
		gboolean actionSet = false;
		G_List_Iterator<SelectionItem *> it = selection->begin();
		SelectionItem *item;
		while ((item = *it++)) {
			SelectableItem *sitem = item->orig;
			if (! strncmp(sitem->identify(), "Text", 4)) {
				if (!actionSet) {
					actionSet = true;
					ConvertAction *newAction = new ConvertAction();
					doc->setAction(newAction);
				}
				TextObject *textObj = (TextObject *)sitem;
				GObject *outlines = textObj->convert2Outline();
				doc->activeLayer->addObject(outlines);
				doc->removeObject(textObj);
				delete textObj;
			}
		}
		if (! actionSet) {
			// no text object found
			theApplication->showError(_("You must choose a text object to convert it to outlines"));
		}
		doc->redraw();
	}
}
void DocumentMenu::onGraphicAttributesDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showProperties();
	}
}
void DocumentMenu::onTextAttributesDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showTextProperties();
	}
}
void DocumentMenu::onCustomColorsDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showCustomColors();
	}
}
void DocumentMenu::onPatternsDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showPatterns();
	}
}
void DocumentMenu::onGradientsDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showGradients();
	}
}
void DocumentMenu::onLayersDialog() {
	if (menuDocWindow) {
		menuDocWindow->doc->application->showLayers();
	}
}
void DocumentMenu::onAlignDistribute() {
	//g_printerr("DocumentMenu::onAlignDistribute\n");
	if (menuDocWindow) {
		menuDocWindow->doc->application->showAlignDistribute();
	}
}

void DocumentMenu::on_docmenu1_activate() {
}


void DocumentMenu::onJustifyLeft() {
	if (menuDocWindow) {
		GDDocument *doc = menuDocWindow->doc;
		TextStyle style;
		style.setJustification(TEXT_STYLE_JUSTIFICATION_LEFT);
		style.setValueMask(TEXT_STYLE_JUSTIFICATION_MASK);
		TextAction *newAction = new TextAction(style);
		doc->setAction(newAction);
		doc->currentTextStyle |= style;
		doc->selection->setTextStyle(style);
		doc->redraw();
	}
}

void DocumentMenu::onJustifyCenter() {
	if (menuDocWindow) {
		GDDocument *doc = menuDocWindow->doc;
		TextStyle style;
		style.setJustification(TEXT_STYLE_JUSTIFICATION_CENTER);
		style.setValueMask(TEXT_STYLE_JUSTIFICATION_MASK);
		TextAction *newAction = new TextAction(style);
		doc->setAction(newAction);
		doc->currentTextStyle |= style;
		doc->selection->setTextStyle(style);
		doc->redraw();
	}
}

void DocumentMenu::onJustifyRight() {
	if (menuDocWindow) {
		GDDocument *doc = menuDocWindow->doc;
		TextStyle style;
		style.setJustification(TEXT_STYLE_JUSTIFICATION_RIGHT);
		style.setValueMask(TEXT_STYLE_JUSTIFICATION_MASK);
		TextAction *newAction = new TextAction(style);
		doc->setAction(newAction);
		doc->currentTextStyle |= style;
		doc->selection->setTextStyle(style);
		doc->redraw();
	}
}
