/* $Id: Expat.cc,v 1.5 2003/01/19 15:33:38 pklaus Exp $
 *
 * Expat.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "Expat.hh"
#include <string.h>
#include <stdio.h>
#include <glib.h>

#define BUFFER_SIZE		1024

void wrapbegintag(void *mydata, const char *name, const char **attrs) {
	if (mydata) {
		ExpatXML *xml = (ExpatXML *) mydata;
		xml->handleBeginTag(name, attrs);
	}
}

void wrapendtag(void *mydata, const char *name) {
	if (mydata) {
		ExpatXML *xml = (ExpatXML *) mydata;
		xml->handleEndTag(name);
	}
}

void wraptextdata(void *mydata, const char *text, int len) {
	if (mydata) {
		ExpatXML *xml = (ExpatXML *) mydata;
		xml->handleText(text, len);
	}
}

Expat::Expat() {
	parser = XML_ParserCreate(NULL);
	XML_SetElementHandler(parser, wrapbegintag, wrapendtag);
	XML_SetCharacterDataHandler(parser, wraptextdata);
	errstring = NULL;
}

int Expat::parse(ExpatXML *rootParser, string filePath) {
	char buffer[BUFFER_SIZE];
	FILE *file;
	rp = rootParser;
	rp->setParser(parser);
	rp->init();
	if ((file = fopen(filePath.c_str(), "r")) == NULL) {
		return -1;
	}
	gboolean done;
	do {
		size_t len = fread(buffer, 1,BUFFER_SIZE, file);
		done = len < BUFFER_SIZE;
		if (! XML_Parse(parser, buffer, len, done)) {
			errstring = XML_ErrorString(XML_GetErrorCode(parser));
			fclose(file);
			return XML_GetCurrentLineNumber(parser);
		}
	} while (! done);
	XML_ParserFree(parser);
	fclose(file);
	return 0;
}


ExpatXML::ExpatXML(ExpatXML *parent, const char *tag, XML_Parser p, const char **attrs) {
	this->parent = parent;
	this->parser = p;
	if (tag) {
		strncpy(this->tagname, tag, MAX_TAG_SIZE);
	}
}

ExpatXML::ExpatXML() {
	parent = NULL;
	parser = NULL;
	tagname[0] = '\0';
}

void ExpatXML::setParser(XML_Parser p) {
	parser = p;
}

void ExpatXML::init() {
	//g_printerr("ExpatXML::init called\n");
	if (parser) {
		XML_SetUserData(parser, this);
	}
}

void ExpatXML::handleBeginTag(const char *name, const char **attrs) {
	//g_printerr("ExpatXML::handleBeginTag: Unhandled tag: %s\n", name);
}

void ExpatXML::handleEndTag(const char *name) {
	//g_printerr("ExpatXML::handleEndTag: name %s\n", name);
	if (!strcmp(name, tagname)) {
		XML_SetUserData(parser, parent);
	}
}

void ExpatXML::handleText(const char *text, int len) {
}

char *ExpatXML::getAttributeValue(const char **attrs, const char *key) {
	const char **tmp = attrs;
	while (*tmp) {
		if (! strcmp(*tmp, key)) {
			tmp++;
			return (char *) *tmp;
		}
		tmp++;
		tmp++;
	}
	return NULL;
}

int ExpatXML::getAttributeValue(const char **attrs, const char *key, int& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval) {
		value = atoi(attval);
		return 0;
	}
	return -1;
}

int ExpatXML::getAttributeValue(const char **attrs, const char *key, float& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval && sscanf(attval, "%f", value) == 1) {
		return 0;
	}
	return -1;
}

int ExpatXML::getAttributeValue(const char **attrs, const char *key, double& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval && sscanf(attval, "%lf", value) == 1) {
		return 0;
	}
	return -1;
}

int ExpatXML::getAttributeValue(const char **attrs, const char *key, char *value, int len) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval) {
		strncpy(value, attval, len);
		return 0;
	}
	return -1;
}
