/* $Id: GDApplication.cc,v 1.13 2004/06/26 14:30:32 pklaus Exp $
 *
 * GDApplication.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include "gestalter.h"
#include "GDApplication.hh"
#include "DocumentMenu.hh"
#include "FontPool.hh"
#include "Properties.hh"
#include "TextProperties.hh"
#include "ToolWindow.hh"
#include "GDDocument.hh"
#include "ClipBoard.hh"
#include "Layers.hh"
#include "GradientListOrgDialog.hh"
#include "PatternListOrgDialog.hh"
#include "AlignDistribute.hh"
#include <gnome--.h>

GDApplication *theApplication;

GDApplication::GDApplication() {
	theApplication = this;
	doQuit = false;
	preferences.load();
	docMenu = new DocumentMenu();
	currentTool = NULL;
	fontPool = new FontPool(this);
	properties = new Properties(this);
	textProperties = new TextProperties(this);
	toolWindow = new ToolWindow(this);
	clipBoard = new ClipBoard();
	layers = new Layers(this);
	alignDistribute = new AlignDistribute(this);
	gradients = new GradientListOrgDialog(this);
	patterns = new PatternListOrgDialog(this);
	//properties->hide();
	documentList = NULL;
	lastSelectionTool = NULL;
	tempOutTool = NULL;
	moveDialog = NULL;
}
	
void GDApplication::addDocument(GDDocument *doc) {
	if (documentList) {
		documentList->insert(0, doc);
	} else {
		documentList = doc;
	}
}

void GDApplication::removeDocument(GDDocument *doc) {
	if (documentList) {
		if (documentList == doc) {
			documentList = doc->next;
		} else {
			documentList->remove(doc);
		}
	}
}

void GDApplication::showProperties() {
	properties->show();
	properties->get_window().raise();
}

void GDApplication::hideProperties() {
	properties->hide();
}

void GDApplication::showTextProperties() {
	textProperties->show();
	textProperties->get_window().raise();
}

void GDApplication::hideTextProperties() {
	textProperties->hide();
}

void GDApplication::showLayers() {
	layers->show();
}

void GDApplication::hideLayers() {
	layers->hide();
}

void GDApplication::showGradients() {
	gradients->show();
}

void GDApplication::hideGradients() {
	gradients->hide();
}

void GDApplication::showPatterns() {
	patterns->show();
}

void GDApplication::hidePatterns() {
	patterns->hide();
}

void GDApplication::showCustomColors() {
	//customColors->show();
}

void GDApplication::hideCustomColors() {
	//customColors->hide();
}

void GDApplication::showAlignDistribute() {
	//g_printerr("GDApplication::showAlignDistribute called\n");
	alignDistribute->show();
}

void GDApplication::hideAlignDistribute() {
	alignDistribute->hide();
}

void GDApplication::showError(string msg) {
	Gnome::MessageBox *mBox = new Gnome::MessageBox(msg.c_str(), GNOME_MESSAGE_BOX_ERROR);
	mBox->append_button(GNOME_STOCK_BUTTON_OK);
	gint ret = mBox->run();
	delete mBox;
}

int GDApplication::showConfirm(string msg) {
	Gnome::MessageBox *mBox = new Gnome::MessageBox(msg.c_str(), GNOME_MESSAGE_BOX_WARNING);
	mBox->append_button(GNOME_STOCK_BUTTON_OK);
	mBox->append_button(GNOME_STOCK_BUTTON_CANCEL);
	gint ret = mBox->run();
	delete mBox;
	return ret;
}

void GDApplication::quit() {
	doQuit = true;
	while (documentList) {
		GDDocument *next = documentList->next;
		if (documentList->closeDocument()) {
			return;
		}
		documentList = next;
	}
	Gtk::Main::quit(); 
}

GDApplication::~GDApplication() {
	delete patterns;
	delete gradients;
	delete layers;
	delete clipBoard;
	delete fontPool; // fontPool must be before toolWindow because toolWindow deletes TextTool and that frees FreeType
	delete textProperties;
	delete properties;
	delete toolWindow;
	delete docMenu;
	delete alignDistribute;
}
