/* $Id: GDApplication.hh,v 1.9 2004/06/26 12:34:41 pklaus Exp $
 *
 * GDApplication.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GDAPPLICATION_HH
#  define _GDAPPLICATION_HH

#include <gtk--.h>
#include "ToolWindow.hh"
#include "Preferences.hh"

class GDDocument;
class Tool;
class DocumentMenu;
class Properties;
class TextProperties;
class FontPool;
class ClipBoard;
class Layers;
class MoveDialog;
class GradientListOrgDialog;
class PatternListOrgDialog;
class AlignDistribute;

class GDApplication : SigC::Object {

public:
	virtual void set_dynamic() { SigC::Object::set_dynamic(); };

protected:
	GDDocument *documentList;

	
public:
	ToolWindow *toolWindow;
	Tool *currentTool;
	Preferences preferences;
	DocumentMenu *docMenu;
	Properties *properties;
	TextProperties *textProperties;
	ClipBoard *clipBoard;
	Layers *layers;
	GradientListOrgDialog *gradients;
	PatternListOrgDialog *patterns;
	MoveDialog *moveDialog;
	AlignDistribute *alignDistribute;
	gboolean doQuit;
	
	Tool *lastSelectionTool;
	Tool *tempOutTool;
	Tool *zoomTool;
	
	FontPool	*fontPool;

	SigC::Signal1<void, Preferences *> prefsChanged;
	
	GDApplication();

	void showProperties();
	void hideProperties();
	
	void showTextProperties();
	void hideTextProperties();

	void showLayers();
	void hideLayers();
	
	void showGradients();
	void hideGradients();
	
	void showPatterns();
	void hidePatterns();
	
	void showCustomColors();
	void hideCustomColors();
	
	void showAlignDistribute();
	void hideAlignDistribute();
	
	void addDocument(GDDocument *doc);
	void removeDocument(GDDocument *doc);
	
	void showError(string msg);
	int showConfirm(string msg);
	
	void quit();
	
	~GDApplication();

};

#endif
