/* $Id: GDCharacter.hh,v 1.13 2004/03/07 21:35:33 pklaus Exp $
 *
 * GDCharacter.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _GDCHARACTER_HH
#  define _GDCHARACTER_HH

#include "gestalter.h"
#include <sigc++/object.h>
#include <sigc++/basic_signal.h>
#include <glib--.h>
#include <string>
#include "TextStyle.hh"
#include "GDStyle.hh"
#include "TransformationMatrix.hh"
#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H

#define CHARCODE_LINEFEED	0x0A

class BezierPath;
class GDDocument;
class GObject;

class GDCharacter {

protected:
	
	TextStyle style;
	GDStyle strokeStyle;
	GDStyle fillStyle;
	guint unicode;

	BezierPath *convertContour(FT_Vector *vectors, char *tags, gint start, gint end, gboolean reverse, GDDocument *doc);
	
public:
	gdouble startX;
	gdouble endX;
	

	GDCharacter();
	GDCharacter(const GDCharacter& from);
	GDCharacter(guint charcode);
	
	string convert2UTF8();

	void setUnicode(guint code);
	guint getUnicode();

	void setStrokeStyle(GDStyle& style);
	GDStyle *getStrokeStyle();

	void setFillStyle(GDStyle& style);
	GDStyle *getFillStyle();

	void setTextStyle(TextStyle& style);
	TextStyle *getTextStyle();
	
	GObject *convert2Outline(GDDocument *doc);
	
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	void setFillMatrix(TransformationMatrix& m);
	void changeDocument(GDDocument *document);
	
	~GDCharacter();

};

#endif
