/* $Id: GDColor.cc,v 1.2 2004/03/07 21:35:33 pklaus Exp $
 *
 * GDColor.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "GDColor.hh"
#include "CustomColor.hh"

GDColor::GDColor() {
	colorType = GDCOLOR_TYPE_RGB;
	color.rgb.r = 0;
	color.rgb.g = 0;
	color.rgb.b = 0;
}

GDColor::GDColor(const GDColor& from) {
	colorType = from.colorType;
	if (colorType == GDCOLOR_TYPE_RGB) {
		color.rgb.r = from.color.rgb.r;
		color.rgb.g = from.color.rgb.g;
		color.rgb.b = from.color.rgb.b;
	} else if (colorType == GDCOLOR_TYPE_CMYK) {
		color.cmyk.c = from.color.cmyk.c;
		color.cmyk.m = from.color.cmyk.m;
		color.cmyk.y = from.color.cmyk.y;
		color.cmyk.k = from.color.cmyk.k;
	} else if (colorType == GDCOLOR_TYPE_CUSTOM) {
		color.custom = from.color.custom;
	}
}

GDColor GDColor::operator=(const GDColor& from) {
	colorType = from.colorType;
	if (colorType == GDCOLOR_TYPE_RGB) {
		color.rgb.r = from.color.rgb.r;
		color.rgb.g = from.color.rgb.g;
		color.rgb.b = from.color.rgb.b;
	} else if (colorType == GDCOLOR_TYPE_CMYK) {
		color.cmyk.c = from.color.cmyk.c;
		color.cmyk.m = from.color.cmyk.m;
		color.cmyk.y = from.color.cmyk.y;
		color.cmyk.k = from.color.cmyk.k;
	} else if (colorType == GDCOLOR_TYPE_CUSTOM) {
		color.custom = from.color.custom;
	}
	return *this;
}

int GDColor::operator==(const GDColor& b) {
	if (colorType != b.colorType) {
		return 0;
	}
	if (colorType == GDCOLOR_TYPE_RGB) {
		if (color.rgb.r != b.color.rgb.r || color.rgb.g != b.color.rgb.g || color.rgb.b != b.color.rgb.b) {
			return 0;
		}
	} else if (colorType == GDCOLOR_TYPE_CMYK) {
		if (color.cmyk.c != b.color.cmyk.c || color.cmyk.m != b.color.cmyk.m ||
				color.cmyk.y != b.color.cmyk.y || color.cmyk.k != b.color.cmyk.k) {
			return 0;
		}
	} else if (colorType == GDCOLOR_TYPE_CUSTOM) {
		if (color.custom != b.color.custom) {
			return 0;
		}
	}
	return 1;
}

void GDColor::getRGBColor(guint& r, guint& g, guint& b) const {
	if (colorType == GDCOLOR_TYPE_RGB) {
		r = color.rgb.r;
		g = color.rgb.g;
		b = color.rgb.b;
		return;
	} else if (colorType == GDCOLOR_TYPE_BLACK) {
		r = 0;
		g = 0;
		b = 0;
	} else if (colorType == GDCOLOR_TYPE_WHITE) {
		r = 255;
		g = 255;
		b = 255;
	} else if (colorType == GDCOLOR_TYPE_CUSTOM && color.custom) {
		color.custom->getRGBColor(r, g, b);
	} else {
		r = 0;
		g = 0;
		b = 0;
	}
}

void GDColor::setFromString(gchar *str) {
	if (str[0] == '#') {
		// rgb value
		str++;
		color.rgb.b = strtol(str, NULL, 16);
		color.rgb.r = (color.rgb.b >> 16) & 0xFF;
		color.rgb.g = (color.rgb.b >> 8) & 0xFF;
		color.rgb.b = color.rgb.b & 0xFF;
		colorType = GDCOLOR_TYPE_RGB;
	} else if (str[0] == 'b') {
		colorType = GDCOLOR_TYPE_BLACK;
		color.rgb.r = 0;
		color.rgb.g = 0;
		color.rgb.b = 0;
	} else if (str[0] == 'w') {
		colorType = GDCOLOR_TYPE_WHITE;
		color.rgb.r = 0xFF;
		color.rgb.g = 0xFF;
		color.rgb.b = 0xFF;
	}
}

void GDColor::setRGB(guint r, guint g, guint b) {
	colorType = GDCOLOR_TYPE_RGB;
	color.rgb.r = r;
	color.rgb.g = g;
	color.rgb.b = b;
}

GDColor::~GDColor() {
}
