/* $Id: GDDocument.hh,v 1.25 2005/05/07 17:39:33 pklaus Exp $
 *
 * GDDocument.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _GDDOCUMENT_HH
#  define _GDDOCUMENT_HH

#include "gestalter.h"
#include <gdk--/gc.h>
#include <gtk--/base.h>
#include <glib--.h>
#include "DocWindow.hh"
#include "SelectionItem.hh"
#include "DLList.hh"
#include "GDResource.hh"
#include "TextStyle.hh"
#include "GDStyle.hh"
#include <iostream>
#include <map>
#include <gtk--/fileselection.h>
#include "SelectionList.hh"
#include "TransformationMatrix.hh"
#include "BoundingBox.hh"

class GDApplication;
class BezierNode;
class BezierPath;
class Layer;
class GObject;
class SelectableItem;
class JoinedPointDialog;
class PrintingDialog;
class Undo;
class GDAction;
class Gradient;
class CustomColor;
class Pattern;

class ResourceTransform {
public:
	GDResource *resource;
	TransformationMatrix matrix;
};

typedef map<string, CustomColor *>	CustomColorMap;
typedef map<string, Gradient *> 	GradientMap;
typedef map<string, Pattern *>		PatternMap;
typedef map<string, ResourceTransform>	ResourceMap;

class GDDocument : public DLList<GDDocument>, public virtual SigC::Object {

	Gtk::FileSelection *fileselection;
	Gtk::FileSelection *placeFileSelection;
	SelectionItemList *joinObjectSelection;
	JoinedPointDialog *joinedPointDialog;
	PrintingDialog *printDialog;
	gboolean deleteAfterSave;

public:
	Gdk_GC	context;
	BezierNode *currNode;
	BezierPath *currPath;
	GDApplication *application;
	G_List<DocWindow *> *docWindowList;
	Layer *layers;
	Layer *activeLayer;
	string filePath;
	SelectionList *selection;
	TextStyle currentTextStyle;
	GDStyle currentFillStyle;
	GDStyle currentStrokeStyle;
	Undo *undo;
	CustomColorMap customColors;
	GradientMap gradients;
	PatternMap patterns;

	GradientMap tmpGradients;
	PatternMap tmpPatterns;
	ResourceMap tmpResourceMap;

	gboolean guidesLocked;
	Gdk_Pixmap stippleDither;

	gdouble pageWidth, pageHeight;
	gdouble printWidth, printHeight;
	position printOffset;
	string pageFormat;
	
	position nullPoint;
	position printFactor;
	guint layerNumber;
	
	BoundingBox redrawRect;
	
	GDDocument(GDApplication *gda);
	
	void setFilePath(string filePath);
	
	void deleteSelectionItems();

	void addLayer(Layer *layer);
	void removeLayer(Layer *layer);
	SigC::Signal1<void, GDDocument *> layersChanged;
	SigC::Signal1<gint, GDAction *> setAction;
	
	void addPath(BezierPath *path);
	
	void removeObject(GObject *object);
	Layer *findObjectLayer(GObject *object);
	guint getObjectNumber(GObject *object);

	DocWindow *newDocWindow();
	gint closeWindow(DocWindow *docWin);
	gint closeDocument();
	void updateWindowTitle();
	
	void draw(RectangleArea& drawArea);
	void draw(DocWindow *docwin, RectangleArea& drawArea);
	void redraw();
	void redraw(RectangleArea& drawArea);
	void redraw(DocWindow *docwin);
	void redraw(DocWindow *docwin, RectangleArea& drawArea);
	void drawPageRectangle(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFramePageRectangle(DocWindow *docwin, RectangleArea& drawArea);
	BoundingBox getBoundingBox();

	gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);
	SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);
	GObject *checkArea(gdouble x, gdouble y);
	SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void selectAll();
	void deSelectAll();
	void getSelection(SelectionItemList& selectionList, gint mode = 0);
	void recalcSelection();
	
	void gatherResource(GDResourceList& collection, gint type);
	
	void showPrintDialog();
	void cancelPrintDialog();
	gint deletePrintDialog(GdkEventAny *ev);
	void doPrint();
	
	gint writeXML(ostream& xmlout);
	gint writePS(ostream& psout);
	
	void onSaveDocumentAs();
	void saveAs();
	gint save();
	gint deleteSaveAsDialog(GdkEventAny *ev);
	void cancelSaveAsDialog();
	
	void placeImage();
	void onPlaceImage();
	gint deletePlaceImageDialog(GdkEventAny *ev);
	void cancelPlaceImageDialog();
	
	void onJoinObjects();
	void cancelJoinObjectDialog();
	gint deleteJoinObjectDialog(GdkEventAny *ev);
	void joinObjects();
	
	void onMoveObjects();
	void cancelMoveDialog();
	gint deleteMoveDialog(GdkEventAny *ev);
	void moveObjects();
	void copyMovedObjects();
	SelectionItemList *translateSelection();
	
	~GDDocument();
	
};
#endif
