/* $Id: GDFont.hh,v 1.10 2005/05/07 17:39:33 pklaus Exp $
 *
 * GDFont.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _GDFONT_HH
#  define _GDFONT_HH

#include "gestalter.h"
#include <string>
#include <glib.h>
#include <sigc++/object.h>
#include <sigc++/basic_signal.h>
#include "GDResource.hh"
#include <ft2build.h>
#include FT_FREETYPE_H

#define GDFONT_TYPE_PFA			1
#define GDFONT_TYPE_PFB			2
#define GDFONT_TYPE_TTF			3

class FontCache;

class GDFont : public GDResource, SigC::Object {

private:
	gint pfb2pfaStream(string filepath, ostream& out);

protected:
	FT_Face face;
	int usageCount;
	string family;
	string file;
	string name;
	string foundry;
	string weight;
	string slant;
	string spacing;
	string charset;
	string faceName;
	string psName;
	gboolean isFixedPitch;
	gboolean initialized;
	gint type;

	void type1GetNames(istream& input);

public:

	GDFont(string& filepath, string& name, string& foundry, string& familyName, 
			string& weight, string& slant, string& spacing, string& charset,
			FontCache *fcache);

	void initialize();
	
	void incUsageCount();
	void decUsageCount();
	
	const char *getName();
	const char *getFamily();
	const char *getFilePath();
	const char *getFoundry();
	const char *getWeight();
	const char *getSlant();
	const char *getSpacing();
	const char *getCharset();
	const char *getPSName();
	const char *getFaceName();
	
	string getNameString();
	string getFamilyString();
	string getFilePathString();
	string getFoundryString();
	string getWeightString();
	string getSlantString();
	string getSpacingString();
	string getCharsetString();
	string getPSNameString();
	string getFaceNameString();

	FT_Face getFace();
	
	gint writePS(ostream& psout);
	gint writeXML(ostream& xmlout) { return 0; };
	
	
	~GDFont();
};


#endif
