/* $Id: GDStyle.cc,v 1.26 2005/05/07 17:39:33 pklaus Exp $
 *
 * GDStyle.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "GDStyle.hh"
#include "GDDocument.hh"
#include "Gradient.hh"
#include "CustomColor.hh"
#include "Pattern.hh"
#include "ClipGroup.hh"
#include <math.h>
#include <libart_lgpl/art_bpath.h>
#include <libart_lgpl/art_vpath.h>
#include <libart_lgpl/art_vpath_bpath.h>
#include <libart_lgpl/art_vpath_dash.h>
#include <libart_lgpl/art_svp_vpath_stroke.h>
#include <libart_lgpl/art_svp_point.h>

struct gdstyle_svp_custom_fill_data {
	Gradient *gradient;
	OffscreenBuffer *osBuffer; // picture buffer for patterns
	OffscreenBuffer *maskBuffer; // picture mask for patterns
	gdouble displayFactor; // unit factor for patterns
	guchar *buffer;
	guint rowstride;
	guint width;
	guint height;
	TransformationMatrix inverse;
	void (*render) (void *callback_data, int x, int y, int value);
};

#define weightedval(v,w) ((((v << 8) | 0xff) * ((w << 8) | 0xff)) >> 24)

void gdstyle_pattern_render_point(void *callback_data, int x, int y, int value) {
	gdstyle_svp_custom_fill_data *data = (gdstyle_svp_custom_fill_data *) callback_data;
	position p;
	p.x = x; p.y = y;
	data->inverse.transform(p);
	p.x *= data->displayFactor;
	p.y *= data->displayFactor;
	gdouble xfac = p.x - floor(p.x);
	gdouble yfac = p.y - floor(p.y);
	gint width = data->osBuffer->pxbWidth;
	gint height = data->osBuffer->pxbHeight;
	gint px = (gint) (p.x) % width;
	gint py = (gint) (p.y) % height;
	gint ipx = (gint) p.x;
	if (ipx < 0) {
		px = -ipx % width;
		if (px) {
			px = width - px;
		}
	}
	gint ipy = (gint) p.y;
	if (ipy < 0) {
		py = -ipy % height;
		if (py) {
			py = height - py;
		}
	}
	guint rowstride = data->osBuffer->pxbRowstride;
	guchar *pbuf = data->osBuffer->pixbuffer + rowstride * py + 3 * px;
	guchar *mbuf = data->maskBuffer->pixbuffer + rowstride * py + 3 * px;
	guchar mval = 0;
	gboolean aliased = true;
	if (px + 1 < width && py + 1 < height) {
		gdouble m00 = (gdouble) *mbuf;
		gdouble m01 = (gdouble) mbuf[3];
		gdouble m10 = (gdouble) mbuf[rowstride];
		gdouble m11 = (gdouble) mbuf[rowstride + 3];
		mval = 0xff - (guchar) ((m00 * (1 - xfac) + m01 * xfac) * (1 - yfac) + (m10 * (1 - xfac) + m11 * xfac) * yfac);		
	} else {
		aliased = false;
		mval = 0xff - *mbuf;
	}
	guint val = abs(value) * mval / 0xff;
	guchar *pix = data->buffer + data->rowstride * y + 3 * x;
	//g_printerr("gdstyle_pattern_render_point: x=%d y=%d v=%d p.x=%f p.y=%f px=%d py=%d\n", x, y, val, p.x, p.y, px, py);
	for (int j = 0; j < 3; j++) {
		guchar patval = 0;
		if (aliased) {
			gdouble v00 = (gdouble) *pbuf;
			gdouble v01 = (gdouble) pbuf[3];
			gdouble v10 = (gdouble) pbuf[rowstride];
			gdouble v11 = (gdouble) pbuf[rowstride + 3];
			patval = (guchar) ((v00 * (1 - xfac) + v01 * xfac) * (1 - yfac) + (v10 * (1 - xfac) + v11 * xfac) * yfac);
		} else {
			patval = *pbuf;
			//g_printerr("gdstyle_pattern_render_point: px=%d w=%d py=%d h=%d\n", px, data->osBuffer->pxbWidth, py, data->osBuffer->pxbHeight);
		}
		*pix++ = weightedval(patval, val) + weightedval(*pix, 0xff - val);
		pbuf++;
	}
}

void gdstyle_gradient_render_point(void *callback_data, int x, int y, int value) {
	gdstyle_svp_custom_fill_data *data = (gdstyle_svp_custom_fill_data *) callback_data;
	position p;
	p.x = x; p.y = y;
	data->inverse.transform(p);
	gdouble cindex;
	if (data->gradient->type == GRADIENT_TYPE_LINEAR) {
		cindex = p.x;
	} else {
		/* for type radial no square root is necessary because the
		   color vector was already squared */
		cindex = p.x * p.x + p.y * p.y;
	}
	//g_printerr("gdstyle_gradient_render_point: x=%d y=%d p.x=%f p.y=%f\n", x, y, p.x, p.y);
	guint r, rl, rh, g, gl, gh, b, bl, bh;
	if (cindex <= 0) {
		data->gradient->colorPoints[0].color.getRGBColor(r, g, b);
	} else if (cindex >= 1) {
		data->gradient->colorPoints[data->gradient->colorPoints.size() - 1].color.getRGBColor(r, g, b);
	} else {
		vector<ColorPoint>::iterator it = data->gradient->colorPoints.begin();
		GDColor lowColor = (*it).color;
		gdouble lowPos = (*it).pos;
		while (it != data->gradient->colorPoints.end() && (*it).pos < cindex) {
			lowColor = (*it).color;
			lowPos = (*it).pos;
			it++;
		}
		GDColor hiColor = (*it).color;
		gdouble hiPos = (*it).pos;
		lowColor.getRGBColor(rl, gl, bl);
		hiColor.getRGBColor(rh, gh, bh);
		if (hiPos != lowPos) {
			gdouble cf = (cindex - lowPos) / (hiPos - lowPos);
			r = (guint) ((gint) rl + (gint) ( (gdouble) ((gint) rh - (gint) rl) * cf ));
			g = (guint) ((gint) gl + (gint) ( (gdouble) ((gint) gh - (gint) gl) * cf ));
			b = (guint) ((gint) bl + (gint) ( (gdouble) ((gint) bh - (gint) bl) * cf ));
			//g_printerr("gd_style_gradient_render_point: rl=%d,gl=%d,bl=%d,rh=%d,gh=%d,bh=%d,r=%d,g=%d,b=%d,cf=%f,ci=%f\n",rl,gl,bl,rh,gh,bh,r,g,b,cf,cindex);
		} else {
			r = rl; g = gl; b = bl;
		}
	}
	//g_printerr("gd_style_gradient_render_point: v=%X x=%d y=%d\n", value, x, y);
	guint val = abs(value);
	guchar *pix = data->buffer + data->rowstride * y + 3 * x;
	//g_printerr("gd_style_gradient_render_point: x=%d,y=%d,r=%d,g=%d,b=%d,ci=%f,v=%X,calc: green: 1val:%d 2val:%d,rl=%d,gl=%d,bl=%d,rh=%d,gh=%d,bh=%d\n", x,y,r,g,b,cindex,val, weightedval(g, val), weightedval(*(pix+1), 0xff - val), rl, gl, bl, rh, gh, bh);
	//g_printerr("before: p=%lX,rs=%d,buf=%lX,rv=%d,gv=%d,bv=%d\n", (long) pix,data->rowstride,(long)data->buffer, *pix, *(pix+1), *(pix+2));
	*pix++ = weightedval(r, val) + weightedval(*pix, 0xff - val);
	*pix++ = weightedval(g, val) + weightedval(*pix, 0xff - val);
	*pix++ = weightedval(b, val) + weightedval(*pix, 0xff - val);
	//g_printerr("after: p=%lX,rs=%d,buf=%lX,rv=%d,gv=%d,bv=%d\n", (long) pix,data->rowstride,(long)data->buffer, *(pix-3), *(pix-2), *(pix-1));
};

void gdstyle_svp_custom_fill_cb(void *callback_data, int y, int start, ArtSVPRenderAAStep *steps, int n_steps) {
	//g_printerr("gdstyle_svp_custom_fill_cb: y=%d start=%d steps=%d\n", y, start, n_steps);
	gdstyle_svp_custom_fill_data *data = (gdstyle_svp_custom_fill_data *) callback_data;
	if (n_steps > 0) {
		int run = 0;
		guint x = 0;
		int running_sum = start;
		if (abs(running_sum) <= 0x80ff) {
			// svp content does start on a higher x value so jump over the x
			// values not of interest
			x = steps[0].x;
			running_sum += steps[0].delta;
			run++;
		}
		int val = abs(running_sum) >> 16;
		for (; run < n_steps - 1; x++) {
			//g_printerr("gdstyle_svp_custom_fill_cb: x=%d y=%d rs=%X r=%d srx=%d srd=%d\n", x, y, running_sum, run, steps[run].x, steps[run].delta);
			while (x >= (guint) steps[run].x && run < n_steps - 1) {
				running_sum += steps[run].delta;
				val = abs(running_sum) >> 16;
				run++;
			}
			data->render(callback_data, x, y, val);
		}
		running_sum += steps[run].delta;
		val = abs(running_sum) >> 16;
		run++;
		if (val > 0) {
			for(; x < data->width; x++) {
				data->render(callback_data, x, y, val);
			}
		}
	}
};

GDStyle::GDStyle() {
	drawType = GDSTYLE_DRAWTYPE_OPAQUE;
	strokeWidth = 1;
	lineCaps = GDSTYLE_CAPS_BUTT;
	lineJoins = GDSTYLE_JOINS_BEVEL;
	lineMiterLimit = -1;
	lineType = GDSTYLE_LINETYPE_PASS;
	isGuide = false;
	dashes[0] = -1;
	dashes[1] = -1;
	dashes[2] = -1;
	dashes[3] = -1;
	dashes[4] = -1;
	dashes[5] = -1;
	valueMask = 0xFFFFFFFF;
	opacity = 1.0;
	gradient = NULL;
	pattern = NULL;
}

GDStyle::GDStyle(const GDStyle& style) {
	setColor(&style);
	strokeWidth = style.strokeWidth;
	lineCaps = style.lineCaps;
	lineJoins = style.lineJoins;
	lineMiterLimit = style.lineMiterLimit;
	lineType = style.lineType;
	isGuide = style.isGuide;
	for(int i = 0; i < 6; i++) {
		dashes[i] = style.dashes[i];
	}
	valueMask = style.valueMask;
	tMatrix = style.tMatrix;
	gradient = style.gradient;
	pattern = style.pattern;
	opacity = style.opacity;
}

GDStyle GDStyle::operator=(const GDStyle& style) {
	setColor(&style);
	strokeWidth = style.strokeWidth;
	lineCaps = style.lineCaps;
	lineJoins = style.lineJoins;
	lineMiterLimit = style.lineMiterLimit;
	lineType = style.lineType;
	isGuide = style.isGuide;
	for (int i = 0; i < 6; i++) {
		dashes[i] = style.dashes[i];
	}
	valueMask = style.valueMask;
	gradient = style.gradient;
	pattern = style.pattern;
	tMatrix = style.tMatrix;
	return *this;
}

GDStyle GDStyle::operator&(const GDStyle& b) {
	GDStyle a = *this;
	if (a.drawType != b.drawType) a.valueMask &= ~GDSTYLE_DRAWTYPE_MASK;
	if (a.color.colorType != b.color.colorType) a.valueMask &= ~GDSTYLE_COLORTYPE_MASK;
	if (a.strokeWidth != b.strokeWidth) a.valueMask &= ~GDSTYLE_STROKEWIDTH_MASK;
	if (a.lineType != b.lineType) a.valueMask &= ~GDSTYLE_LINETYPE_MASK;
	if (a.lineCaps != b.lineCaps) a.valueMask &= ~GDSTYLE_CAPS_MASK;
	if (a.lineJoins != b.lineJoins || 
		a.lineMiterLimit != b.lineMiterLimit) a.valueMask &= ~GDSTYLE_JOINS_MASK;
	if (a.isGuide != b.isGuide) a.valueMask &= ~GDSTYLE_GUIDE_MASK;
	guint ar,ag,ab,br,bg,bb;
	a.getRGBColor(ar, ag, ab);
	b.getRGBColor(br, bg, bb);
	if (ar != br || ag != bg || ab != bb) a.valueMask &= ~GDSTYLE_COLOR_MASK;
	if ((a.dashes[0] != b.dashes[0] ||
		a.dashes[1] != b.dashes[1] ||
		a.dashes[2] != b.dashes[2] ||
		a.dashes[3] != b.dashes[3] ||
		a.dashes[4] != b.dashes[4] ||
		a.dashes[5] != b.dashes[5])  &&
		(a.lineType == GDSTYLE_LINETYPE_DASHED ||
		b.lineType == GDSTYLE_LINETYPE_DASHED)) a.valueMask &= ~GDSTYLE_DASHES_MASK;
	if (a.gradient != b.gradient) a.valueMask &= ~GDSTYLE_GRADIENT_MASK;
	if (a.pattern != b.pattern) a.valueMask &= ~GDSTYLE_PATTERN_MASK;
	if (! (a.tMatrix == b.tMatrix)) a.valueMask &= ~GDSTYLE_MATRIX_MASK;
	return a;
}

void GDStyle::operator&=(const GDStyle& gs) {
	valueMask &= gs.valueMask;
	valueMask &= (*this & gs).valueMask;
}

void GDStyle::operator|=(const GDStyle& gs) {
	if (gs.valueMask & GDSTYLE_DRAWTYPE_MASK) {
		drawType = gs.drawType;
	}
	if (gs.valueMask & GDSTYLE_COLORTYPE_MASK) {
		color.colorType = gs.color.colorType;
	}
	if (gs.valueMask & GDSTYLE_STROKEWIDTH_MASK) {
		strokeWidth = gs.strokeWidth;
	}
	if (gs.valueMask & GDSTYLE_LINETYPE_MASK) {
		lineType = gs.lineType;
	}
	if (gs.valueMask & GDSTYLE_COLOR_MASK) {
		color = gs.color;
	}
	if (gs.valueMask & GDSTYLE_DASHES_MASK) {
		for (int i = 0; i < 6; i++) {
			dashes[i] = gs.dashes[i];
		}
	}
	if (gs.valueMask & GDSTYLE_CAPS_MASK) {
		lineCaps = gs.lineCaps;
	}
	if (gs.valueMask & GDSTYLE_JOINS_MASK) {
		lineJoins = gs.lineJoins;
		lineMiterLimit = gs.lineMiterLimit;
	}
	if (gs.valueMask & GDSTYLE_GRADIENT_MASK) {
		gradient = gs.gradient;
	}
	if (gs.valueMask & GDSTYLE_PATTERN_MASK) {
		pattern = gs.pattern;
	}
	if (gs.valueMask & GDSTYLE_MATRIX_MASK) {
		tMatrix = gs.tMatrix;
	}
	if (gs.valueMask & GDSTYLE_GUIDE_MASK) {
		isGuide = gs.isGuide;
	}
	valueMask |= gs.valueMask;
}

int GDStyle::operator==(const GDStyle& gs) {
	if (drawType != gs.drawType) return 0;
	if (color.colorType != gs.color.colorType) return 0;
	if (strokeWidth != gs.strokeWidth) return 0;
	if (lineType != gs.lineType) return 0;
	if (lineCaps != gs.lineCaps) return 0;
	if (lineJoins != gs.lineJoins) return 0;
	if (lineMiterLimit != gs.lineMiterLimit) return 0;
	if (isGuide != gs.isGuide) return 0;
	if (drawType == GDSTYLE_LINETYPE_DASHED) {
		for (int i = 0; i < 6; i++) {
			if (dashes[i] != gs.dashes[i]) return 0;
		}
	}
	guint ar,ag,ab,br,bg,bb;
	getRGBColor(ar, ag, ab);
	gs.getRGBColor(br, bg, bb);
	if (ar != br || ag != bg || ab != bb) return 0;
	if (gradient != gs.gradient) return 0;
	if (pattern != gs.pattern) return 0;
	if (! (tMatrix == gs.tMatrix)) return 0;
	return 1;
}
	

void GDStyle::getRGBColor(guint& r, guint& g, guint& b) const {
	if (drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
		r = 65535;
		g = 0;
		b = 0;
	} else {
		color.getRGBColor(r, g, b);
	}
}

void GDStyle::setColor(const GDStyle *fromStyle) {
	color = fromStyle->color;
	drawType = fromStyle->drawType;
}

void GDStyle::setStrokeWidth(gdouble width) {
	strokeWidth = width;
}


void GDStyle::setColorFromString(gchar *str) {
	drawType = GDSTYLE_DRAWTYPE_OPAQUE;
	color.setFromString(str);
}

void GDStyle::changeDocument(GDDocument *document) {
	if (gradient) {
		gradient = document->gradients[gradient->getName()];
		//g_printerr("GDStyle::changeDocument changed gradient new one is %lX\n", (long) gradient);
	}
	if (pattern) {
		pattern = document->patterns[pattern->getName()];
		//g_printerr("GDStyle::changeDocument changed pattern new one is %lX\n", (long) pattern);
	}
}

void GDStyle::writeColorXML(ostream& xmlout, gchar *name) {
	if (drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
		return;
	} else if (drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		gchar colorval[8];
		
		if (color.colorType == GDCOLOR_TYPE_RGB || color.colorType == GDCOLOR_TYPE_CUSTOM) {
			guint r,g,b;
			color.getRGBColor(r,g,b);
			snprintf(colorval, 8, "#%02hhX%02hhX%02hhX", r, g, b);
		} else if (color.colorType == GDCOLOR_TYPE_BLACK) {
			snprintf(colorval, 8, "black");
		} else if (color.colorType == GDCOLOR_TYPE_WHITE) {
			snprintf(colorval, 8, "white");
		}
		xmlout << " " << name << "=\"" << colorval << "\"";
	}
}

gint GDStyle::writeColorPS(ostream& psout, gboolean include_cmd) {
	if (drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
		return 0;
	} else if (drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		if (color.colorType == GDCOLOR_TYPE_RGB || color.colorType == GDCOLOR_TYPE_CUSTOM) {
			guint r,g,b;
			color.getRGBColor(r,g,b);
			psout << (gdouble) r / 255.0 << ' ';
			psout << (gdouble) g / 255.0 << ' ';
			psout << (gdouble) b / 255.0;
		} else if (color.colorType == GDCOLOR_TYPE_BLACK) {
			psout << "0 0 0";
		} else if (color.colorType == GDCOLOR_TYPE_WHITE) {
			psout << "1.0 1.0 1.0";
		}
		if (include_cmd) {
			psout << " sc\n";
		}
	}
	return 1;
}

gint GDStyle::writePSStrokeStyle(ostream& psout, GDDocument *doc, gboolean include_cmd) {
	gdouble pfx = doc->printFactor.x;
	gdouble pfy = doc->printFactor.y;
	// dashes
	if (lineType == GDSTYLE_LINETYPE_DASHED || ! include_cmd) {
		psout << " [";
		for (int i = 0; i < 6; i++) {
			if (dashes[i] < 0) {
				i = 6;
			} else {
				psout << " " << dashes[i];
			}
		}
		psout << " ] 0";
		if (include_cmd) psout << " sd";
	}
	// lineCaps
	int caps = 0;
	if (lineCaps == GDSTYLE_CAPS_ROUND) {
		caps = 1;
	} else if (lineCaps == GDSTYLE_CAPS_SQUARE) {
		caps = 2;
	}
	psout << " " << caps;
	if (include_cmd) psout << " slc";
	// lineJoins
	int joins = 0;
	if (lineJoins == GDSTYLE_JOINS_ROUND) {
		joins = 1;
	} else if (lineJoins == GDSTYLE_JOINS_BEVEL) {
		joins = 2;
	}
	psout << " " << joins;
	if (include_cmd) psout << " slj";
	// lineMiterLimit
	if (lineJoins == GDSTYLE_JOINS_MITER || ! include_cmd) {
		psout << " " << (lineMiterLimit > 0 ? lineMiterLimit : 10); // if no miter limit defined take PS default of 10
		if (include_cmd) psout << " sml";
	}
	// strokeWidth
	psout << " " << strokeWidth * (pfx + pfy) / 2;
	if (include_cmd) psout << " slw";
	return 1;
}

void GDStyle::writeStrokePS(ostream& psout, GDDocument *doc) {
	if (isGuide) return;
	if (writeColorPS(psout)) {
		psout << "gs";
		writePSStrokeStyle(psout, doc, true);
		psout << " stroke gr\n";
	}
}

void GDStyle::writeGradientPS(ostream& psout, GDDocument *doc) {
	psout << "gradient_" << (long) gradient;
	psout << " ";
	// calc the matrix
	TransformationMatrix tmp = tMatrix;
	TransformationMatrix t;
	// translate to doc origin
	t.reset();
	t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
	tmp *= t;
	// change the coordinate system to postscript
	// flip on the x-axis
	t.reset();
	t.m[1][1] = -1;
	tmp *= t;
	// move by page height
	t.reset();
	t.setTranslation(0, doc->pageHeight);
	tmp *= t;
	tmp.writePS(psout);
}

void GDStyle::writePatternPS(ostream& psout, GDDocument *doc) {
	psout << "pattern_" << (long) pattern;
	psout << " ";
	// calc the matrix
	TransformationMatrix tmp;
	tmp.m[1][1] = -1;
	tmp *= tMatrix;
	TransformationMatrix t;
	// translate to doc origin
	t.reset();
	t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
	tmp *= t;
	// change the coordinate system to postscript
	// flip on the x-axis
	t.reset();
	t.m[1][1] = -1;
	tmp *= t;
	// move by page height
	t.reset();
	t.setTranslation(0, doc->pageHeight);
	tmp *= t;
	tmp.writePS(psout);
}

void GDStyle::writeFillPS(ostream& psout, GDDocument *doc) {
	if (isGuide) return;
	if (drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		if (writeColorPS(psout)) {
			psout << "gs fill gr\n";
		}
	} else if (drawType == GDSTYLE_DRAWTYPE_GRADIENT) {
		psout << "gs\n";
		writeGradientPS(psout, doc);
		psout << " mpt spt fill gr\n";
	} else if (drawType == GDSTYLE_DRAWTYPE_PATTERN) {
		psout << "gs\n";
		writePatternPS(psout, doc);
		psout << " mpt spt fill gr\n";
	}
}

void GDStyle::writeStrokeXML(ostream& xmlout, GDDocument *doc) {
	writeColorXML(xmlout, "stroke");
	writeXMLStrokeStyle(xmlout);
	if (isGuide) {
		xmlout << " gestalter:guide=\"yes\"";
	}
}

void GDStyle::writeFillXML(ostream& xmlout, GDDocument *doc) {
	if (drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		writeColorXML(xmlout, "fill");
	} else if (drawType == GDSTYLE_DRAWTYPE_GRADIENT) {
		xmlout << " fill=\"url(#gradientinc_" << (long) this << ")\"";
	} else if (drawType == GDSTYLE_DRAWTYPE_PATTERN) {
		xmlout << " fill=\"url(#patterninc_" << (long) this << ")\"";
	}
}

void GDStyle::writeFillPreparationXML(ostream& xmlout, GDDocument *doc) {
	if (drawType == GDSTYLE_DRAWTYPE_GRADIENT) {
		TransformationMatrix tmp = tMatrix;
		TransformationMatrix t;
		t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
		tmp *= t;
		xmlout << "<defs>\n";
		xmlout << "<" << (gradient->type == GRADIENT_TYPE_LINEAR ? "linearGradient" : "radialGradient");
		xmlout << " id=\"gradientinc_" << (long) this << "\" gradientUnits=\"userSpaceOnUse\"";
		xmlout << " xlink:href=\"#gradient_" << (long) gradient << "\" gradientTransform=\"";
		tmp.writeXML(xmlout);
		xmlout << "\"/>\n";
		xmlout << "</defs>\n";
		
	} else if (drawType == GDSTYLE_DRAWTYPE_PATTERN) {
		TransformationMatrix tmp = tMatrix;
		TransformationMatrix t;
		t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
		tmp *= t;
		xmlout << "<defs>\n";
		xmlout << "<pattern id=\"patterninc_" << (long) this << "\" patternUnits=\"userSpaceOnUse\"";
		xmlout << " xlink:href=\"#pattern_" << (long) pattern << "\" patternTransform=\"";
		tmp.writeXML(xmlout);
		xmlout << "\"/>\n";
		xmlout << "</defs>\n";
		
	}
}

gint GDStyle::writeXMLStrokeStyle(ostream& xmlout) {
	xmlout << " stroke-width=\"" << strokeWidth << "\"";
	xmlout << " stroke-linejoin=\"";
	if (lineJoins == GDSTYLE_JOINS_BEVEL) {
		xmlout << "bevel";
	} else if (lineJoins == GDSTYLE_JOINS_ROUND) {
		xmlout << "round";
	} else {
		xmlout << "miter";
	}
	xmlout << "\"";
	xmlout << " stroke-linecap=\"";
	if (lineCaps == GDSTYLE_CAPS_SQUARE) {
		xmlout << "square";
	} else if (lineCaps == GDSTYLE_CAPS_ROUND) {
		xmlout << "round";
	} else {
		xmlout << "butt";
	}
	xmlout << "\"";
	if (lineJoins == GDSTYLE_JOINS_MITER) {
		xmlout << " stroke-miterlimit=\"" << (lineMiterLimit >= 0 ? lineMiterLimit : 10) << "\"";
	}
	if (lineType == GDSTYLE_LINETYPE_DASHED) {
		xmlout << " stroke-dasharray=\"";
		for (int i = 0; i < 6; i++) {
			if (dashes[i] < 0) {
				i = 6;
			} else {
				xmlout << (i > 0 ? "," : "") << dashes[i];
			}
		}
		xmlout << "\"";
	}
	return 1;
}

void GDStyle::setStrokeOptionsFromXML(XMLOBJECT *xmlObj, const char **attrs) {
	gchar *sWidth = xmlObj->getAttributeValue(attrs, "stroke-width");
	if (sWidth) {
		strokeWidth = g_strtod(sWidth, NULL);
	}
	gchar *mLimit = xmlObj->getAttributeValue(attrs, "stroke-miterlimit");
	if (mLimit) {
		lineMiterLimit = g_strtod(mLimit, NULL);
	}
	gchar *lCap = xmlObj->getAttributeValue(attrs, "stroke-linecap");
	if (lCap) {
		if (! strncasecmp(lCap, "square", 6)) {
			lineCaps = GDSTYLE_CAPS_SQUARE;
		} else if (! strncasecmp(lCap, "round", 5)) {
			lineCaps = GDSTYLE_CAPS_ROUND;
		} else {
			lineCaps = GDSTYLE_CAPS_BUTT;
		}
	}
	gchar *lJoin = xmlObj->getAttributeValue(attrs, "stroke-linejoin");
	if (lJoin) {
		if (! strncasecmp(lJoin, "bevel", 6)) {
			lineJoins = GDSTYLE_JOINS_BEVEL;
		} else if (! strncasecmp(lJoin, "round", 5)) {
			lineJoins = GDSTYLE_JOINS_ROUND;
		} else {
			lineJoins = GDSTYLE_JOINS_MITER;
		}
	}
	gchar *lDashes = xmlObj->getAttributeValue(attrs, "stroke-dasharray");
	int i = 0;
	while (i < 6 && lDashes) {
		dashes[i++] = g_strtod(lDashes, NULL);
		lDashes = strstr(lDashes, ",");
		if (lDashes) lDashes++; // jump over the comma
	}
	if (i > 0) {
		lineType = GDSTYLE_LINETYPE_DASHED;
	}
}

void GDStyle::setStrokeStyleFromXML(XMLOBJECT *xmlObj, const char **attrs, GDDocument *doc) {
	gchar *guide = xmlObj->getAttributeValue(attrs, "gestalter:guide");
	if (guide && ! strncasecmp(guide, "yes", 3)) {
		isGuide = true;
	}
	gchar *sColorString = xmlObj->getAttributeValue(attrs, "stroke");
	if (sColorString) {
		setColorFromString(sColorString);
	} else {
		drawType = GDSTYLE_DRAWTYPE_TRANSPARENT;
	}
	setStrokeOptionsFromXML(xmlObj, attrs);
}

void GDStyle::setFillStyleFromXML(XMLOBJECT *xmlObj, const char **attrs, GDDocument *doc) {
	gchar *guide = xmlObj->getAttributeValue(attrs, "gestalter:guide");
	if (guide && ! strncasecmp(guide, "yes", 3)) {
		isGuide = true;
	}
	gchar *fColorString = xmlObj->getAttributeValue(attrs, "fill");
	if (fColorString) {
		if (! strncmp(fColorString, "url(#", 5)) {
			string fillString = fColorString;
			string refString = fillString.substr(5, fillString.size() - 6);
			ResourceMap::iterator it = doc->tmpResourceMap.find(refString);
			if (it != doc->tmpResourceMap.end()) {
				ResourceTransform gradTrans = (*it).second;
				TransformationMatrix tmp;
				tmp.setTranslation(doc->nullPoint.x, doc->nullPoint.y);
				tMatrix = gradTrans.matrix;
				tMatrix *= tmp;
				if (gradTrans.resource->getType() == GDRESOURCE_TYPE_GRADIENT) {
					gradient = (Gradient *) gradTrans.resource;
					drawType = GDSTYLE_DRAWTYPE_GRADIENT;
				} else if (gradTrans.resource->getType() == GDRESOURCE_TYPE_PATTERN) {
					pattern = (Pattern *) gradTrans.resource;
					drawType = GDSTYLE_DRAWTYPE_PATTERN;
				}
			} else {
				g_printerr("GDStyle::setFillStyleFromXML: gradient reference \"%s\" (%s) not found\n", refString.c_str(), fColorString);
			}
		} else {
			setColorFromString(fColorString);
		}
	} else {
		drawType = GDSTYLE_DRAWTYPE_TRANSPARENT;
	}
}

void GDStyle::strokeAVP(ArtVpath *avp, guchar *buffer, guint rowstride, guint width, guint height, gdouble ox, gdouble oy, gdouble scaleFactor) {
	if (isGuide) {
		ArtVpathDash dash;
		gdouble guideDashes[2];
		guideDashes[0] = 1;
		guideDashes[1] = 4;
		dash.dash = guideDashes;
		dash.n_dash = 2;
		dash.offset = 0;
		ArtVpath *oldvp = avp;
		avp = art_vpath_dash(avp, &dash);
		art_free(oldvp);
		ArtSVP *asvp = art_svp_vpath_stroke(avp, 
								ART_PATH_STROKE_JOIN_BEVEL,
								ART_PATH_STROKE_CAP_BUTT,
								1, 
								4, 0.5);
		art_free(avp);
		art_rgb_svp_alpha(asvp, 0, 0,
					width,
					height,
					0xFF,  // color black, fully opaque
					buffer,
					rowstride,
					NULL);
		art_svp_free(asvp);
	} else {
		if (lineType == GDSTYLE_LINETYPE_DASHED) {
			ArtVpathDash dash;
			gdouble currDashes[6];
			dash.dash = currDashes;
			for (int i = 0; i < 6; i++) {
				if (dashes[i] < 0) {
					dash.n_dash = i;
					i = 6;
				} else {
					dash.dash[i] = dashes[i] * scaleFactor;
				}
			}
			dash.offset = 0;
			if (dash.n_dash) {
				ArtVpath *oldvp = avp;
				avp = art_vpath_dash(avp, &dash);
				art_free(oldvp);
			}
		}
		ArtPathStrokeJoinType joins;
		ArtPathStrokeCapType caps;
		if (lineCaps == GDSTYLE_CAPS_ROUND) {
			caps = ART_PATH_STROKE_CAP_ROUND;
		} else if (lineCaps == GDSTYLE_CAPS_SQUARE) {
			caps = ART_PATH_STROKE_CAP_SQUARE;
		} else {
			caps = ART_PATH_STROKE_CAP_BUTT;
		}
		if (lineJoins == GDSTYLE_JOINS_BEVEL) {
			joins = ART_PATH_STROKE_JOIN_BEVEL;
		} else if (lineJoins == GDSTYLE_JOINS_ROUND) {
			joins = ART_PATH_STROKE_JOIN_ROUND;
		} else {
			joins = ART_PATH_STROKE_JOIN_MITER;
		}
		gdouble swidth = strokeWidth * scaleFactor;
		if (swidth < 0.1) swidth = 0.1;
		ArtSVP *asvp = art_svp_vpath_stroke(avp, 
								joins,
								caps,
								swidth, 
								lineMiterLimit, 0.5);
		art_free(avp);
		guint r,g,b;
		getRGBColor(r,g,b);
		//g_printerr("BezierPath::draw: stroke color= %d %d %d\n", r, g, b);
		guint32 color = (r << 24) | (g << 16) | (b << 8) | 0xFF;
		art_rgb_svp_alpha(asvp, 0, 0,
					width,
					height,
					color,
					buffer,
					rowstride,
					NULL);
		art_svp_free(asvp);
	}
}
void GDStyle::fillSVP(ArtSVP *svp, guchar *buffer, guint rowstride, guint width, guint height, gdouble ox, gdouble oy, gdouble scaleFactor) {
	//g_printerr("GDStyle::fillSVP drawtype = %d, gradient = %lX, this=%lX\n", drawType, (long) gradient, (long)this);
	if (isGuide) return; // no filling for guides
	if (drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		guint r,g,b;
		getRGBColor(r,g,b);
		//g_printerr("BezierPath::draw: fill color= %d %d %d\n", r, g, b);
		guint32 color = (r << 24) | (g << 16) | (b << 8) | 0xFF;
		art_rgb_svp_alpha(svp, 0, 0, width, height, color, buffer, rowstride, NULL);
	} else if (drawType == GDSTYLE_DRAWTYPE_GRADIENT) {
		if (gradient) {
			//g_printerr("tMatrix:\n");
			//tMatrix.printMatrix();
			TransformationMatrix reverse = tMatrix;
			TransformationMatrix tmp;
			tmp.setTranslation(-ox, -oy);
			reverse *= tmp;
			tmp.reset();
			tmp.setScale(scaleFactor, scaleFactor);
			reverse *= tmp;
			//g_printerr("tMatrix (incl. display):\n");
			//reverse.printMatrix();
			reverse = reverse.inverse();
			//g_printerr("reverse:\n");
			//reverse.printMatrix();
			vector<ColorPoint> backup;
			if (gradient->type == GRADIENT_TYPE_RADIAL) {
				backup = gradient->colorPoints;
				for (vector<ColorPoint>::iterator it = gradient->colorPoints.begin(); it != gradient->colorPoints.end(); it++) {
					double val = (*it).pos;
					(*it).pos = val * val;
				}
			}
			gdstyle_svp_custom_fill_data fill_data = { gradient, NULL, NULL, 0, buffer, rowstride, width, height, reverse, &gdstyle_gradient_render_point };
			art_svp_render_aa(svp, 0, 0, width, height, &gdstyle_svp_custom_fill_cb, (void *) &fill_data);
			if (gradient->type == GRADIENT_TYPE_RADIAL) {
				gradient->colorPoints = backup;
			}
		}
	} else if (drawType == GDSTYLE_DRAWTYPE_PATTERN) {
		if (pattern) {
			TransformationMatrix reverse = tMatrix;
			position p1, p2;
			p1.x = 0; p1.y = 0;
			p2.x = 1.0; p2.y = 0;
			tMatrix.transform(p1);
			tMatrix.transform(p2);
			p2.x -= p1.x;
			p2.y -= p1.y;
			gdouble displayFactor = sqrt(p2.x * p2.x + p2.y * p2.y);
			displayFactor *= scaleFactor; // including the scaleFactor the pattern should be exactly the size of the actual screen device
			TransformationMatrix tmp;
			tmp.setScale(scaleFactor, scaleFactor);
			tmp.setTranslation(-ox, -oy);
			reverse *= tmp;
			reverse = reverse.inverse();
			OffscreenBuffer osBuffer;
			osBuffer.scaleFactor = displayFactor;
			osBuffer.origin.x = 0;
			osBuffer.origin.y = 0;
			guint w = (guint) (osBuffer.scaleFactor * pattern->width);
			guint h = (guint) (osBuffer.scaleFactor * pattern->height);
			osBuffer.allocBuffer(w * 3, w, h);
			osBuffer.whiteBackground();
			BoundingBox bbox = pattern->content->getBoundingBox();
			pattern->content->draw(osBuffer, bbox);
			OffscreenBuffer maskBuffer;
			maskBuffer.scaleFactor = displayFactor;
			maskBuffer.origin.x = 0;
			maskBuffer.origin.y = 0;
			maskBuffer.allocBuffer(w * 3, w, h);
			maskBuffer.whiteBackground();
			pattern->content->drawBlack(maskBuffer, bbox);
			//g_printerr("GDStyle::fillSVP df=%f sf=%f w=%d h=%d\n", displayFactor, scaleFactor, w, h);
			//g_printerr("GDStyle::fillSVP:reverse:\n");
			//reverse.printMatrix();
			gdstyle_svp_custom_fill_data fill_data = { NULL, &osBuffer, &maskBuffer, displayFactor, buffer, rowstride, width, height, reverse, &gdstyle_pattern_render_point };
			art_svp_render_aa(svp, 0, 0, width, height, &gdstyle_svp_custom_fill_cb, (void *) &fill_data);
		}
	}
}

void GDStyle::transformByMatrix(TransformationMatrix& m) {
	//g_printerr("GDStyle::transformByMatrix: this=%lX\n", (long) this);
	//tMatrix.printMatrix();
	tMatrix *= m;
	position p1, p2;
	p1.x = 0; p1.y = 0;
	p2.x = strokeWidth;
	p2.y = 0;
	m.transform(p1);
	m.transform(p2);
	p2.x -= p1.x;
	p2.y -= p1.y;
	//g_printerr("GDStyle::transformByMatrix strokeWidth=%f\n", strokeWidth);
	strokeWidth = sqrt(p2.x * p2.x + p2.y * p2.y);
	//m.printMatrix();
	//g_printerr("GDStyle::transformByMatrix strokeWidth=%f\n", strokeWidth);
}

void GDStyle::gatherResource(GDResourceList& collection, gint type) {
	//g_printerr("GDStyle::gatherResource this=%lX, type=%X, drawType=%d, gradient=%lX, pattern=%lX\n", (long) this, type, drawType, (long) gradient, (long) pattern);
	if (type & GDRESOURCE_TYPE_GRADIENT) {
		if (drawType == GDSTYLE_DRAWTYPE_GRADIENT && gradient) {
			collection.insert(gradient);
		}
	}
	if (type & GDRESOURCE_TYPE_PATTERN) {
		if (drawType == GDSTYLE_DRAWTYPE_PATTERN && pattern) {
			collection.insert(pattern);
		}
	}
	if (type & GDRESOURCE_TYPE_COLOR) {
		if (drawType == GDSTYLE_DRAWTYPE_OPAQUE && color.colorType == GDCOLOR_TYPE_CUSTOM) {
			collection.insert(color.color.custom);
		}
	}
}

void GDStyle::calcMorphValues(GDStyle *source, GDStyle *target, double n) {
	*this = *source;
	strokeWidth = source->strokeWidth + (target->strokeWidth - source->strokeWidth) * n;
	guint sr, sg, sb, tr, tg, tb;
	source->color.getRGBColor(sr, sg, sb);
	target->color.getRGBColor(tr, tg, tb);
	color.setRGB((int) (sr + (tr - sr) * n), (int) (sg + (tg - sg) * n), (int) (sb + (tb - sb) * n) );
	opacity = source->opacity + (target->opacity - source->opacity) * n;
	lineMiterLimit = source->lineMiterLimit + (target->lineMiterLimit - source->lineMiterLimit) * n;
	isGuide = false;
	
}


GDStyle::~GDStyle() {
}
