/* $Id: GradientListSelectionDialog.cc,v 1.2 2004/03/06 15:00:19 pklaus Exp $
 *
 * GradientListSelectionDialog.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "config.h"
#include "GradientListSelectionDialog.hh"
#include "GDDocument.hh"
#include "GradientListOrgDialog.hh"

GradientListSelectionDialog::GradientListSelectionDialog(GDDocument *document, Gradient *active) : ListSelectionDialog() {
	//g_printerr("GradientListSelectionDialog::GradientListSelectionDialog called: doc: %lX, activeGrad: %lX\n", document, active);
	currentGradient = active;
	doc = document;
	updateList(document);
}

void GradientListSelectionDialog::onClickColumn(gint column) {
}

void GradientListSelectionDialog::onSelectRow(gint row, gint column, GdkEvent *event) {
	//g_printerr("GradientListSelectionDialog::onSelectRow: row=%d, col=%d\n", row, column);
	if (!event || event->type == GDK_BUTTON_PRESS) {
		gint count = row;
		GradientMap::iterator it = doc->gradients.begin();
		for (; it != doc->gradients.end() && count; it++, count--);
		currentGradient = (*it).second;
	} else if (event->type == GDK_2BUTTON_PRESS) {
		okButton->clicked();
	}
}

void GradientListSelectionDialog::onResize(GtkAllocation *all) {
}

void GradientListSelectionDialog::updateList(GDDocument *document) {
	//g_printerr("GradientListSelectionDialog::updateList doc:%lX\n", document);
	Gradient *cur = currentGradient;
	objectList->clear();
	gint count = 0;
	gint selected = -1;
	if (! document) return;
	for (GradientMap::iterator it = document->gradients.begin(); it != document->gradients.end(); it++) {
		char *text[] = { "", "" };
		//g_printerr("GradientListSelectionDialog::updateList row:%d grad:%lX currg:%lX\n", count, (long) (*it).second, (long) cur);
		if ((*it).second == cur) {
			selected = count;
		}
		count++;
		int row = objectList->append(text);
		Gdk_Bitmap previewMask;
		Gdk_Pixmap previewIcon;
		GradientListOrgDialog::createGradientPreview((*it).second, previewIcon, previewMask, document);
		objectList->cell(row, 0).set_pixmap(previewIcon, previewMask);
		objectList->cell(row, 1).set_text((*it).first);
	}
	if (selected >= 0) {
		objectList->row(selected).select();
	}
	//g_printerr("GradientListSelectionDialog::updateList reached end selected=%d\n", selected);
}

GradientListSelectionDialog::~GradientListSelectionDialog() {
}
