/* $Id: HWrapBox.cc,v 1.3 2002/12/29 16:43:39 pklaus Exp $
 *
 * HWrapBox.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "HWrapBox.hh"

HWrapBox::HWrapBox(gboolean homogeneous) {
	max_child_width = 0;
	max_child_height = 0;
	this->homogeneous = homogeneous ? true : false;
}

inline void HWrapBox::get_child_requisition(Gtk::Widget& child, GtkRequisition& child_requisition) {
	if (homogeneous) {
		child_requisition.width = max_child_width;
		child_requisition.height = max_child_height;
		//g_printerr("HWrapBox::get_child_requisition (homo) called (%d,%d)\n", child_requisition.width, child_requisition.height);
	} else {
    	gtk_widget_get_child_requisition (GTK_WIDGET(child.gtkobj()), &child_requisition);
		//g_printerr("HWrapBox::get_child_requisition (child) called (%d,%d)\n", child_requisition.width, child_requisition.height);
	}
	
}

gfloat HWrapBox::get_layout_size(guint max_width, guint& width_inc) {
	//g_printerr("HWrapBox::get_layout_size called (max %d)\n", max_width);
	WrapBoxChild *child;
	gboolean last_row_filled = true;
	guint left_over = 0;
	guint total_height = 0;
	
	width_inc = max_child_width + 1;
	//g_printerr("max_child_width: %d, width_inc: %d\n", max_child_width, width_inc);
	guint n_rows = 0;
	for (child = children; child; child = child->next) {
		if (! child->widget->is_visible()) continue;
		
		GtkRequisition child_requisition;
		get_child_requisition(*child->widget, child_requisition);
		if (!last_row_filled) {
			width_inc = MIN(width_inc, child_requisition.width - left_over);
			//g_printerr("crw: %d, width_inc: %d\n", child_requisition.width, width_inc);
		}
		guint row_width = child_requisition.width;
		guint row_height = child_requisition.height;
		WrapBoxChild *row_child;
		guint n = 1;
		for (row_child = child->next; row_child && n < child_limit; row_child = row_child->next) {
			if (row_child->widget->is_visible()) {
				get_child_requisition(*row_child->widget, child_requisition);
				if (row_width + hspacing + child_requisition.width > max_width) {
					break;
				}
				row_width += hspacing + child_requisition.width;
				row_height = MAX(row_height, child_requisition.height);
				n++;
			}
			child = row_child;
		}
		last_row_filled = n >= child_limit;
		left_over = last_row_filled ? 0 : max_width - (row_width + hspacing);
		total_height += (n_rows ? vspacing : 0) + row_height;
		n_rows++;
	}
	if (width_inc > max_child_width) {
		width_inc = 0;
	}
	//g_printerr("total_height: %f, width_inc: %d\n", total_height, width_inc);
	return MAX(total_height, 1);
}

void HWrapBox::size_request_impl(GtkRequisition *requisition) {
	//g_printerr("HWrapBox::size_request_impl called (%d,%d)\n", requisition->width, requisition->height);
	WrapBoxChild *child;
	
	requisition->width = 0;
	requisition->height = 0;
	max_child_width = 0;
	max_child_height = 0;
	
	// size_request all children
	for (child = children; child; child = child->next) {
		if (child->widget->is_visible()) {
			GtkRequisition child_requisition;
			gtk_widget_size_request(child->widget->gtkobj(), &child_requisition);
			max_child_width = MAX(max_child_width, child_requisition.width);
			max_child_height = MAX(max_child_height, child_requisition.height);
		}
	}
	// figure all possible layouts
	gfloat ratio_dist = 32768;
	gfloat layout_width = max_child_width;
	guint row_inc = 0;
	do {
		layout_width += row_inc;
		gfloat layout_height = get_layout_size((gint) layout_width, row_inc);
		gfloat ratio = layout_width / layout_height; // <h2v-skip>
		gfloat dist = MAX(ratio, aspect_ratio) - MIN(ratio, aspect_ratio);
		if (dist < ratio_dist) {
			ratio_dist = dist;
			requisition->width = (gint16) layout_width;
			requisition->height = (gint16)layout_height;
		}
	} while (row_inc);
	requisition->width += GTK_CONTAINER(gtkobj())->border_width * 2; // <h2v-skip>
	requisition->height += GTK_CONTAINER(gtkobj())->border_width * 2 + 5; // <h2v-skip>
	//g_printerr("result size_request: %d, %d\n", requisition->width, requisition->height);
}

GSList *HWrapBox::rlist_line_children(WrapBoxChild **child_p, GtkAllocation& area,
						guint& max_child_size, gboolean& expand_line) {
	//g_printerr("HWrapBox::rlist_line_children called\n");
	GSList *slist = NULL;
	guint row_width = area.width;
	WrapBoxChild *child = *child_p;
	max_child_size = 0;
	expand_line = FALSE;
	
	while (child && ! child->widget->is_visible()) {
		*child_p = child->next;
		child = *child_p;
	}
	
	if (child) {
		GtkRequisition child_requisition;
		guint n = 1;
		
		get_child_requisition(*child->widget, child_requisition);
		guint width = 0;
		width += child_requisition.width;
		max_child_size = MAX(max_child_size, child_requisition.height);
		expand_line |= child->vexpand;
		slist = g_slist_prepend(slist, child);
		*child_p = child->next;
		child = *child_p;
		while (child && n < child_limit) {
			if (child->widget->is_visible()) {
				get_child_requisition(*child->widget, child_requisition);
				if (width + hspacing + child_requisition.width > row_width ||
							child->forced_break) {
					break;
				}
				width += hspacing + child_requisition.width;
				max_child_size = MAX(max_child_size, child_requisition.height);
				expand_line |= child->vexpand;
				slist = g_slist_prepend(slist, child);
				n++;
			}
			*child_p = child->next;
			child = *child_p;
		}
	}
	return slist;
}

void HWrapBox::layout_row(GtkAllocation& area, GSList *children,
					guint children_per_line, gboolean vexpand) {
	//g_printerr("HWrapBox::layout_row called\n");
	//g_printerr("area: (%d,%d %d,%d), cpl: %d, vexp: %d\n", area.x, area.y, area.width, area.height, children_per_line, vexpand);
	GSList *slist;
	guint n_children = 0;
	guint n_expand_children = 0;
	guint have_expand_children = 0;
	guint total_width = 0;
	gfloat extra;
	GtkAllocation child_allocation;
	
	for (slist = children; slist; slist = slist->next) {
		WrapBoxChild * child = (WrapBoxChild *)slist->data;
		GtkRequisition child_requisition;
		
		n_children++;
		if (child->hexpand) {
			n_expand_children++;
		}
		get_child_requisition(*child->widget, child_requisition);
		total_width += child_requisition.width;
	}
	gfloat width = MAX(1, area.width - (n_children - 1) * hspacing);
	//g_printerr("width: %f, n_children: %d, hspacing: %d\n", width, n_children, hspacing);
	if (width > total_width) {
		extra = width - total_width;
	} else {
		extra = 0;
	}
	have_expand_children = n_expand_children && extra;
	gfloat x = area.x;
	if (homogeneous) {
		width = MAX(1, area.width - (children_per_line - 1) * hspacing);
		width /= ((gdouble) children_per_line);
		extra = 0;
		//g_printerr("homogeneous: width: %f\n", width);
	} else if (have_expand_children && justify != GTK_JUSTIFY_FILL) {
		width = extra;
		extra /= ((gdouble) n_expand_children);
	} else {
		if (justify == GTK_JUSTIFY_FILL) {
			width = extra;
			have_expand_children = TRUE;
			n_expand_children = n_children;
			extra /= ((gdouble) n_expand_children);
		} else if (justify == GTK_JUSTIFY_CENTER) {
			x += extra / 2;
			width = 0;
			extra = 0;
		} else if (justify == GTK_JUSTIFY_LEFT) {
			width = 0;
			extra = 0;
		} else if (justify == GTK_JUSTIFY_RIGHT) {
			x += extra;
			width = 0;
			extra = 0;
		}
	}
	n_children = 0;
	for (slist = children; slist; slist = slist->next) {
		//g_printerr("Loop begin: width=%f x=%f\n", width, x);
		WrapBoxChild *child = (WrapBoxChild *)slist->data;
		child_allocation.x = (gint16) x;
		child_allocation.y = area.y;
		if (homogeneous) {
			child_allocation.height = area.height;
			child_allocation.width = (gint16) width;
			//x += child_allocation.width + hspacing; // was bug
		} else {
			GtkRequisition child_requisition;
			get_child_requisition(*child->widget, child_requisition);
			if (child_requisition.height >= area.height) {
				child_allocation.height = area.height;
			} else {
				child_allocation.height = child_requisition.height;
				if (line_justify == GTK_JUSTIFY_FILL || child->vfill) {
					child_allocation.height = area.height;
				} else if (child->vexpand || line_justify == GTK_JUSTIFY_CENTER) {
					child_allocation.y += (area.height - child_requisition.height) / 2;
				} else if (line_justify == GTK_JUSTIFY_BOTTOM) {
					child_allocation.y += area.height - child_requisition.height;
				}
			}
			if (have_expand_children) {
				child_allocation.width = child_requisition.width;
				if (child->hexpand || justify == GTK_JUSTIFY_FILL) {
					n_expand_children--;
					guint space = (guint) (extra * n_expand_children);
					space = (guint) width - space;
					width -= space;
					if (child->hfill) {
						child_allocation.width += space;
					} else {
						child_allocation.x += space / 2;
						x += space;
					}
				}
			} else {
				child_allocation.x += (gint16) (n_children * extra);
				child_allocation.width = MIN(child_requisition.width, area.width - child_allocation.x + area.x);
			}
		}
		x += child_allocation.width + hspacing;
		gtk_widget_size_allocate(child->widget->gtkobj(), &child_allocation);
		n_children++;
		//g_printerr("ca: (%d,%d %d,%d) x: %f n: %d\n", 
		//			child_allocation.x, child_allocation.y, child_allocation.width,
		//			child_allocation.height, x, n_children);
	}
}


struct Line {
	GSList *children;
	guint16	min_size;
	guint	expand : 1;
	Line	*next;
};

void HWrapBox::layout_rows(GtkAllocation& area) {
	//g_printerr("HWrapBox::layout_rows called (%d,%d %d,%d)\n", area.x, area.y, area.width, area.height);
	WrapBoxChild *next_child = children;
	guint total_height = 0;
	guint n_expand_lines = 0;
	guint n_lines = 0;
	guint min_height;
	GSList *slist;
	Line *line_list = NULL;
	gboolean vexpand;
	gfloat shrink_height;
	
	slist = rlist_line_children(&next_child, area, min_height, vexpand);
	slist = g_slist_reverse(slist);
	guint children_per_line = g_slist_length(slist);
	while (slist) {
		Line *line = g_new(Line, 1);
		line->children = slist;
		line->min_size = min_height;
		line->expand = vexpand;
		if (vexpand) {
			n_expand_lines++;
		}
		line->next = line_list;
		line_list = line;
		n_lines++;
		slist = rlist_line_children(&next_child, area, min_height, vexpand);
		slist = g_slist_reverse(slist);
	}
	if (total_height > area.height) {
		shrink_height = total_height - area.height;
	} else {
		shrink_height = 0;
	}
	if (1) {  // reverse lines and shrink
		Line *prev = NULL;
		Line *last = NULL;
		gfloat n_shrink_lines = n_lines;
		while (line_list) {
			Line *tmp = line_list->next;
			if (shrink_height) {
				Line *line = line_list;
				guint shrink_fract = (guint) (shrink_height / n_shrink_lines + 0.5);
				if (line->min_size > shrink_fract) {
					shrink_height -= shrink_fract;
					line->min_size -= shrink_fract;
				} else {
					shrink_height -= line->min_size - 1;
					line->min_size = 1;
				}
			}
			n_shrink_lines--;
			last = line_list;
			line_list->next = prev;
			prev = line_list;
			line_list = tmp;
		}
		line_list = last;
	}
	if (n_lines) {
		Line *line;
		gfloat extra = 0;
		
		gfloat height = area.height;
		height = MAX(n_lines, height - (n_lines - 1) * vspacing);
		if (homogeneous) {
			height /= ((gdouble) n_lines);
		} else if (n_expand_lines) {
			height = MAX(0, height - total_height);
			extra = height / ((gdouble) n_expand_lines);
		} else {
			height = 0;
		}
		gfloat y = area.y;
		line = line_list;
		while (line) {
			GtkAllocation row_allocation;
			Line *next_line = line->next;
			row_allocation.x = area.x;
			row_allocation.width = area.width;
			if (homogeneous) {
				row_allocation.height = (guint16) height;
			} else {
				row_allocation.height = line->min_size;
				if (line->expand) {
					row_allocation.height += (guint16) extra;
				}
			}
			row_allocation.y = (gint16) y;
			y += row_allocation.height + vspacing;
			layout_row(row_allocation, line->children, children_per_line, line->expand);
			g_slist_free(line->children);
			g_free(line);
			line = next_line;
		}
	}
	//g_printerr("HWrapBox::layout_rows finished (%d,%d %d,%d)\n", area.x, area.y, area.width, area.height);
}

void HWrapBox::size_allocate_impl(GtkAllocation* allocation) {
	//g_printerr("HWrapBox::size_allocate_impl called (%d,%d %d,%d)\n",
	//			allocation->x, allocation->y, allocation->width, allocation->height);
	GtkAllocation area;
	guint border = GTK_CONTAINER(gtkobj())->border_width;
	GTK_WIDGET(gtkobj())->allocation = *allocation;
	area.x = allocation->x + border;
	area.y = allocation->y + border;
	area.width = MAX(1, (gint) allocation->width - border * 2);
	area.height = MAX(1, (gint) allocation->height - border * 2);
	layout_rows(area);
	//g_printerr("HWrapBox::size_allocate_impl finished (%d,%d %d,%d)\n",
	//			allocation->x, allocation->y, allocation->width, allocation->height);
}

	
