/* $Id: LayerRename.cc,v 1.3 2004/03/26 13:12:01 pklaus Exp $
 *
 * LayerRename.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gestalter.h"
#include <stdio.h>
#include "LayerRename.hh"
#include "Layer.hh"
#include "../pixmaps/layer_color.xpm"
#include <libgnome/libgnome.h> // for _() macro

LayerRename::LayerRename() {
	csd = NULL;
}

void LayerRename::changeColor() {
	csd = manage(new class Gtk::ColorSelectionDialog(_("Select Color")));
	gdouble col[4] = { (currentColor >> 24 & 0xFF) / 255, (currentColor >> 16 & 0xFF) / 255, (currentColor >> 8 & 0xFF) / 255, 1 };
	csd->get_colorsel()->set_color(col);
	csd->set_modal(1);
	csd->get_ok_button()->clicked.connect(SigC::slot(this, &LayerRename::newCurrentColor));
	csd->get_cancel_button()->clicked.connect(csd->destroy.slot());
	csd->show();
}

void LayerRename::setLayerInfo(Layer *which, guint number) {
	currentLayer = which;
	layerNumber = number;
	setIconColor(which->getSelectionColor());
	layerNameEntry->set_text(which->getName());
}

guint LayerRename::getLayerNumber() {
	return layerNumber;
}
	
Layer *LayerRename::getLayer() {
	return currentLayer;
}

guint LayerRename::getLayerColor() {
	return currentColor;
}

string LayerRename::getLayerNameString() {
	return layerNameEntry->get_text();
}

void LayerRename::setIconColor(guint color) {
	char colorbuffer[20];
	currentColor = color;
	snprintf(colorbuffer, sizeof(colorbuffer), "+\tc #%02X%02X%02X", 
				(color >> 24) & 0xFF, 
				(color >> 16) & 0xFF,
				(color >> 8) & 0xFF);
	layer_color_xpm[3] = colorbuffer;
	Gdk_Pixmap myColorPixmap;
	Gdk_Bitmap myColorMask;
	Gdk_Color transparentColor;
	myColorPixmap.create_from_xpm_d(get_window(), myColorMask, transparentColor, layer_color_xpm);
	colorPixmap->set(myColorPixmap, myColorMask);
}

void LayerRename::newCurrentColor() {
	if (csd) {
		gdouble col[4];
		csd->get_colorsel()->get_color(col);
		setIconColor(((guint) (col[0] * 255)) << 24 |
						((guint) (col[1] * 255)) << 16 |
						((guint) (col[2] * 255)) <<  8 |  0xFF);
		csd->destroy();
		csd = NULL;
	}
}
