/* $Id: Layers.cc,v 1.16 2004/03/06 15:00:19 pklaus Exp $
 *
 * Layers.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "gestalter.h"
#include <stdio.h>
#include "Layers.hh"
#include "GDApplication.hh"
#include "GDDocument.hh"
#include "Layer.hh"
#include "LayerRename.hh"
#include "../pixmaps/layer_viewable.xpm"
#include "../pixmaps/layer_edit.xpm"
#include "../pixmaps/layer_emptybox.xpm"
#include "../pixmaps/layer_color.xpm"
#include "../pixmaps/layer_selection.xpm"
#include <gtk--/clist.h>

#define	LAYER_LIST_COLUMN_VIEWABLE	0
#define	LAYER_LIST_COLUMN_EDITABLE	1
#define	LAYER_LIST_COLUMN_NAME		2
#define	LAYER_LIST_COLUMN_SELECTION	3

Layers::Layers(GDApplication *myApp) {
	application = myApp;
	layerList->column(LAYER_LIST_COLUMN_VIEWABLE).set_widget(viewableIcon);
	layerList->column(LAYER_LIST_COLUMN_VIEWABLE).set_width(24);
	layerList->column(LAYER_LIST_COLUMN_VIEWABLE).set_resizable(0);
	layerList->column(LAYER_LIST_COLUMN_EDITABLE).set_widget(editableIcon);
	layerList->column(LAYER_LIST_COLUMN_EDITABLE).set_width(18);
	layerList->column(LAYER_LIST_COLUMN_EDITABLE).set_resizable(0);
	layerList->column(LAYER_LIST_COLUMN_NAME).set_widget(layerTitleEmptyBox);
	layerList->column(LAYER_LIST_COLUMN_NAME).set_width(90);
	layerList->column(LAYER_LIST_COLUMN_NAME).set_resizable(0);
	layerList->column(LAYER_LIST_COLUMN_SELECTION).set_widget(layerCurrentSelectionEmptyBox);
	layerList->column(LAYER_LIST_COLUMN_SELECTION).set_width(7);
	layerList->column(LAYER_LIST_COLUMN_SELECTION).set_resizable(0);
	doc = NULL;
	layerRenameDialog = NULL;
	layerCount = 0;
	// prepare list icons
	Gdk_Color transparentColor;
	viewablePixmap.create_from_xpm_d(application->toolWindow->get_window(), viewableMask, transparentColor, layer_viewable_xpm);
	editablePixmap.create_from_xpm_d(application->toolWindow->get_window(), editableMask, transparentColor, layer_edit_xpm);
	emptyPixmap.create_from_xpm_d(application->toolWindow->get_window(), emptyMask, transparentColor, layer_emptybox_xpm);
}

void Layers::onClickLayer(gint column) {
	//g_printerr("Layers::onClickLayer clicked on column %d\n", column);	
	Gdk_Pixmap pixmap;
	Gdk_Bitmap mask;
	if (column == LAYER_LIST_COLUMN_VIEWABLE) {
		Layer *lay = doc->layers;
		guint crow = 0;
		while (lay) {
			lay->viewable = true;
			pixmap = viewablePixmap;
			mask = viewableMask;
			layerList->cell(crow, LAYER_LIST_COLUMN_VIEWABLE).set_pixmap(pixmap, mask);
			crow++;
			lay = lay->next;
		}
		doc->redraw();
	} else if (column == LAYER_LIST_COLUMN_EDITABLE) {
		Layer *lay = doc->layers;
		guint crow = 0;
		while (lay) {
			lay->editable = true;
			pixmap = editablePixmap;
			mask = editableMask;
			layerList->cell(crow, LAYER_LIST_COLUMN_EDITABLE).set_pixmap(pixmap, mask);
			crow++;
			lay = lay->next;
		}
	}
}

void Layers::onSelectLayer(gint row, gint column, GdkEvent *event) {
	Gdk_Pixmap pixmap = emptyPixmap;
	Gdk_Bitmap mask = emptyMask;
	if (! event) return;
	if (! doc) return;
	//g_printerr("Layers::onSelectLayer row: %d, column: %d, eventtype: %d\n", row, column, event->type);
	if (column == LAYER_LIST_COLUMN_NAME) {
		if (event->type == GDK_BUTTON_PRESS) {
			// select this rows layer
			selectedRow = row;
			doc->activeLayer = (*(doc->layers))[row];
			checkButtons(row);
		} else if (event->type == GDK_2BUTTON_PRESS) {
			//g_printerr("Layer::onSelectLayer: layer %d was double clicked\n", row);
			if (! layerRenameDialog) {
				layerRenameDialog = new LayerRename();
				layerRenameDialog->okButton->clicked.connect(SigC::slot(this, &Layers::renameLayer));
				layerRenameDialog->cancelButton->clicked.connect(SigC::slot(this, &Layers::cancelLayerRenameDialog));
				layerRenameDialog->delete_event.connect(SigC::slot(this, &Layers::deleteLayerRenameDialog));
				layerRenameDialog->set_modal(true);
			}
			layerRenameDialog->show();
			layerRenameDialog->setLayerInfo((*(doc->layers))[row], row);
		}
	} else if (column == LAYER_LIST_COLUMN_VIEWABLE) {
		if (event->type == GDK_BUTTON_PRESS) {
			Layer *layer = (*(doc->layers))[row];
			layer->viewable = ! layer->viewable;
			if (layer->viewable) {
				pixmap = viewablePixmap;
				mask = viewableMask;
			}
			layerList->cell(row, LAYER_LIST_COLUMN_VIEWABLE).set_pixmap(pixmap, mask);
			doc->redraw();
		} else if (event->type == GDK_2BUTTON_PRESS) {
			Layer *lay = doc->layers;
			gint crow = 0;
			while (lay) {
				if (crow == row) {
					lay->viewable = true;
					pixmap = viewablePixmap;
					mask = viewableMask;
				} else {
					lay->viewable = false;
					pixmap = emptyPixmap;
					mask = emptyMask;
				}
				layerList->cell(crow, LAYER_LIST_COLUMN_VIEWABLE).set_pixmap(pixmap, mask);
				crow++;
				lay = lay->next;
			}
			doc->redraw();
		}
	} else if (column == LAYER_LIST_COLUMN_EDITABLE) {
		if (event->type == GDK_BUTTON_PRESS) {
			Layer *layer = (*(doc->layers))[row];
			layer->editable = ! layer->editable;
			if (layer->editable) {
				pixmap = editablePixmap;
				mask = editableMask;
			}
			layerList->cell(row, LAYER_LIST_COLUMN_EDITABLE).set_pixmap(pixmap, mask);
		} else if (event->type == GDK_2BUTTON_PRESS) {
			Layer *lay = doc->layers;
			gint crow = 0;
			while (lay) {
				if (crow == row) {
					lay->editable = true;
					pixmap = editablePixmap;
					mask = editableMask;
				} else {
					lay->editable = false;
					pixmap = emptyPixmap;
					mask = emptyMask;
				}
				layerList->cell(crow, LAYER_LIST_COLUMN_EDITABLE).set_pixmap(pixmap, mask);
				crow++;
				lay = lay->next;
			}
		}
	}
	layerList->row(selectedRow).select();
}

gint Layers::deleteLayerRenameDialog(GdkEventAny *ev) {
	cancelLayerRenameDialog();
}

void Layers::cancelLayerRenameDialog() {
	layerRenameDialog->hide();
	
}

void Layers::renameLayer() {
	layerRenameDialog->hide();
	Layer *layer = layerRenameDialog->getLayer();
	guint layerNum = layerRenameDialog->getLayerNumber();
	layer->setName(layerRenameDialog->getLayerNameString());
	layer->setSelectionColor(layerRenameDialog->getLayerColor());
	updateLayers(doc);
}

void Layers::onNewLayer() {
	if (doc) {
		Layer *newLayer = new Layer(doc);
		doc->addLayer(newLayer);
	}
}

void Layers::onLayerUp() {
	if (doc) {
		doc->layers = doc->layers->moveUp(doc->activeLayer);
		updateLayers(doc);
		doc->redraw();
	}
}

void Layers::onLayerDown() {
	if (doc) {
		doc->layers = doc->layers->moveDown(doc->activeLayer);
		updateLayers(doc);
		doc->redraw();
	}
}

void Layers::onLayerDuplicate() {
	if (doc && doc->activeLayer) {
		Layer *cLayer = doc->activeLayer->clone(true);
		doc->addLayer(cLayer);
	}
}

void Layers::onLayerDelete() {
	if (doc) {
		Layer *curLayer = doc->activeLayer;
		if (curLayer) {
			if (curLayer->next) {
				doc->activeLayer = curLayer->next;
			} else {
				doc->activeLayer = curLayer->prev;
			}
			doc->removeLayer(curLayer);
			delete curLayer;
		}
		doc->redraw();
	}
}

gint Layers::hideLayers(GdkEventAny *ev) {
	application->hideLayers();
	return 1;
}


void Layers::resizeLayerList(GtkAllocation *all) { 
	//g_printerr("Layers::resizeLayerList called w:%d h:%d\n", all->width, all->height);
	layerList->column(LAYER_LIST_COLUMN_NAME).set_width(all->width - 82);
}

void Layers::closeLayersDialog() {   
	application->hideLayers();
}

void Layers::setDocument(GDDocument *document) {
	//g_printerr("Layers::setDocument: document=%lX\n", document);
	if (doc != document) {
		doc = document;
		if (selConnection.connected()) {
			selConnection.disconnect();
		}
		if (layerConnection.connected()) {
			layerConnection.disconnect();
		}
		updateLayers(document);
		if (document) {
			selConnection = doc->selection->changed.connect(SigC::slot(this, &Layers::updateSelection));
			layerConnection = doc->layersChanged.connect(SigC::slot(this, &Layers::updateLayers));
		}
	}
}

void Layers::checkButtons(guint selected) {
	if (selected == 0) {
		buttonLayerUp->set_sensitive(false);
	} else {
		buttonLayerUp->set_sensitive(true);
	}
	if (selected >= layerCount - 1) {
		buttonLayerDown->set_sensitive(false);
	} else {
		buttonLayerDown->set_sensitive(true);
	}
}
	

void Layers::updateLayers(GDDocument *document) {
	Gdk_Pixmap pixmap;
	Gdk_Bitmap mask;
	char colorbuffer[20];
	layerList->clear();
	layerCount = 0;
	//g_printerr("Pixmaptest: %d, %d\n", pixmap.width(), pixmap.height());
	if (! document) return;
	for (Layer *layer = document->layers; layer; layer = layer->next) {
		char *text[] = { "", "", "", "" };
		layerCount++;
		int row = layerList->append(text);
		//g_printerr("Layers::updateLayers: inserting layer %lX (%s) (row: %d)\n", layer, layer->getName().c_str(), row);
		if (layer->viewable) {
			//g_printerr("Layers::updateLayers: layer %s is viewable\n", layer->getName().c_str());
			pixmap = viewablePixmap;
			mask = viewableMask;
			//viewablePixmap->get(pixmap, mask);
		} else {
			pixmap = emptyPixmap;
			mask = emptyMask;
			//layerCurrentSelectionEmptyBox->get(pixmap, mask);
		}
		layerList->cell(row, LAYER_LIST_COLUMN_VIEWABLE).set_pixmap(pixmap, mask);
		if (layer->editable) {
			//editablePixmap->get(pixmap, mask);
			pixmap = editablePixmap;
			mask = editableMask;
		} else {
			pixmap = emptyPixmap;
			mask = emptyMask;
			//layerCurrentSelectionEmptyBox->get(pixmap, mask);
		}
		layerList->cell(row, LAYER_LIST_COLUMN_EDITABLE).set_pixmap(pixmap, mask);
		layerList->cell(row, LAYER_LIST_COLUMN_NAME).set_text(layer->getName());
		guint color = layer->getSelectionColor();
		snprintf(colorbuffer, sizeof(colorbuffer), "+\tc #%02X%02X%02X", 
					(color >> 24) & 0xFF, 
					(color >> 16) & 0xFF,
					(color >> 8) & 0xFF);
		//g_printerr("color: %lX, string: %s\n", color, colorbuffer);
		layer_color_xpm[3] = colorbuffer;
		Gdk_Pixmap colorPixmap;
		Gdk_Bitmap colorMask;
		Gdk_Color transparentColor;
		colorPixmap.create_from_xpm_d(document->application->toolWindow->get_window(), colorMask, transparentColor, layer_color_xpm);
		layerList->cell(row, LAYER_LIST_COLUMN_NAME).set_pixmap(colorPixmap, colorMask);
		if (layer == doc->activeLayer) {
			layerList->row(row).select();
			selectedRow = row;
		}
	}
	updateSelection(document);
}

void Layers::updateSelection(GDDocument *document) {
	//g_printerr("Layers::updateSelection called\n");
	if (! document) return;
	SelectionItemList selList;
	document->getSelection(selList, SELECTIONITEM_MODE_ORIG_ONLY);
	if (layerList->selection().size()) {
		checkButtons((*(layerList->selection().begin())).get_row_num());
	}
	char colorbuffer[20];
	Layer *lay = document->layers;
	guint crow = 0;
	while (lay) {
		//g_printerr("checking layer %s\n", lay->getName().c_str());
		if (lay->containsSelectedItems) {
			guint color = lay->getSelectionColor();
			snprintf(colorbuffer, sizeof(colorbuffer), ".\tc #%02X%02X%02X", 
						(color >> 24) & 0xFF, 
						(color >> 16) & 0xFF,
						(color >> 8) & 0xFF);
			layer_selection_xpm[2] = colorbuffer;
			Gdk_Pixmap selPixmap;
			Gdk_Bitmap selMask;
			Gdk_Color transparentColor;
			selPixmap.create_from_xpm_d(document->application->toolWindow->get_window(), selMask, transparentColor, layer_selection_xpm);
			layerList->cell(crow, LAYER_LIST_COLUMN_SELECTION).set_pixmap(selPixmap, selMask);
		} else {
			layerList->cell(crow, LAYER_LIST_COLUMN_SELECTION).set_pixmap(emptyPixmap, emptyMask);
		}
		crow++;
		lay = lay->next;
	}
	
}

Layers::~Layers() {
	if (layerRenameDialog) {
		delete layerRenameDialog;
	}
}
