/* $Id: LibXML.cc,v 1.4 2004/03/20 00:01:37 pklaus Exp $
 *
 * LibXML.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "LibXML.hh"
#include <string.h>
#include <stdio.h>
#include <glib.h>

#define BUFFER_SIZE		1024

static xmlSAXHandler gestXMLHandler = {
	NULL,					// internalSubset
	NULL,					// isStandalone
	NULL,					// hasInternalSubset
	NULL,					// hasExternalSubset
	NULL,					// resolveEntity
	NULL,					// getEntity
	NULL,					// entityDecl
	NULL,					// notationDecl
	NULL,					// attributeDecl
	NULL,					// elementDecl
	NULL,					// unparsedEntityDecl
	NULL,					// setDocumentLocator
	NULL,					// startDocument
	NULL,					// endDocument
	wrapbegintag,			// startElement
	wrapendtag,				// endElement
	NULL,					// reference
	wraptextdata,			// characters
	NULL,					// ignorableWhitespace
	NULL,					// processingInstruction
	NULL,					// comment
	NULL,					// warning
	NULL,					// error
	NULL					// fatalError
};


void wrapbegintag(void *mydata, const xmlChar *name, const xmlChar **attrs) {
	if (mydata) {
		parser_data *my_data = (parser_data *) mydata;
		my_data->xmlObject->handleBeginTag((const char *) name, (const char **)attrs);
	}
}

void wrapendtag(void *mydata, const xmlChar *name) {
	if (mydata) {
		parser_data *my_data = (parser_data *) mydata;
		my_data->xmlObject->handleEndTag((const char *)name);
	}
}

void wraptextdata(void *mydata, const xmlChar *text, int len) {
	if (mydata) {
		parser_data *my_data = (parser_data *) mydata;
		my_data->xmlObject->handleText((const char *) text, len);
	}
}

LibXML::LibXML() {
	errstring = NULL;
}

int LibXML::parse(LibXMLObject *rootParser, string filePath) {
	parser_data my_data;
	
	my_data.xmlObject = rootParser;
	rp = rootParser;
	rp->setParser(&my_data);
	rp->init();
	xmlSAXUserParseFile(&gestXMLHandler, &my_data, filePath.c_str());
	return 0;
}


LibXMLObject::LibXMLObject(LibXMLObject *parent, const char *tag, parser_data *parser, const char **attrs) {
	this->parent = parent;
	this->parser = parser;
	if (tag) {
		tagname = tag;
	}
}

LibXMLObject::LibXMLObject() {
	parent = NULL;
	tagname = "";
}

void LibXMLObject::init() {
	//g_printerr("LibXMLObject::init called\n");
	parser->xmlObject = this;
}

void LibXMLObject::handleBeginTag(const char *name, const char **attrs) {
	//g_printerr("LibXMLObject::handleBeginTag: Unhandled tag: %s\n", name);
}

void LibXMLObject::handleEndTag(const char *name) {
	//g_printerr("LibXMLObject::handleEndTag: name %s\n", name);
	if (! tagname.compare(name)) {
		parser->xmlObject = parent;
	}
	delete this;
}

void LibXMLObject::handleText(const char *text, int len) {
}

char *LibXMLObject::getAttributeValue(const char **attrs, const char *key) {
	if (!attrs) return NULL;
	const char **tmp = attrs;
	while (*tmp) {
		if (! strcmp(*tmp, key)) {
			tmp++;
			return (char *) *tmp;
		}
		tmp++;
		tmp++;
	}
	return NULL;
}

int LibXMLObject::getAttributeValue(const char **attrs, const char *key, int& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval) {
		value = atoi(attval);
		return 0;
	}
	return -1;
}

int LibXMLObject::getAttributeValue(const char **attrs, const char *key, float& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval && sscanf(attval, "%f", &value) == 1) {
		return 0;
	}
	return -1;
}

int LibXMLObject::getAttributeValue(const char **attrs, const char *key, double& value) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval && sscanf(attval, "%lf", &value) == 1) {
		return 0;
	}
	return -1;
}

int LibXMLObject::getAttributeValue(const char **attrs, const char *key, char *value, int len) {
	const char *attval = getAttributeValue(attrs, key);
	if (attval) {
		strncpy(value, attval, len);
		return 0;
	}
	return -1;
}
