/* $Id: PageOriginTool.cc,v 1.6 2004/03/20 00:01:37 pklaus Exp $
 *
 * PageOriginTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../pixmaps/page_origin.xpm"
#include "../cursors/page_origin.xbm"
#include "../cursors/page_origin_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "PageOriginTool.hh"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"
#include "GDApplication.hh"

PageOriginTool::PageOriginTool() {
	pixmapData = page_origin_xpm;
	toolName = "PageOriginTool";
	moving = false;
	CursorFactory::createCursorFromData(pageOriginCursor,
					page_origin_bits, page_origin_mask_bits,
					page_origin_width, page_origin_height,
					page_origin_x_hot, page_origin_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
}

void PageOriginTool::activate() {
	Tool::activate();
}

void PageOriginTool::deactivate() {
	Tool::deactivate();
}

gint PageOriginTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;
	downButton = ev->button;

	if (downButton == 1) {
		doc = docwin->doc;
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		oldx = (gint) ev->x;
		oldy = (gint) ev->y;
		drawable.draw_rectangle(doc->context, 0, oldx, oldy,
				(int) (doc->pageWidth * docwin->osBuffer.scaleFactor),
				(int) (doc->pageHeight * docwin->osBuffer.scaleFactor));
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint PageOriginTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK && ! (ev->state & GDK_MOD1_MASK)) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		drawable.draw_rectangle(doc->context, 0, oldx, oldy,
				(int) (doc->pageWidth * docwin->osBuffer.scaleFactor),
				(int) (doc->pageHeight * docwin->osBuffer.scaleFactor));
		oldx = (gint) ev->x;
		oldy = (gint) ev->y;
		drawable.draw_rectangle(doc->context, 0, oldx, oldy,
				(int) (doc->pageWidth * docwin->osBuffer.scaleFactor),
				(int) (doc->pageHeight * docwin->osBuffer.scaleFactor));
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint PageOriginTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		doc->context.set_function(GDK_SET);
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		doc->nullPoint.x = sx;
		doc->nullPoint.y = sy;
		doc->redraw();
		moving = false;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void PageOriginTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
	} else {
		docwin->DocWinDrawingArea->get_window().set_cursor(pageOriginCursor);
	}
}
