/* $Id: PathMingler.hh,v 1.3 2004/05/28 20:41:38 pklaus Exp $
 *
 * PathMingler.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _PATHMINGLER_HH
#  define _PATHMINGLER_HH

#include "GObject.hh"
#include <set>

#define PATHMINGLER_FOREIGN_COVER_PREVIOUS	1
#define PATHMINGLER_FOREIGN_COVER_NEXT		2

class GDDocument;

class ISObject {
public:
	GObject *object;
	BezierPathVector *paths;
};

typedef vector<ISObject *> ISObjectVector;

class ObjectIntersection {
public:
	ObjectIntersection() { objectA = NULL; objectB = NULL; newNodeA = NULL; newNodeB = NULL; addNodeA = NULL; addNodeB = NULL;
		dupNewNodeA = NULL; dupNewNodeB = NULL; dupAddNodeA = NULL; dupAddNodeB = NULL; statusA = 0; statusB = 0;
		coverage = -1; coverObject = NULL; };
	GObject *objectA;
	GObject *objectB;
	GDStyle fStyleA;
	GDStyle sStyleA;
	GDStyle fStyleB;
	GDStyle sStyleB;
	BezierNode *newNodeA;
	BezierNode *addNodeA;
	BezierNode *dupNewNodeA;
	BezierNode *dupAddNodeA;
	BezierNode *newNodeB;
	BezierNode *addNodeB;
	BezierNode *dupNewNodeB;
	BezierNode *dupAddNodeB;
	Intersection intersection;
	guint statusA;
	guint statusB;
	gint coverage;
	GObject *coverObject;
};

typedef vector<ObjectIntersection *> ObjectIntersectionVector;

struct ltgobject {
	bool operator()(const GObject *a, const GObject *b) const {
		return a < b;
	}
};

typedef map<GObject *, gint, ltgobject> GObjectDepthMap;

struct ltpath {
	bool operator()(const BezierPath *a, const BezierPath *b) {
		return a < b;
	}
};

typedef map<BezierPath *, BezierPath *, ltpath>	BezierPathMap;

struct ltnode {
	bool operator()(const BezierNode *a, const BezierNode *b) {
		return a < b;
	}
};

typedef map<BezierNode *, BezierNode *, ltnode> BezierNodeMap;

class SplitNode {
public:
	BezierNode *baseNode;
	BezierNode **newNode;
	gdouble splitRatio;
	
	SplitNode() { baseNode = NULL; newNode = NULL; splitRatio = 0; };
	SplitNode(const SplitNode& n) { baseNode = n.baseNode; newNode = n.newNode; splitRatio = n.splitRatio; };
	SplitNode(SplitNode& n) { baseNode = n.baseNode; newNode = n.newNode; splitRatio = n.splitRatio; };
	SplitNode operator=(const SplitNode& n) { baseNode = n.baseNode; newNode = n.newNode; splitRatio = n.splitRatio; return *this; };

	bool operator<(const SplitNode& n) const { return (baseNode == n.baseNode) ? splitRatio < n.splitRatio : baseNode < n.baseNode; };
	bool operator>(const SplitNode& n) const { return (baseNode == n.baseNode) ? splitRatio > n.splitRatio : baseNode > n.baseNode; };
	bool operator<=(const SplitNode& n) const { return (baseNode == n.baseNode) ? splitRatio <= n.splitRatio : baseNode <= n.baseNode; };
	bool operator>=(const SplitNode& n) const { return (baseNode == n.baseNode) ? splitRatio >= n.splitRatio : baseNode >= n.baseNode; };
};

class NodePair {
public:
	BezierNode *nodeA;
	BezierNode *nodeB;
	GDStyle fStyle;
	GDStyle sStyle;
	gint styleDepth;

	NodePair() { nodeA = NULL; nodeB = NULL; styleDepth = 0; };
	NodePair(BezierNode *nA, BezierNode *nB, GDStyle& f, GDStyle& s, gint sd) { nodeA = nA; nodeB = nB; fStyle = f; sStyle = s; styleDepth = sd; };
	NodePair(BezierNode *nA, BezierNode *nB, GObject *obj, gint sd) { nodeA = nA; nodeB = nB; fStyle = *(obj->getFillStyle()); sStyle = *(obj->getStrokeStyle()); styleDepth = sd; };
	NodePair(const NodePair& n) { nodeA = n.nodeA; nodeB = n.nodeB; fStyle = n.fStyle; sStyle = n.sStyle; styleDepth = n.styleDepth; };
	NodePair(NodePair& n) { nodeA = n.nodeA; nodeB = n.nodeB; fStyle = n.fStyle; sStyle = n.sStyle; styleDepth = n.styleDepth; };
	NodePair operator=(const NodePair& n) { nodeA = n.nodeA; nodeB = n.nodeB; fStyle = n.fStyle; sStyle = n.sStyle; styleDepth = n.styleDepth; return *this; };

	bool operator<(const NodePair& n) const { return styleDepth < n.styleDepth; };
	bool operator>(const NodePair& n) const { return styleDepth > n.styleDepth; };
	bool operator<=(const NodePair& n) const { return styleDepth <= n.styleDepth; };
	bool operator>=(const NodePair& n) const { return styleDepth >= n.styleDepth; };
};

typedef vector<NodePair> NodePairVector;



class PathMingler {
protected:	
	static ISObjectVector getIntersectionObjects(GDDocument *doc, GObjectDepthMap& gobjects);
	static ObjectIntersectionVector calcIntersections(ISObjectVector& objectPaths);
	static void createIntersectionNodes(ObjectIntersectionVector& result);
	//static ObjectIntersectionVector getTopObjectISVector(GDDocument *doc, GObjectDepthMap& gobjects, GObject **topObj);
	static ObjectIntersectionVector getISVector(GDDocument *doc, GObjectDepthMap& gobjects, GObject **topObj = NULL, GObject **bottomObj = NULL);
	static void getMiddlePoint(BezierNode *startNode, position& middle);
	static int getCoverage(ObjectIntersection *objIntersection, GObjectDepthMap& gobjects, GObject **coverObject);
	static BezierNode *getDupNode(BezierNode *node, BezierPathMap& duplicates);
	static void calcCoverage(ObjectIntersectionVector& oisVector, GObjectDepthMap& godMap, gboolean doCoverObject = TRUE);
	static void splitPaths(ObjectIntersectionVector& oisVector);
	static void connectNodePairs(NodePairVector& nodePairs);
	static void orderPaths(ObjectIntersection *ois, BezierNode **coverPathA, BezierNode **openPathA, BezierNode **coverPathB, BezierNode **openPathB);
	static void duplicateInnerPaths(ObjectIntersectionVector& oisVector, BezierPathMap& dupPathMap, GDDocument *doc);
	static BezierNode *splitNode(BezierNode *node);
	static int getStatus(BezierNode *newNode, GObject *oppObject);

public:
	static void pathMingleUnite(GDDocument *doc);
	static void pathMingleIntersect(GDDocument *doc);
	static void pathMingleExclude(GDDocument *doc);
	static void pathMingleMinusFront(GDDocument *doc);
	static void pathMingleMinusBack(GDDocument *doc);
	static void pathMingleTrim(GDDocument *doc);
	static void pathMingleDivide(GDDocument *doc);
	static void pathMingleOutline(GDDocument *doc);
	static void pathMingleMerge(GDDocument *doc);
	static void pathMingleCrop(GDDocument *doc);


};

#endif
