/* $Id: PatternListOrgDialog.cc,v 1.5 2004/07/21 06:31:20 pklaus Exp $
 *
 * PatternListOrgDialog.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "PatternListOrgDialog.hh"
#include "GDApplication.hh"
#include "GDDocument.hh"
#include "Pattern.hh"
#include "EditPatternDialog.hh"
#include "ClipGroup.hh"
#include "BezierNode.hh"
#include "BezierPath.hh"
#include <string>
#include <gtk--.h>
#include <math.h>

#include <libgnome/libgnome.h>  /* for _() macro */

PatternListOrgDialog::PatternListOrgDialog(GDApplication *app) : ListOrgDialog(app) {
	inEdit = NULL;
	currentPattern = NULL;
	untitledNumber = 1;
	editPatternDlg = NULL;
}

void PatternListOrgDialog::onClickColumn(gint column) {
	// do nothing, shouldn't be possible
}

void PatternListOrgDialog::onSelectRow(gint row, gint column, GdkEvent *event) {
	//g_printerr("GradientListOrgDialog::onSelectRow: row=%d, col=%d\n", row, column);
	if (!event) return;
	if (event->type == GDK_BUTTON_PRESS) {
		gint count = row;
		PatternMap::iterator it = doc->patterns.begin();
		for (; it != doc->patterns.end() && count; it++, count--);
		currentPattern = (*it).second;
	} else if (event->type == GDK_2BUTTON_PRESS) {
		if (currentPattern) {
			inEdit = new Pattern(*currentPattern);
			showEditDialog(inEdit);
		}
	}
}

void PatternListOrgDialog::onResize(GtkAllocation *all) {
}

gint PatternListOrgDialog::hideWindow(GdkEventAny *ev) {
	//g_printerr("PatternListOrgDialog::hideWindow\n");
	application->hidePatterns();
	return 1;
}

void PatternListOrgDialog::newClicked() {
	currentPattern = NULL;
	ClipGroup *group = new ClipGroup(doc);
	SelectionItemList *selection = new SelectionItemList();
	doc->getSelection(*selection, SELECTIONITEM_MODE_ORIG_ONLY);
	G_List_Iterator<SelectionItem *> it = selection->begin();
	SelectionItem *item;
	GObject *lastobject = NULL;
	while ((item = *it++)) {
		SelectableItem *sitem = item->orig;
		GObject *object = sitem->getGObject();
		if (lastobject != object) {
			if (lastobject) {
				GObject * cloneObj = (GObject *) lastobject->clone();
				cloneObj->prev = NULL;
				cloneObj->next = NULL;
				//g_printerr("PatternListOrgDialog::newClicked adding object %s (%lX) to clipgroup\n", cloneObj->identify(), (long) cloneObj);
				group->addPart(cloneObj);
			}
			lastobject = object;
		}
	}
	if (lastobject) {
		GObject * cloneObj = (GObject *) lastobject->clone();
		cloneObj->prev = NULL;
		cloneObj->next = NULL;
		group->addClipPart(cloneObj);
	}
	delete selection;
	if (! lastobject) {
		delete group;
		application->showError(_("You've chosen to create a new pattern but there is no selection available.\nPlease select the pattern objects together with the bounding rectangle and hit 'new' again"));
		//g_printerr("PatternListOrgDialog::newClicked no selection\n");
		return;
	}
	gdouble w = 0,h = 0, ox = G_MAXDOUBLE, oy = G_MAXDOUBLE;
	gboolean ok = TRUE;
	if (strcmp(lastobject->identify(), "BezierPath")) {
		ok = FALSE;
	} else {
		BezierPath *limitpath = (BezierPath *) lastobject;
		BezierNode *corners[4];
		corners[0] = limitpath->nodes;
		ok = FALSE;
		if (corners[0]) {
			corners[1] = corners[0]->next;
			if (corners[1] && corners[1] != corners[0]) {
				corners[2] = corners[1]->next;
				if (corners[2] && corners[2] != corners[0]) {
					corners[3] = corners[2]->next;
					if (corners[3] && corners[3]->next == corners[0]) {
						ok = TRUE;
					}
				}
			}
		}
		if (ok) {
			// check if each node is a corner (not a smooth point)
			for (int i = 0; i < 4 && ok; i++) {
				if (corners[i]->onCurve.x != corners[i]->prevHandle.x || 
					corners[i]->onCurve.x != corners[i]->nextHandle.x ||
					corners[i]->onCurve.y != corners[i]->prevHandle.y || 
					corners[i]->onCurve.y != corners[i]->nextHandle.y) {
					ok = FALSE;
					//g_printerr("PatternListOrgDialog::newClicked node %d is not a corner\n", i);
				}
			}
		}
		if (ok) {
			// check for rectangle
			for (int i = 0; i < 4 && ok; i++) {
				//g_printerr("PatternListOrgDialog::newClicked check corner %d: %f,%f\n", i, corners[i]->onCurve.x, corners[i]->onCurve.y);
				if (corners[i]->onCurve.x < ox) ox = corners[i]->onCurve.x;
				if (corners[i]->onCurve.y < oy) oy = corners[i]->onCurve.y;
				if (fabs(corners[(i+1)%4]->onCurve.x - corners[i]->onCurve.x) > w) {
					w = fabs(corners[(i+1)%4]->onCurve.x - corners[i]->onCurve.x);
				}
				if (fabs(corners[(i+1)%4]->onCurve.y - corners[i]->onCurve.y) > h) {
					h = fabs(corners[(i+1)%4]->onCurve.y - corners[i]->onCurve.y);
				}
				if (corners[i]->onCurve.x != corners[(i+1)%4]->onCurve.x && 
					corners[i]->onCurve.y != corners[(i+1)%4]->onCurve.y) {
					ok = FALSE;
					//g_printerr("PatternListOrgDialog::newClicked corner %d is not rectangular\n", i);
				}
			}
		}
	}
	if (! ok) {
		delete group;
		application->showError(_("The top object in the selection is not a rectangle.\nYou must specify a bounding rectangle and this must be the top object in the selection."));
		return;
	}
	// transform the new clipgroup to fit a 1.0 square at the origin
	TransformationMatrix m;
	m.setTranslation(-ox, -oy);
	TransformationMatrix t;
	gdouble fact = w;
	if (h > w) fact = h;
	gdouble revfact = 1 / fact;
	t.setScale(revfact, revfact);
	m *= t;
	group->transformByMatrix(m);
	w *= revfact;
	h *= revfact;
	//g_printerr("PatternListOrgDialog::newClicked ox=%f oy=%f fac=%f\n", ox, oy, fact);
	inEdit = new Pattern(group, w, h, fact);
	gchar number[20];
	snprintf(number, 18, "%d", untitledNumber++);
	string tmpname = string(_("untitled-"));
	tmpname.append(number);
	inEdit->setName(tmpname);
	showEditDialog(inEdit);
}

void PatternListOrgDialog::deleteClicked() {
	if (currentPattern && doc) {
		// check if this gradient is used in the document
		GDResourceList resList;
		doc->gatherResource(resList, GDRESOURCE_TYPE_PATTERN);
		if (resList.count(currentPattern) < 1) {
			doc->patterns.erase(currentPattern->getName());
			delete currentPattern;
			updateList(doc);
		}
	}
}

void PatternListOrgDialog::editClicked() {
	if (currentPattern) {
		inEdit = new Pattern(*currentPattern);
		showEditDialog(inEdit);
	}
}

void PatternListOrgDialog::closeDialog() {
	//g_printerr("GradientListOrgDialog::closeDialog\n");
	application->hidePatterns();
}

void PatternListOrgDialog::showEditDialog(Pattern *toEdit) {
	editPatternDlg = new EditPatternDialog(toEdit);
	editPatternDlg->getOKButton()->clicked.connect(SigC::slot(this, &PatternListOrgDialog::saveChangedPattern));
	editPatternDlg->getCancelButton()->clicked.connect(SigC::slot(this, &PatternListOrgDialog::cancelEditPatternDialog));
	editPatternDlg->delete_event.connect(SigC::slot(this, &PatternListOrgDialog::deleteEditPatternDialog));
	editPatternDlg->set_modal(true);
}

gint PatternListOrgDialog::deleteEditPatternDialog(GdkEventAny *ev) {
	cancelEditPatternDialog();
}

void PatternListOrgDialog::cancelEditPatternDialog() {
	editPatternDlg->hide();
	
}

void PatternListOrgDialog::saveChangedPattern() {
	if (doc) {
		editPatternDlg->hide();
		if (currentPattern) {
			doc->patterns.erase(currentPattern->getName());
			*currentPattern = *inEdit;
			doc->patterns[currentPattern->getName()] = currentPattern;
			delete inEdit;
		} else {
			doc->patterns[inEdit->getName()] = inEdit;
			currentPattern = inEdit;
			inEdit = NULL;
		}
		updateList(doc);
	}
}



void PatternListOrgDialog::setDocument(GDDocument *document) {
	ListOrgDialog::setDocument(document);
	updateList(document);
}

void PatternListOrgDialog::updateList(GDDocument *document) {
	if (! document) document = doc;
	if (! document) return;
	objectList->clear();
	gint count = 0;
	gint selected = -1;
	if (! document) return;
	for (PatternMap::iterator it = document->patterns.begin(); it != document->patterns.end(); it++) {
		char *text[] = { "", "" };
		if ((*it).second == currentPattern) {
			selected = count;
		}
		count++;
		int row = objectList->append(text);
		gchar masktempl[65];
		Gdk_Bitmap previewMask;
		Gdk_Pixmap previewIcon;
		Gdk_GC context = document->application->toolWindow->get_style()->get_black_gc();
		createPatternPreview((*it).second, previewIcon, previewMask, context, document);
		objectList->cell(row, 0).set_pixmap(previewIcon, previewMask);
		objectList->cell(row, 1).set_text((*it).first);
	}
	if (selected >= 0) {
		objectList->row(selected).select();
	}
}

Gdk_Pixmap PatternListOrgDialog::createPatternPreview(Pattern *pattern, Gdk_Pixmap& previewPixmap, Gdk_Bitmap& previewMask, Gdk_GC& context, GDDocument *document) {
	//g_printerr("PatternListOrgDialog::createPatternPreview pattern=%lX\n", (long) pattern);
	OffscreenBuffer osBuffer;
	guint aw = 32;
	guint ah = 16;
	guint w = aw;
	guint h = ah;
	osBuffer.scaleFactor = (gdouble) w / pattern->width < (gdouble) h / pattern->height ? (gdouble) w / pattern->width : (gdouble) h / pattern->height;
	w = (guint) (osBuffer.scaleFactor * pattern->width);
	h = (guint) (osBuffer.scaleFactor * pattern->height);
	osBuffer.allocBuffer(w * 3, w, h);
	osBuffer.whiteBackground();
	osBuffer.origin.x = 0;
	osBuffer.origin.y = 0;
	BoundingBox bbox = pattern->content->getBoundingBox();
	pattern->content->draw(osBuffer, bbox);
	gint x = (aw - w) / 2;
	gint y = (ah - h) / 2;
	guchar *pixbuf = osBuffer.pixbuffer;
	previewPixmap.create(document->application->toolWindow->get_window(), aw, ah);
	//g_printerr("PatternListOrgDialog::createPatternPreview pixmap w=%d h=%d\n", previewPixmap.width(), previewPixmap.height());
	gdk_draw_rgb_image (previewPixmap.gdkobj(), context.gdkobj(), 
					x, y, w, h, GDK_RGB_DITHER_NONE, pixbuf,
					osBuffer.pxbRowstride);
	gchar transp[65];
	memset(transp, 0, 64);
	int l0 = x / 8;
	int l1 = (x + w) / 8;
	int v0 = ~((1 << (x % 8)) - 1);
	int v1 = (1 << ((x + w) % 8)) - 1;
	if (l0 == l1) {
		v0 &= v1;
		v1 = v0;
	}
	guchar c0=(guchar) v0, c1 = (guchar) v1;
	//g_printerr("PatternListOrgDialog::createPatternPreview x=%d y=%d w=%d h=%d l0=%d l1=%d v0=%02X v1=%02X\n", x, y, w, h, l0, l1, c0, c1);
	for (uint k = y; k < y + h; k++) {
		int b = k * aw / 8;
		transp[b+l0] = v0;
		for (int m = l0+1; m < l1; m++) {
			transp[b+m] = 0xff;
		}
		transp[b+l1] = v1;
	}
	previewMask.create(document->application->toolWindow->get_window(), transp, aw, ah);
	return previewPixmap;
}
