/* $Id: PatternListSelectionDialog.cc,v 1.2 2004/03/23 11:42:41 pklaus Exp $
 *
 * PatternListSelectionDialog.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "config.h"
#include "PatternListSelectionDialog.hh"
#include "GDDocument.hh"
#include "PatternListOrgDialog.hh"

PatternListSelectionDialog::PatternListSelectionDialog(GDDocument *document, Pattern *active) : ListSelectionDialog() {
	//g_printerr("PatternListSelectionDialog::PatternListSelectionDialog called: doc: %lX, activePatt: %lX\n", document, active);
	currentPattern = active;
	doc = document;
	updateList(document);
}

void PatternListSelectionDialog::onClickColumn(gint column) {
}

void PatternListSelectionDialog::onSelectRow(gint row, gint column, GdkEvent *event) {
	//g_printerr("PatternListSelectionDialog::onSelectRow: row=%d, col=%d\n", row, column);
	if (!event || event->type == GDK_BUTTON_PRESS) {
		gint count = row;
		PatternMap::iterator it = doc->patterns.begin();
		for (; it != doc->patterns.end() && count; it++, count--);
		currentPattern = (*it).second;
	} else if (event->type == GDK_2BUTTON_PRESS) {
		okButton->clicked();
	}
}

void PatternListSelectionDialog::onResize(GtkAllocation *all) {
}

void PatternListSelectionDialog::updateList(GDDocument *document) {
	//g_printerr("PatternListSelectionDialog::updateList doc:%lX\n", document);
	Pattern *cur = currentPattern;
	objectList->clear();
	gint count = 0;
	gint selected = -1;
	if (! document) return;
	for (PatternMap::iterator it = document->patterns.begin(); it != document->patterns.end(); it++) {
		char *text[] = { "", "" };
		//g_printerr("PatternListSelectionDialog::updateList row:%d grad:%lX currg:%lX\n", count, (long) (*it).second, (long) cur);
		if ((*it).second == cur) {
			selected = count;
		}
		count++;
		int row = objectList->append(text);
		Gdk_Bitmap previewMask;
		Gdk_Pixmap previewIcon;
		Gdk_GC context = get_style()->get_white_gc();
		PatternListOrgDialog::createPatternPreview((*it).second, previewIcon, previewMask, context, document);
		objectList->cell(row, 0).set_pixmap(previewIcon, previewMask);
		objectList->cell(row, 1).set_text((*it).first);
	}
	if (selected >= 0) {
		objectList->row(selected).select();
	}
	//g_printerr("PatternListSelectionDialog::updateList reached end selected=%d\n", selected);
}

PatternListSelectionDialog::~PatternListSelectionDialog() {
}
