/* $Id: PixelImageData.cc,v 1.6 2003/12/06 21:42:11 pklaus Exp $
 *
 * PixelImageData.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "PixelImageData.hh"
#include <gdk-pixbuf/gdk-pixbuf.h>

PixelImageData::PixelImageData() {
	referenceCount = 0;
	data = NULL;
	width = 0;
	height = 0;
	rowstride = 0;
}

PixelImageData::PixelImageData(string filename) {
	referenceCount = 1;
	data = NULL;
	width = 0;
	height = 0;
	rowstride = 0;
	// load the image data
	GdkPixbuf *gdkpixbuf = gdk_pixbuf_new_from_file(filename.c_str());
	if (gdkpixbuf) {
		width = gdk_pixbuf_get_width(gdkpixbuf);
		height = gdk_pixbuf_get_height(gdkpixbuf);
		rowstride = gdk_pixbuf_get_rowstride(gdkpixbuf);
		guchar *pixdata = gdk_pixbuf_get_pixels(gdkpixbuf);
		//g_printerr("PixelImageData::PixelImageData got image w: %d, h: %d, rs: %d\n", width, height, rowstride);
		int nch = gdk_pixbuf_get_n_channels(gdkpixbuf);
		gboolean alpha = gdk_pixbuf_get_has_alpha(gdkpixbuf);
		int bits = gdk_pixbuf_get_bits_per_sample(gdkpixbuf);
		//g_printerr("additional data: n_channel: %d, alpha: %d, bits: %d\n", nch, alpha, bits);
		if (nch == 4 && rowstride == width * 4) {
			data = (guchar *) memcpy((guchar *) g_malloc(width * height * 4), pixdata, width * height * 4);
			rowstride = width * 4;
		} else if (nch == 3 && ! alpha) {
			data = pixelCopy((guchar *) g_malloc(width * height * 4), pixdata, width, height, 3, 4);
			rowstride = width * 4;
		} else {
			g_printerr("PixelImageData: Unknown pixel data format\n");
		}
		gdk_pixbuf_unref(gdkpixbuf);
	}
}

guchar *PixelImageData::pixelCopy(guchar *dest, guchar *src, int width, int height, int incpp, int outcpp) {
	guchar *result = dest;
	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			for (int cpp = 0; cpp < outcpp; cpp++) {
				if (cpp < incpp) {
					/*if (x == 20 && y == 20 ) {
						g_printerr("pixelCopy: src[%d] = %X\n", cpp, *src);
					}*/
					*dest++ = *src++;
				} else {
					*dest++ = 0xFF;
				}
			}
		}
	}
	return result;
}

void PixelImageData::incRefCount() {
	referenceCount++;
}

void PixelImageData::decRefCount() {
	referenceCount--;
	if (referenceCount <= 0) {
		delete this;
	}
}

void PixelImageData::writeXML(ostream& xmlout) {
	guchar *writeData = data;
	//g_printerr("PixelImageData::writeXML size:%d data:%lX width:%d height:%d\n", width*height*4, data, width, height);
	for (guint y = height; y > 0; y--) {
		for (guint x = width * 4; x > 0; x--) {
			writeHexByte(xmlout, *writeData);
			writeData++;
		}
		xmlout << "\n";
	}
}

void PixelImageData::writePS(ostream& psout) {
	// write rgb data with alpha channel
	guchar *writeData = data;
	//g_printerr("PixelImageData::writePS size:%d data:%lX width:%d height:%d\n", width*height*4, data, width, height);
	for (guint y = height; y > 0; y--) {
		for (guint x = width; x > 0; x--) {
			writeHexByte(psout, writeData[3]); // alpha channel
			writeHexByte(psout, writeData[0]);
			writeHexByte(psout, writeData[1]);
			writeHexByte(psout, writeData[2]);
			writeData += 4;
		}
		psout << "\n";
	}
}

void PixelImageData::writePSColor(ostream& psout) {
	// write rgb data with alpha channel
	guchar *writeData = data;
	//g_printerr("PixelImageData::writePSColor size:%d data:%lX width:%d height:%d\n", width*height*4, data, width, height);
	for (guint y = height; y > 0; y--) {
		for (guint x = width; x > 0; x--) {
			writeHexByte(psout, 255 - (255 - writeData[0]) / (256 - writeData[3]));
			writeHexByte(psout, 255 - (255 - writeData[1]) / (256 - writeData[3]));
			writeHexByte(psout, 255 - (255 - writeData[2]) / (256 - writeData[3]));
			writeData += 4;
		}
		psout << "\n";
	}
}

void PixelImageData::writePSMask(ostream& psout) {
	// write alpha channel as single bit mask
	guchar *writeData = data;
	//g_printerr("PixelImageData::writePSMask size:%d data:%lX width:%d height:%d\n", width*height*4, data, width, height);
	for (guint y = height; y > 0; y--) {
		guint bit = 0;
		guchar val = 0;
		for (guint x = width; x > 0; x--) {
			if (writeData[0]) {
				val |= 1 << bit;
			}
			bit++;
			if (bit > 7) {
				writeHexByte(psout, val);
				val = 0;
				bit = 0;
			}
			writeData += 4;
		}
		if (bit) {
			writeHexByte(psout, val);
		}
		psout << "\n";
	}
}

void PixelImageData::writePSGray(ostream& psout) {
	// write gray value of color data
	guchar *writeData = data;
	//g_printerr("PixelImageData::writePSGray size:%d data:%lX width:%d height:%d\n", width*height*4, data, width, height);
	for (guint y = height; y > 0; y--) {
		for (guint x = width; x > 0; x--) {
		   int val = (writeData[0] + writeData[1] + writeData[2]) / (256 - writeData[3]);
			val /= 3;
			writeHexByte(psout, val);
			writeData += 4;
		}
		psout << "\n";
	}
}

void PixelImageData::writeHexByte(ostream& out, guchar ch) {
	guchar hn = ch >> 4;
	if (hn > 9) {
		hn += 'A' - 10;
	} else {
		hn += '0';
	}
	guchar ln = ch & 0x0F;
	if (ln > 9) {
		ln += 'A' - 10;
	} else {
		ln += '0';
	}
	out << hn << ln;
}

PixelImageData::~PixelImageData() {
	if (data) {
		g_free(data);
	}
}
