/* $Id: SelectionItemList.cc,v 1.19 2004/07/21 06:31:20 pklaus Exp $
 *
 * SelectionItemList.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "SelectionItemList.hh"
#include "SelectionItem.hh"
#include "SelectableItem.hh"
#include "GDDocument.hh"
#include "BezierNode.hh"
#include "BezierPath.hh"
#include "Layer.hh"
#include "BoundingBox.hh"
#include <unistd.h>

SelectionItemList::SelectionItemList() {
	lastTMSet = false;
}

gint SelectionItemList::idleDraw() {
	//g_printerr("SelectionItemList::idleDraw called\n");
	// we have something to draw
	RectangleArea winRect = currDocwin->getContentRectangle();
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->moving->drawSelection(currDocwin, currContext, winRect);
		si->moving->setCoords(si->orig);
		si->moving->transformByMatrix(nextTransformation, false);
		si->moving->drawSelection(currDocwin, currContext, winRect);
	}
	newTransformation = 0;
	return 0;
}

void SelectionItemList::draw(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->moving->drawSelection(docwin, context, drawArea);
	}
}

void SelectionItemList::transformByMatrix(TransformationMatrix &m, gboolean incFillPattern) {
	//g_printerr("SelectionItemList::transformByMatrix\n");
	lastTM = m; // store for later retrieval
	lastTMSet = true;
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->moving->setCoords(si->orig);
		si->moving->transformByMatrix(m, incFillPattern);
	}
}

void SelectionItemList::transformByMatrixRelative(TransformationMatrix &m, gboolean incFillPattern) {
	//g_printerr("SelectionItemList::transformByMatrixRelative\n");
	lastTM = m; // store for later retrieval
	lastTMSet = true;
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->moving->transformByMatrix(m, incFillPattern);
	}
}

void SelectionItemList::setFillMatrix(TransformationMatrix& m) {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->orig->setFillMatrix(m);
	}
}

void SelectionItemList::gatherResource(GDResourceList& collection, gint type) {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		si->orig->gatherResource(collection, type);
	}
}

void SelectionItemList::transformAndDraw(DocWindow *docwin, Gdk_GC& context, TransformationMatrix& m, gboolean incFillPattern) {
	lastTM = m;
	lastTMSet = true;
	currDocwin = docwin;
	currContext = context;
	nextTransformation = m;
	area.left = docwin->osBuffer.origin.x;
	area.top = docwin->osBuffer.origin.y;
	area.right = area.left + docwin->DocWinDrawingArea->width() * docwin->osBuffer.scaleFactor;
	area.bottom = area.top + docwin->DocWinDrawingArea->height() * docwin->osBuffer.scaleFactor;
	newTransformation++;
	if (newTransformation == 1) {
		Gtk::Main::idle.connect(slot(this, &SelectionItemList::idleDraw));
	}
}

TransformationMatrix SelectionItemList::getLastTransformation() {
	return lastTM;
}

void SelectionItemList::startTransformation(DocWindow *docwin, Gdk_GC& context) {
	RectangleArea winRect = docwin->getContentRectangle();
	newTransformation = 0;
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		//g_printerr("Applying item\n");
		if (si->moving) {
			si->moving->startSelectionMovement(docwin, context);
		}
	}
	// the draw command was removed from the tools and must be called here now
	draw(docwin, context, winRect);
}

void SelectionItemList::endTransformation(DocWindow *docwin, Gdk_GC& context) {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		//g_printerr("Applying item\n");
		if (si->moving) {
			si->moving->endSelectionMovement(docwin, context);
		}
	}
}

void SelectionItemList::applyToOriginal() {
	// wait for drawing (and changing) routines to end
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		//g_printerr("Applying item %lX\n", si->orig);
		if (lastTMSet) {
			si->orig->transformByMatrix(lastTM, true);
		} else {
			si->orig->setCoords(si->moving);
		}
	}
}

void SelectionItemList::clearOriginal() {
	//g_printerr("SelectionItemList::clearOriginal\n");
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si = *siit++)) {
		//g_printerr("Removing item o: %lX, m: %lX\n", si->orig, si->moving);
		si->orig->clear();
	}
	//g_printerr("SelectionItemList::clearOriginal finished\n");
}

void SelectionItemList::changePosition(GDDocument *doc, gint what) {
	//g_printerr("SelectionItemList::changePosition\n");
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	GObject *lastobject = NULL;
	while ((si = *siit++)) {
		SelectableItem *item = si->orig;
		GObject *gobject = item->getGObject();
		// we want the most general
		while (gobject->parent) {
			//g_printerr("SelectionItemList::changePosition gobject %lX has parent %lX\n", gobject, gobject->parent);
			gobject = gobject->parent;
		}
		if (gobject == lastobject) {
			// every object should be transformed only once
			//g_printerr("SelectionItemList::changePosition gobject %lX is the same as the last one\n", gobject);
			continue;
		}
		lastobject = gobject;
		Layer *layer = doc->findObjectLayer(gobject);
		//g_printerr("SelectionItemList::changePosition: gobject: %lX (%s) what=%d\n", gobject, gobject->identify(), what);
		if (! layer) {
			//g_printerr("Cannot find layer for object %lX\n", gobject);
			continue;
		}
		/* g_printerr("BEFORE: ");
		for (GObject *pa = layer->paths; pa; pa = pa->next) {
			g_printerr("%lX, ", pa);
		}
		g_printerr("\n"); */
		if (what == GDPOSITION_RAISE) {
			//g_printerr("RAISE: gobject: %lX, paths: %lX\n", gobject, layer->paths);
			if (gobject->next) {
				if (layer->paths == gobject) {
					layer->paths = gobject->next;
				}
				if (gobject->prev) {
					gobject->prev->next = gobject->next;
				}
				gobject->next->prev = gobject->prev;
				if (gobject->next->next) {
					gobject->next->next->prev = gobject;
				}
				GObject *oldnext = gobject->next->next;
				gobject->next->next = gobject;
				gobject->prev = gobject->next;
				gobject->next = oldnext;
			}
		} else if (what == GDPOSITION_TOP) {
			//g_printerr("TOP: gobject: %lX, paths: %lX\n", gobject, layer->paths);
			if (gobject->next) {
				if (! (layer->paths->remove(gobject))) {
					g_printerr("Cannot remove gobject %lX\n", (long) gobject);
				}
				if (layer->paths == gobject) {
					layer->paths = gobject->next;
				}
				gobject->next = NULL;
				gobject->prev = NULL;
				layer->paths->insert(NULL, gobject);
			}
		} else if (what == GDPOSITION_BOTTOM) {
			//g_printerr("BOTTOM: gobject: %lX, paths: %lX\n", gobject, layer->paths);
			if (gobject->prev) {
				//g_printerr("BOTTOM: object is not yet on bottom\n");
				if (! (layer->paths->remove(gobject))) {
					g_printerr("Cannot remove gobject %lX\n", (long) gobject);
				}
				gobject->prev = NULL;
				gobject->next = layer->paths;
				layer->paths->prev = gobject;
				layer->paths = gobject;
			}
		} else if (what == GDPOSITION_LOWER) {
			//g_printerr("LOWER: gobject: %lX, paths: %lX\n", gobject, layer->paths);
			if (gobject->prev) {
				if (layer->paths == gobject->prev) {
					layer->paths = gobject;
				}
				gobject->prev->next = gobject->next;
				if (gobject->next) {
					gobject->next->prev = gobject->prev;
				}
				GObject *oldprev = gobject->prev->prev;
				gobject->prev->prev = gobject;
				gobject->next = gobject->prev;
				gobject->prev = oldprev;
				if (oldprev) {
					oldprev->next = gobject;
				}
			}
		}	
		/* g_printerr("AFTER: ");
		for (GObject *pa = layer->paths; pa; pa = pa->next) {
			g_printerr("%lX, ", pa);
		}
		g_printerr("\n"); */
	}
	//g_printerr("SelectionItemList::changePosition finished\n");
}

void SelectionItemList::insertMoving(GDDocument *doc) {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	BezierNode *oldnext = NULL;
	BezierPath *npath = NULL;
	while ((si = *siit++)) {
		// ensure the moving copy is transformed correctly
		if (lastTMSet) {
			si->moving->setCoords(si->orig);
			si->moving->transformByMatrix(lastTM, true);
		}
		//g_printerr("SelectionItemList::insertMoving handling item\n");
		SelectableItem *item = si->moving;
		if (! strcmp(item->identify(), "BezierNode")) {
			//g_printerr("SelectionItemList::insertMoving BezierNode\n");
			if (oldnext == item) {
				//g_printerr("SelectionItemList::insertMoving BezierNode old path %lX\n", npath);
				BezierNode *node = (BezierNode *) item;
				oldnext = node->next;
				node->next = NULL;
				npath->addNode(node);
				//g_printerr("node added to path\n");
			} else {
				BezierNode *node = (BezierNode *) item;
				oldnext = node->next;
				node->next = NULL;
				npath = new BezierPath(doc);
				//g_printerr("SelectionItemList::insertMoving BezierNode new path %lX\n", npath);
				doc->activeLayer->addObject(npath);
				if (node->prev) {
					BezierNode *n = (BezierNode *) node->prev->clone();
					n->next = 0;
					npath->addNode(n);
				}
				npath->addNode(node);
			}
		} else {
			if (oldnext) {
				//g_printerr("Add compagnion node before next object\n");
				BezierNode *node = (BezierNode *) oldnext->clone();
				node->next = NULL;
				npath->addNode(node);
				oldnext->path->deSelectAll();
				npath->selectAll();
				oldnext = NULL;
				npath = NULL;
			}
			//g_printerr("SelectionItemList::insertMoving GObject (%s, %lX)\n", item->identify(), item);
			// every non-BezierNode SelectableItem is a GObject
			item->setDocument(doc);
			doc->activeLayer->addObject((GObject *) item);
			((GObject *) item)->selectAll();
		}
		si->moving = NULL; // ensure the new Item is not deleted when selection is not used anymore
	}
	if (oldnext) {
		//g_printerr("Add compagnion node before finishing\n");
		BezierNode *node = (BezierNode *) oldnext->clone();
		node->next = NULL;
		npath->addNode(node);
		oldnext->path->deSelectAll();
		npath->selectAll();
	}
	//g_printerr("SelectionItemList::insertMoving finished\n");
}


BoundingBox SelectionItemList::getBoundingBox() {
	SelectionItem *si;
	BoundingBox boundings;
	G_List_Iterator<SelectionItem *> siit = begin();
	//g_printerr("SelectionItemList::getBoundingBox: boundings: %lX\n", boundings);
	//g_printerr("BoundingBox initiated, calling objects (%f,%f) (%f,%f)\n", boundings->left, boundings->top, boundings->right, boundings->bottom);
	while ((si = *siit++)) {
		BoundingBox bb;
		if (si->moving) {
			bb = si->moving->getBoundingBox();
		} else {
			bb = si->orig->getBoundingBox();
		}
		//g_printerr("Object bb: %lX\n", bb);
		//g_printerr("Object BBox: %f,%f %f,%f\n", bb->left, bb->top, bb->right, bb->bottom);
		boundings.combine(bb);
	}
	//g_printerr("Final BBox: %f,%f %f,%f\n", Bbox->left, Bbox->top, Bbox->right, Bbox->bottom);
	return boundings;
}

void SelectionItemList::reSelect() {
	SelectionItem *si;
	G_List_Iterator<SelectionItem *> siit = begin();
	while ((si = *siit++)) {
		si->orig->select(false);
	}
}

SelectionItemList::~SelectionItemList() {
	G_List_Iterator<SelectionItem *> siit = begin();
	SelectionItem *si;
	while ((si =*siit++)) {
		//g_printerr("Deleting item\n");
		delete si;
	}
	//g_printerr("SelectionItemList::~SelectionItemList finished\n");
}
