/* $Id: ShearTool.cc,v 1.3 2003/09/18 20:38:33 pklaus Exp $
 *
 * ShearTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "ShearTool.hh"
#include <math.h>

ShearTool::ShearTool() {
	pixmapData = shear_xpm;
	toolName = "ShearTool";
	valueDialog = NULL;
}


TransformationMatrix ShearTool::calcMatrix(gdouble sx, gdouble sy, 
						gdouble cx, gdouble cy,
						gdouble x, gdouble y, gint state) {

	TransformationMatrix tm;
	gdouble dsx = sx - cx;
	if (fabs(dsx) < 0.000001) {
		dsx = copysign(0.000001, dsx);
	}
	gdouble dsy = sy - cy;
	if (fabs(dsy) < 0.000001) {
		dsy = copysign(0.000001, dsy);
	}
	gdouble ax, ay;
	if (dsy == 0) {
		ax = 0;
	} else {
		ax = (x - sx) / dsy;
	}
	if (dsx == 0) {
		ay = 0;
	} else {
		ay = (y - sy) / dsx;
	}
	if (state & GDK_SHIFT_MASK) {
		if (fabs(x - sx) > fabs(y - sy)) {
			ay = 0;
		} else {
			ax = 0;
		}
	}
	//g_printerr("ShearTool::calcMatrix \nx:%f y:%f\nsx:%f sy:%f\ncx:%f cy:%f\nax:%f ay:%f\n", x, y, sx, sy, cx, cy, ax, ay);
	tm.setShear(ax, ay);
	tm.setCenter(cx, cy);
	return tm;
}
TransformationMatrix ShearTool::setMatrix(gdouble angle, gdouble axisAngle) {
	TransformationMatrix tm;
	tm.setRotation(-axisAngle);
	TransformationMatrix temp;
	temp.setShear(tan(-angle), 0);
	tm *= temp;
	temp.reset();
	temp.setRotation(axisAngle);
	tm *= temp;
	tm.setCenter(cx, cy);
	return tm;
}

gint ShearTool::deleteValueDialog(GdkEventAny *ev) {
	cancelValueDialog();
	return 1;
}

void ShearTool::cancelValueDialog() {
	valueDialog->hide();
	mode = 0;
}

void ShearTool::transformObjects() {
	gdouble axisAngle = valueDialog->getAxisAngle();
	gdouble angle = valueDialog->getAngle();
	TransformationMatrix tm = setMatrix(angle, axisAngle);
	doTransform(tm, false);
	valueDialog->hide();
}

void ShearTool::copyObjects() {
	gdouble axisAngle = valueDialog->getAxisAngle();
	gdouble angle = valueDialog->getAngle();
	TransformationMatrix tm = setMatrix(angle, axisAngle);
	doTransform(tm, true);
	valueDialog->hide();
}

void ShearTool::showValueDialog() {
	if (! valueDialog) {
		valueDialog = new ShearDialog();
		valueDialog->getOKButton()->clicked.connect(SigC::slot(this, &ShearTool::transformObjects));
		valueDialog->getCopyButton()->clicked.connect(SigC::slot(this, &ShearTool::copyObjects));
		valueDialog->getCancelButton()->clicked.connect(SigC::slot(this, &ShearTool::cancelValueDialog));
		valueDialog->delete_event.connect(SigC::slot(this, &ShearTool::deleteValueDialog));
		valueDialog->set_modal(true);
	}
	valueDialog->show();
}

