/* $Id: StandardTextObject.hh,v 1.29 2004/07/21 06:31:20 pklaus Exp $
 *
 * StandardTextObject.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _STANDARDTEXTOBJECT_HH
#  define _STANDARDTEXTOBJECT_HH

#include <gdk--.h>
#include "GObject.hh"
#include "TransformationMatrix.hh"
#include "TextObject.hh"
#include <vector>
#include <sys/timeb.h>

class TransformationMatrix;
class BoundingBox;
class SelectableItem;
class BezierPath;
class DocWindow;
class GDStyle;
class SelectionItemList;
class GDDocument;
class TextRow;
class ObjectGroup;

enum drawing_routine {
	bufferDraw,
	drawDrawable,
	xorDraw
};

struct OutputDescription {
	DocWindow *docwin;
	OffscreenBuffer *osBuffer;
};

typedef void (*invertDrawingFunction) (OutputDescription&, position, position, position, position);

class StandardTextObject : public TextObject {

	vector<TextRow *> text;
	
	// style storage
	GDStyle *fillStyle;
	GDStyle *strokeStyle;
	
	// text selection
	gint selOriginRow, selOriginColumn;
	gint selTargetRow, selTargetColumn;
	gint selStartRow, selStartColumn;
	gint selEndRow, selEndColumn;
	

	void bufferDrawPoint(OffscreenBuffer& osBuffer, position point, guint color);
	void doDraw(OffscreenBuffer& osBuffer, DocWindow *docwin, Gdk_GC* context, drawing_routine, RectangleArea& drawArea);
	void insertRow(gint after, TextRow *row);
	void removeRow(gint pos);
	
	void calcTextPosition(gdouble px, gdouble py, gint &tcol, gint &trow);
	void drawTextSelRange(OutputDescription& odes, invertDrawingFunction drawInvert, guint sRow, guint sColumn, guint eRow, guint eColumn);
	void removeSelectedText();

public:

	StandardTextObject(GDDocument *document);
	StandardTextObject(GDDocument *document, gint dummy);
	
	void appendRow(TextRow *row) { insertRow(-1, row); };

	// GObject methods
	void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	BoundingBox getBoundingBox();
	gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);
	SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);
	GObject *checkArea(gdouble x, gdouble y);
	SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	gint getSelection(SelectionItemList& selectionList, gint mode);
	void deSelectAll();
	gint writeXML(ostream& xmlout);
	gint writePS(ostream& psout);
	void reverseOrder() {};
	void setDocument(GDDocument *document);

	// DrawObject methods
	void drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea);
	GDStyle *getStrokeStyle();
	GDStyle *getFillStyle();
	void setStrokeStyle(const GDStyle& from);
	void setFillStyle(const GDStyle& from);
	void setTextStyle(TextStyle& style);
	void gatherResource(GDResourceList& collection, gint type);
	
	// SelectableItem methods
	void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	void setFillMatrix(TransformationMatrix& m);
	SelectableItem *clone();
	void setCoords(SelectableItem* from);
	gboolean selected();
	int getCoordinates(gdouble& x, gdouble& y);
	void clear();
	gchar *identify();
	
	// TextObject methods
	gint insertCharacter(int character);
	gint insertKeyVal(int keyval, int state);
	gint startTextSelection(DocWindow *docwin, Gdk_GC& context, gdouble x, gdouble y);
	gint setTextSelection(DocWindow *docwin, gdouble x, gdouble y);
	void drawTextCursorMark(DocWindow *docwin, RectangleArea& drawArea);
	void drawTextSelection(DocWindow *docwin, RectangleArea& drawArea);
	void bufferDrawTextSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	TextStyle getTextStyle();
	vector<GDCharacter> getTextSelection();
	void insertTextSelection(vector<GDCharacter>);
	ObjectGroup *convert2Outline();
	
	~StandardTextObject();

};

#endif
