/* $Id: TextObject.hh,v 1.27 2004/07/21 06:31:20 pklaus Exp $
 *
 * TextObject.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _TEXTOBJECT_HH
#  define _TEXTOBJECT_HH

#include <gdk--.h>
#include "GObject.hh"
#include "TransformationMatrix.hh"
#include <time.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H
#include <vector>
#include "GDCharacter.hh"
#include "DocWindow.hh"

class TransformationMatrix;
class BoundingBox;
class SelectableItem;
class BezierPath;
class GDStyle;
class SelectionItemList;
class GDDocument;
class GDFont;
class TextStyle;
class ObjectGroup;

class TextPixelBuffer {
public:
	OffscreenBuffer osBuffer;
	gint left, top, right, bottom;
};
	

class TextObject : public GObject {

protected:
	TransformationMatrix tMatrix;
	TransformationMatrix invTMatrix;
	GDStyle *drawStyle;
	GDStyle *fillStyle;
	
	gboolean selection; // is TextObject selected or not
	
	guint lastTarget;
	
	TextPixelBuffer *lastBuffer;
	TextPixelBuffer *currBuffer;
	
	int bufferCopyCharBitmap(guchar *buffer, guint buffsize, guint rowstride, FT_Glyph *glyph,
								gint x, gint y);
	int copyCharBitmap(Gdk_Window window, Gdk_GC context, FT_Glyph *glyph, gint x, gint y);
	int selCopyCharBitmap(Gdk_Window window, Gdk_GC context, FT_Glyph *glyph, gint x, gint y);
	guint getOutcode(position point, 
					gdouble left, gdouble top, gdouble right, gdouble bottom);
	int checkRectangleLineIntersection(position start, position end,
					gdouble left, gdouble top, gdouble right, gdouble bottom);
	void drawPoint(DocWindow *docwin, Gdk_GC& context, position point);
	
public:
	
	gboolean editing; // TextObject is currently being edited

	TextObject(GDDocument *document);
	
	// GObject methods
	virtual void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	virtual void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	virtual void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	virtual void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	virtual void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	virtual BoundingBox getBoundingBox() = 0;
	virtual gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0) = 0;
	virtual SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints) = 0;
	virtual GObject *checkArea(gdouble x, gdouble y) = 0;
	virtual SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance) = 0;
	virtual void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) = 0;
	virtual void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) = 0;
	GObject *removeObject(GObject *object);
	GObject *findObject(GObject *object);
	virtual void selectAll();
	virtual void deSelectAll();
	gint getSelection(SelectionItemList& selectionList, gint mode);
	virtual gint writeXML(ostream& xmlout) = 0;
	virtual gint writePS(ostream& psout) = 0;
	
	// DrawObject methods
	virtual void drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	virtual void drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea) = 0;
	GDStyle *getStrokeStyle();
	GDStyle *getFillStyle();
	
	// SelectableItem methods
	virtual void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	virtual void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	virtual SelectableItem *clone() = 0;
	virtual void setCoords(SelectableItem* from) = 0;
	gboolean selected();
	virtual int getCoordinates(gdouble& x, gdouble& y) = 0;
	virtual void clear() = 0;
	virtual gchar *identify() = 0;
	
	// own methods
	virtual gint insertCharacter(int character) = 0;
	virtual gint insertKeyVal(int keyval, int state) = 0;
	virtual gint startTextSelection(DocWindow *docwin, Gdk_GC& context, gdouble x, gdouble y) = 0;
	virtual gint setTextSelection(DocWindow *docwin, gdouble x, gdouble y) = 0;
	virtual void drawTextCursorMark(DocWindow *docwin, RectangleArea& drawArea) = 0;
	virtual void drawTextSelection(DocWindow *docwin, RectangleArea& drawArea) = 0;
	virtual void bufferDrawTextSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	virtual TextStyle getTextStyle() = 0;
	virtual vector<GDCharacter> getTextSelection() = 0;
	virtual void insertTextSelection(vector<GDCharacter>) = 0;
	virtual ObjectGroup *convert2Outline() = 0;
	
	~TextObject();

};

#endif
