/* $Id: TextTool.cc,v 1.20 2004/07/21 06:31:20 pklaus Exp $
 *
 * TextTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "TextTool.hh"
#include "../pixmaps/text.xpm"
#include "../cursors/text_edit.xbm"
#include "../cursors/text_edit_mask.xbm"
#include "../cursors/text_rect.xbm"
#include "../cursors/text_rect_mask.xbm"
#include "../cursors/text_curve.xbm"
#include "../cursors/text_curve_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"
#include "Layer.hh"
#include "StandardTextObject.hh"
#include "TextObject.hh"
#include "TextAction.hh"
#include "AddAction.hh"
#include <ft2build.h>
#include FT_FREETYPE_H
#include <gtk--/main.h>

FT_Library freetype_library;

TextTool::TextTool() {
	int error = FT_Init_FreeType(&freetype_library);
	if (error) {
		g_printerr("Error while initiating the FreeType library (%d)\n", error);
	}
	pixmapData = text_xpm;
	toolName = "TextTool";
	moving = false;
	nodeCaught = false;
	coIsActive = false;
	cursorDrawing = false;
	currentObject = NULL;
	CursorFactory::createCursorFromData(TextEditCursor,
					text_edit_bits, text_edit_mask_bits,
					text_edit_width, text_edit_height,
					text_edit_x_hot, text_edit_y_hot);
	CursorFactory::createCursorFromData(TextRectCursor,
					text_rect_bits, text_rect_mask_bits,
					text_rect_width, text_rect_height,
					text_rect_x_hot, text_rect_y_hot);
	CursorFactory::createCursorFromData(TextCurveCursor,
					text_curve_bits, text_curve_mask_bits,
					text_curve_width, text_curve_height,
					text_curve_x_hot, text_curve_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
	
}

void TextTool::activate() {
	Tool::activate();
}

void TextTool::deactivate() {
	coIsActive = false;
	if (currentObject) {
		currentObject->editing = false;
	}
	currentObject = NULL;
	currentDocWin = NULL;
	Tool::deactivate();
}

gint TextTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;
	downButton = ev->button;

	if (downButton == 1) {
		currentDocWin = docwin;
		currentObject = NULL;
		coIsActive = false;
		doc = docwin->doc;
		doc->deSelectAll();
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		GObject *sobj = doc->checkArea(sx, sy);
		//g_printerr("TextTool:buttonDown: checkArea returned %lX\n", sobj);
		//if (sobj) {
		//	g_printerr("TextTool::buttonDown sobj->identify %s\n", sobj->identify());
		//}
		if (sobj && strstr(sobj->identify(), "Text")) {
			// click is within a TextObject
			currentObject = (TextObject *) sobj;
			currentObject->select(false);
			currentObject->editing = true;
			// calculate text cursor position
			currentObject->startTextSelection(docwin, doc->context, sx, sy);
		} else {
			gboolean caught = magnet(sx, sy, docwin);
		}
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint TextTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		if (currentObject) {
			currentObject->setTextSelection(docwin,
								ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x,
								ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y);
		}
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		nodeCaught = false;
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint TextTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		//g_printerr("TextTool::buttonUp %f\n", doc->nullPoint.x);
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		if (currentObject) {
			// set text selection
			currentObject->setTextSelection(docwin, dx, dy);
			if (! coIsActive){
				coIsActive = true;
				// active the Cursor Handler
				startCursorHandler();
			}
		} else {
			gboolean caught = magnet(dx, dy, docwin);
			StandardTextObject *tobject = new StandardTextObject(doc);
			AddAction *newAction = new AddAction(tobject, doc->activeLayer);
			doc->setAction(newAction);
			currentObject = tobject;
			coIsActive = true;
			startCursorHandler();
			//Gtk::Main::idle.connect(slot(this, &TextTool::idleHandler));
			TransformationMatrix tm;
			tm.setTranslation(dx, dy);
			tobject->transformByMatrix(tm);
			docwin->doc->activeLayer->addObject(tobject);
		}
		doc->context.set_function(GDK_SET);
		doc->redraw();
		moving = false;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

gint TextTool::keyPressed(GdkEventKey *ev, DocWindow *docwin) {
	//g_printerr("TextTool::keyReleased length:%d\n", ev->length);
	if (coIsActive && currentObject) {
		if (ev->length) {
			for (int i = 0; i < ev->length; i++) {
				//g_printerr("TextTool::keyReleased adding char %c\n", ev->string[i]);
				guint charval = ev->string[i] & 0x000000FF;
				if (charval == 8) {
					TextAction *newAction = new TextAction(currentObject, delete_backward);
					docwin->doc->setAction(newAction);
				} else if (charval == 127) {
					TextAction *newAction = new TextAction(currentObject, delete_forward);
					docwin->doc->setAction(newAction);
				} else {
					TextAction *newAction = new TextAction(currentObject, charval);
					docwin->doc->setAction(newAction);
				}
				currentObject->insertCharacter(charval);
			}
		} else {
			currentObject->insertKeyVal(ev->keyval, ev->state);
		}
		docwin->doc->redraw();
	}
	return 1;
}

void TextTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (nodeCaught) {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
		} else {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
		}
	} else {
		if (state & GDK_MOD1_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(TextEditCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(TextEditCursor);
		}
	}
}

void TextTool::startCursorHandler() {
	if (! cursorDrawing) {
		cursorDrawing = true;
		Gtk::Main::timeout.connect(slot(this, &TextTool::cursorHandler), 500);
	}
}

gint TextTool::cursorHandler() {
	//g_printerr("TextTool::idleHandler called\n");
	if (! coIsActive) {
		cursorDrawing = false;
		return 0;
	}
	RectangleArea docWinRect = currentDocWin->getContentRectangle();
	currentObject->drawTextCursorMark(currentDocWin, docWinRect);
	return 1;
}

TextObject *TextTool::getCurrentObject() {
	if (coIsActive) {
		return currentObject;
	} else {
		return NULL;
	}
}

gboolean TextTool::editing() {
	if (coIsActive && currentObject) {
		return currentObject->editing;
	} else {
		return false;
	}
}

TextTool::~TextTool() {
	FT_Done_FreeType(freetype_library);
}
