/* $Id: Tool.cc,v 1.8 2004/03/24 19:31:43 pklaus Exp $
 *
 * Tool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "config.h"
#include "Tool.hh"
#include "DocumentMenu.hh"
#include <gtk--/pixmap.h>
#include <math.h>
#include "BezierNode.hh"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "GDApplication.hh"
#include "../cursors/navigate.xbm"
#include "../cursors/navigate_mask.xbm"
#include "CursorFactory.hh"


Tool::Tool() {
	CursorFactory::createCursorFromData(navigateCursor,
					navigate_bits, navigate_mask_bits,
					navigate_width, navigate_height,
					navigate_x_hot, navigate_y_hot);
	moving = false;
}

Gtk::RadioButton *Tool::getNewButton(Gtk::RadioButton_Helpers::Group& group) {
	button = manage(new class Gtk::RadioButton(group));
	button->set_mode(false);
	Gtk::Pixmap *pixmap = manage(new class Gtk::Pixmap(pixmapData));
	
	button->add(*pixmap);
	button->show();
	pixmap->show();
	return button;
}

Gtk::RadioButton *Tool::getButton() {
	return button;
}

void Tool::activate() {
	moving = false;
}

void Tool::deactivate() {
	moving = false;
}

gint Tool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	
	downButton = ev->button;
	
	if (downButton == 2) {
		moving = true;
		ox = ev->x;
		oy = ev->y;
		docwin->DocWinDrawingArea->get_window().set_cursor(navigateCursor);
	} else if (downButton == 3) {
		return docwin->onDocWindowMenuPopup(ev);
	}
	
	return FALSE;
}

gint Tool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON2_MASK) {
		if (fabs(ev->x - ox) + fabs(ev->y - oy) > 10 &&
				ev->time - lastTime > 200) {
			gdouble dx = (ev->x - ox) / docwin->osBuffer.scaleFactor;
			gdouble dy = (ev->y - oy) / docwin->osBuffer.scaleFactor;
			//g_printerr("dx: %f, dy: %f, orx: %f, ory: %f\n", dx, dy, docwin->origin.x, docwin->origin.y);
			//g_printerr("Tool::motion: time=%d\n", ev->time);
			docwin->osBuffer.origin.x -= dx;
			docwin->osBuffer.origin.y -= dy;
			ox = ev->x;
			oy = ev->y;
			//docwin->doc->draw(docwin);
			if (docwin->doScrollRedraw) {
				docwin->doScrollRedraw = false;
				docwin->updateScaling();
			}
			//g_printerr("orx: %f, ory: %f\n", docwin->origin.x, docwin->origin.y);
			lastTime = ev->time;
		}
	}

	return FALSE;
}

gint Tool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	moving = false;
	setCursor(ev->state, docwin);
	return FALSE;
}

gint Tool::showOptions(DocWindow *docwin) {
	return FALSE;
}

gboolean Tool::magnet(gdouble& x, gdouble& y, DocWindow *docwin) {
	gdouble ax = x;
	gdouble ay = y;
	gdouble tolerance = docwin->doc->application->preferences.catchingRange / docwin->osBuffer.scaleFactor;
	gdouble nearest = docwin->doc->attractPoint(x, y, ax, ay, tolerance);
	if (nearest < tolerance * tolerance) {
		x = ax;
		y = ay;
		return TRUE;
	}
	/*magnetItem = docwin->doc->checkPoints(x, y, 
					docwin->doc->application->preferences.catchingRange / 
					docwin->osBuffer.scaleFactor, true);
	gdouble nx, ny;
	if (magnetItem && magnetItem->getCoordinates(nx, ny)) {
		x = nx;
		y = ny;
		return TRUE;
	}*/
	return FALSE;
}

gint Tool::keyPressed(GdkEventKey *ev, DocWindow *docwin) {
	return 0;
}

gint Tool::keyReleased(GdkEventKey *ev, DocWindow *docwin) {
	return 0;
}

void Tool::setCursor(guint state, DocWindow *docwin) {
}

void Tool::drawRectangle(Gdk_Drawable& drawable, Gdk_GC& context,
									gint x1, gint y1,
									gint x2, gint y2) {
	gint x,y,w,h;
	
	if (x1 > x2) {
		x = x2;
		w = x1 - x2;
	} else {
		x = x1;
		w = x2 - x1;
	}
	if (y1 > y2) {
		y = y2;
		h = y1 - y2;
	} else {
		y = y1;
		h = y2 - y1;
	}
	drawable.draw_rectangle(context, 0, x, y, w, h);
}
