/* $Id: ToolWindow.cc,v 1.13 2005/05/07 17:39:33 pklaus Exp $
 *
 * ToolWindow.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


// generated 2001/11/3 20:45:41 CET by pklaus@iron.
// using glademm V0.6.2
//
// newer (non customized) versions of this file go to ToolWindow.cc_new

// This file is for your program, I won't touch it again!

#include "gestalter.h"
#include "ToolWindow.hh"
#include "gtk--/main.h"
#include "GDDocument.hh"
#include "BezierTool.hh"
#include "SelectPointTool.hh"
#include "SelectObjectTool.hh"
#include "RectangleTool.hh"
#include "CircleTool.hh"
#include "ZoomTool.hh"
#include "ScaleTool.hh"
#include "RotateTool.hh"
#include "ShearTool.hh"
#include "FlipTool.hh"
#include "AddPointTool.hh"
#include "RemovePointTool.hh"
#include "CutTool.hh"
#include "PageOriginTool.hh"
#include "TextTool.hh"
#include "FillTransTool.hh"
#include "MorphTool.hh"
#include "Tool.hh"
#include "HWrapBox.hh"
#include "DocumentMenu.hh"
#include "SVGInput.hh"
#include "Properties.hh"
#include "TextProperties.hh"
#include "FontPool.hh"
#include "AboutDialog.hh"
#include "GDApplication.hh"
#include "PreferencesDialog.hh"
#include <libgnome/libgnome.h>



ToolWindow::ToolWindow(GDApplication *gda) {
	application = gda;
	fileselection = NULL;
	numTools = 0;
	toolList = new G_ListImpl();
	setGeometry(); // this enables the "stepping" resize effect
	//set_default_size(78,130);
	toolWrapBox = manage(new HWrapBox(false));
	ToolWindowVBox->pack_start(*toolWrapBox, false, false, 0);
	toolWrapBox->set_justify(GTK_JUSTIFY_TOP);
	toolWrapBox->set_line_justify(GTK_JUSTIFY_LEFT);
	toolWrapBox->set_aspect_ratio(5.0/5.9);
	toolWrapBox->set_border_width(0);
	toolWrapBox->show();
	// add all standard tools
	SelectObjectTool *selectObject = new SelectObjectTool();
	addTool(selectObject);
	SelectPointTool *selectPoint = new SelectPointTool();
	addTool(selectPoint);
	BezierTool *bezier = new BezierTool();
	addTool(bezier);
	TextTool *text = new TextTool();
	addTool(text);
	RectangleTool *rectangle = new RectangleTool();
	addTool(rectangle);
	CircleTool *circle = new CircleTool();
	addTool(circle);
	ZoomTool *zoom = new ZoomTool();
	addTool(zoom);
	CutTool *cut = new CutTool();
	addTool(cut);
	ScaleTool *scale = new ScaleTool();
	addTool(scale);
	RotateTool *rotate = new RotateTool();
	addTool(rotate);
	ShearTool *shear = new ShearTool();
	addTool(shear);
	FlipTool *flip = new FlipTool();
	addTool(flip);
	FillTransTool *filltrans = new FillTransTool();
	addTool(filltrans);
	MorphTool *morph = new MorphTool();
	addTool(morph);
	AddPointTool *addpoint = new AddPointTool();
	addTool(addpoint);
	RemovePointTool *removepoint = new RemovePointTool();
	addTool(removepoint);
	PageOriginTool *pageOrigin = new PageOriginTool();
	addTool(pageOrigin);
	application->zoomTool = zoom;
	sigSetTool("SelectObjectTool");
}

void ToolWindow::addTool(Tool *tool) {
	numTools++;
	Gtk::RadioButton *button = tool->getNewButton(radioGroup);
	set_data(tool->toolName, tool);
	button->toggled.connect(SigC::bind(SigC::slot(this, &ToolWindow::sigSetTool), (gchar *)tool->toolName));
	toolWrapBox->pack(*button, false, false, false, false);
	button->show_now();
	//toolHBox->pack_start(*button, false, false, 0);
	toolList->insert(0, tool);
}

void ToolWindow::sigSetTool(gchar *name) {
	Tool *tool = (Tool *) get_data(name);
	/* this part is not necessary anymore because the tools are now a radiogroup
	if (! tool->getButton()->get_active()) {
		tool->getButton()->set_active(1);
	}
	if (tool == application->currentTool) {
		return;
	}
	GList *list = toolList->list();
	while (list) {
		Tool *curTool = (Tool *)(list->data);
		if (curTool != tool) {
			curTool->getButton()->set_active(0);
		}
		list = list->next;
	}*/
	if (application->currentTool) {
		application->currentTool->deactivate();
	}
	application->currentTool = tool;
	application->currentTool->activate();
	if (! (strcmp(tool->toolName, "SelectObjectTool")) || 
						! (strcmp(tool->toolName, "SelectPointTool"))) {
		application->lastSelectionTool = tool;
	}
	
}

void ToolWindow::on_new_document_activate() {
	GDDocument *newDoc = new GDDocument(application);
	application->addDocument(newDoc);
}

void ToolWindow::on_open_document_activate() {
	gchar* title = _("Open document:");
	if (fileselection == NULL) {
		fileselection = new Gtk::FileSelection(Gtk::nstring(title));
		fileselection->get_ok_button()->clicked.connect(SigC::slot(this, &ToolWindow::openDocument));
		fileselection->get_cancel_button()->clicked.connect(fileselection->destroy.slot());
		fileselection->set_modal(true);
	}
	fileselection->show();
	
}

void ToolWindow::openDocument() {
	if (fileselection) {
		string filePath = fileselection->get_filename();
		fileselection->hide();
		SVGInput::open(filePath);
	}
}

void ToolWindow::onPreferences() {
	preferencesDialog = new PreferencesDialog(&(application->preferences));
	preferencesDialog->getOKButton()->clicked.connect(SigC::slot(this, &ToolWindow::savePrefs));
	preferencesDialog->getCancelButton()->clicked.connect(SigC::slot(this, &ToolWindow::cancelPrefs));
	preferencesDialog->set_modal(true);
	preferencesDialog->show();
}

void ToolWindow::savePrefs() {
	preferencesDialog->setPreferencesFromDialog(&(application->preferences));
	application->prefsChanged(&(application->preferences));
	application->preferences.save();
	delete preferencesDialog;
}

void ToolWindow::cancelPrefs() {
	delete preferencesDialog;
}

void ToolWindow::on_file_activate() {   
}

void ToolWindow::save_exit() {   
	application->quit();
}

void ToolWindow::on_show_help_activate() {   
}

void ToolWindow::show_about(){
	AboutDialog *aboutWindow = new AboutDialog();
	aboutWindow->show();
}

gint ToolWindow::app_exit(GdkEventAny *ev){
	Gtk::Main::quit(); 
	return 0;
}

void ToolWindow::setGeometry() {
	//g_printerr("ToolWindow::style_changed_impl called\n");
	Gtk::Style *style = get_style();
	gint xthick = ((GtkStyleClass *) style->gtkobj()->klass)->xthickness;
	gint ythick = ((GtkStyleClass *) style->gtkobj()->klass)->ythickness;
	
	GdkGeometry geometry;
	geometry.min_width = 24 + 1 * xthick;
	geometry.min_height = 26 + 24 + 1 * ythick;
	geometry.width_inc = 24 + 1 * xthick;
	geometry.height_inc = 24 + 1 * ythick;
	
	set_geometry_hints(*this, &geometry, (GdkWindowHints) (GDK_HINT_MIN_SIZE | GDK_HINT_RESIZE_INC));
	//g_printerr("geometry: (%d, %d  %d, %d)\n", geometry.min_width, geometry.min_height, geometry.width_inc, geometry.height_inc);
}

gint ToolWindow::on_selection_clear_event(GdkEventSelection *ev) {
	g_printerr("ToolWindow::on_selection_clear_event\n");
	return 0;
}

void ToolWindow::on_selection_get(GtkSelectionData *selection_data,guint info,guint time) {  
	g_printerr("ToolWindow::on_selection_get\n");
}

gint ToolWindow::on_selection_notify_event(GdkEventSelection *ev) {
	g_printerr("ToolWindow::on_selection_notify_event\n");
	return 0;
}

void ToolWindow::on_selection_received(GtkSelectionData *selection_data,guint time) {
	g_printerr("ToolWindow::on_selection_received\n");
}

gint ToolWindow::on_selection_request_event(GdkEventSelection *ev) {
	g_printerr("ToolWindow::on_selection_request_event\n");
	return 0;
}

ToolWindow::~ToolWindow() {
	GList *list = toolList->list();
	while (list) {
		Tool *tool = (Tool *)(list->data);
		delete tool;
		list = list->next;
	}
	delete toolList;
}
