/* $Id: TransformationMatrix.hh,v 1.7 2004/03/06 15:00:19 pklaus Exp $
 *
 * TransformationMatrix.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _TRANSFORMATIONMATRIX_HH
#  define _TRANSFORMATIONMATRIX_HH
#include "gestalter.h"
#include <glib--.h>

struct position {
	gdouble x;
	gdouble y;
};

class TransformationMatrix {

	gdouble det22(int i, int k);

public:
	gdouble m[3][3];
	
	TransformationMatrix();
	
	TransformationMatrix(const TransformationMatrix& n);
	
	void reset();
	
	void setTranslation(gdouble dx, gdouble dy);
	void setRotation(gdouble angle);
	void setScale(gdouble hf, gdouble vf);
	void setShear(gdouble ax, gdouble ay);
	void setCenter(gdouble x, gdouble y);
	
	void transform(position& p);
	
	TransformationMatrix inverse();

	gboolean isIdentityMatrix();
	
	void printMatrix();
	
	TransformationMatrix& operator*=(const TransformationMatrix& a);
	TransformationMatrix operator* (const TransformationMatrix& b);
	gboolean operator==(const TransformationMatrix& b);

	gdouble getAngle(gdouble x, gdouble y);
	
	void writePS(ostream& psout);
	void writeXML(ostream& xmlout);

};

#endif
