/* $Id: WrapBox.cc,v 1.6 2003/06/23 07:14:21 pklaus Exp $
 *
 * WrapBox.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "WrapBox.hh"
#include <gtk/gtkcontainer.h>

WrapBox::WrapBox() {
	//g_printerr("WrapBox::WrapBox called\n");
	set_flags(GTK_NO_WINDOW);
	homogeneous = 0;
	justify = GTK_JUSTIFY_LEFT;
	line_justify = GTK_JUSTIFY_BOTTOM;
	hspacing = 0;
	vspacing = 0;
	n_children = 0;
	children = NULL;
	aspect_ratio = 1;
	child_limit = 32767;
}

void WrapBox::set_homogeneous(gboolean homogeneous) {
	homogeneous = homogeneous != false;
	if (this->homogeneous != homogeneous) {
		this->homogeneous = homogeneous;
		queue_resize();
	}
}

void WrapBox::set_hspacing(guint hspacing) {
	if (this->hspacing != hspacing) {
		this->hspacing = hspacing;
		queue_resize();
	}
}

void WrapBox::set_vspacing(guint vspacing) {
	if (this->vspacing != vspacing) {
		this->vspacing = vspacing;
		queue_resize();
	}
}

void WrapBox::set_justify(GtkJustification justify) {
	if (justify > GTK_JUSTIFY_FILL) {
		return;
	}
	if (this->justify != justify) {
		this->justify = justify;
		queue_resize();
	}
}

void WrapBox::set_line_justify(GtkJustification line_justify) {
	if (justify > GTK_JUSTIFY_FILL) {
		return;
	}
	if (this->line_justify != line_justify) {
		this->line_justify = line_justify;
		queue_resize();
	}
}

void WrapBox::set_aspect_ratio(gfloat aspect_ratio) {
	if (aspect_ratio < 1.0/256.0 || aspect_ratio > 256.0) {
		return;
	}
	if (this->aspect_ratio != aspect_ratio) {
		this->aspect_ratio = aspect_ratio;
		queue_resize();
	}
}

void WrapBox::pack(Gtk::Widget& child, gboolean hexpand, gboolean hfill,
				gboolean vexpand, gboolean vfill) {
	//g_printerr("WrapBox::pack called\n");
	WrapBoxChild *child_info = new WrapBoxChild(child);
	child_info->hexpand = hexpand ? true : false;
	child_info->hfill = hfill ? true : false;
	child_info->vexpand = vexpand ? true : false;
	child_info->vfill = vfill ? true : false;
	child_info->forced_break = false;
	child_info->next = NULL;
	if (children) {
		WrapBoxChild *last = children;
		while (last->next) last = last->next;
		last->next = child_info;
	} else {
		children = child_info;
	}
	n_children++;
	gtk_widget_set_parent(child.gtkobj(), GTK_WIDGET(this->gtkobj()));
	if (is_realized()) {
		gtk_widget_realize(child.gtkobj());
	}
	if (is_visible() && child.is_visible()) {
		if (is_mapped()) {
			gtk_widget_map(child.gtkobj());
		}
		child.queue_resize();
	}
}

void WrapBox::reorder_child(Gtk::Widget& child, gint position) {
	//g_printerr("WrapBox::reorder_child called\n");
	WrapBoxChild *child_info = NULL;
	WrapBoxChild *last = NULL;
	
	for (child_info = children; child_info; last = child_info, child_info = last->next) {
		if (child_info->widget == &child) {
			break;
		}
	}
	if (child_info && children->next) {
		if (last) {
			last->next = child_info->next;
		} else {
			children = child_info->next;
		}
		last = NULL;
		WrapBoxChild *tmp = children;
		while (position && tmp->next) {
			position--;
			last = tmp;
			tmp = last->next;
		}
		if (position) {
			tmp->next = child_info;
			child_info->next = NULL;
		} else {
			child_info->next = tmp;
			if (last) {
				last->next = child_info;
			} else {
				children = child_info;
			}
		}
		if (child.is_visible() && is_visible()) {
			child.queue_resize();
		}
	}
}

void WrapBox::query_child_packing(Gtk::Widget& child, gboolean& hexpand,
					gboolean& hfill, gboolean& vexpand, gboolean& vfill) {
	//g_printerr("WrapBox::query_child_packing called\n");
	WrapBoxChild *child_info;
	
	for (child_info = children; child_info; child_info = child_info->next) {
		if (child_info->widget == &child) {
			break;
		}
	}
	if (child_info) {
		hexpand = child_info->hexpand;
		hfill = child_info->hfill;
		vexpand = child_info->vexpand;
		vfill = child_info->vfill;
	}
}

void WrapBox::query_child_forced_break(Gtk::Widget& child, gboolean& forced_break) {
	//g_printerr("WrapBox::query_child_forced_break called\n");
	WrapBoxChild *child_info;
	
	for (child_info = children; child_info; child_info = child_info->next) {
		if (child_info->widget == &child) {
			break;
		}
	}
	if (child_info) {
		forced_break = child_info->forced_break;
	}
}

void WrapBox::set_child_packing(Gtk::Widget& child, gboolean hexpand, gboolean hfill,
					gboolean vexpand, gboolean vfill) {
	//g_printerr("WrapBox::set_child_packing called\n");
	WrapBoxChild *child_info;
	
	hexpand = hexpand != false;
	hfill = hfill != false;
	vexpand = vexpand != false;
	vfill = vfill != false;
	
	for (child_info = children; child_info; child_info = child_info->next) {
		if (child_info->widget == &child) {
			break;
		}
	}
	if (child_info && (child_info->hexpand != hexpand || child_info->vexpand != vexpand ||
		child_info->hfill != hfill || child_info->vfill != vfill)) {
		
		child_info->hexpand = hexpand;
		child_info->hfill = hfill;
		child_info->vexpand = vexpand;
		child_info->vfill = vfill;
		if (child.is_visible() && is_visible()) {
			child.queue_resize();
		}
	}
}

void WrapBox::set_child_forced_break(Gtk::Widget& child, gboolean forced_break) {
	//g_printerr("WrapBox::set_child_forced_break called\n");
	WrapBoxChild *child_info;
	
	forced_break = forced_break != false;
	for (child_info = children; child_info; child_info = child_info->next) {
		if (child_info->widget == &child) {
			break;
		}
	}
	if (child_info && (child_info->forced_break != forced_break)) {
		child_info->forced_break = forced_break;
		if (child.is_visible() && is_visible()) {
			child.queue_resize();
		}
	}
}

guint *WrapBox::query_line_lengths(guint& n_lines) {
	//g_printerr("WrapBox::query_line_lengths called\n");
	guint *lines = NULL;
	
	n_lines = 0;
	
	GtkAllocation *allocation =  &GTK_WIDGET(gtkobj())->allocation;
	guint border = gtkobj()->border_width;
	GtkAllocation area;
	area.x = allocation->x + border;
	area.y = allocation->y + border;
	area.width = MAX(1, (gint) allocation->width - border * 2);
	area.height = MAX(1, (gint) allocation->height - border * 2);
	
	WrapBoxChild *next_child = children;
	guint max_child_size = 0;
	gboolean expand_line;
	GSList *slist = rlist_line_children(&next_child, area, max_child_size, expand_line);
	while (slist) {
		guint l = n_lines++;
		lines = g_renew(guint, lines, n_lines);
		lines[l] = g_slist_length(slist);
		g_slist_free(slist);
		slist = rlist_line_children(&next_child, area, max_child_size, expand_line);
	}
	return lines;
}

void WrapBox::map__impl() {
	//g_printerr("WrapBox::map__impl called\n");
	WrapBoxChild *child;
	set_flags(GTK_MAPPED);
	for (child = children; child; child = child->next) {
		if (child->widget->is_visible() && ! child->widget->is_mapped()) {
			//g_printerr("WrapBox::map__impl map subobject %lX\n", child);
			gtk_widget_map(child->widget->gtkobj());
		}
	}
}

void WrapBox::unmap__impl() {
	//g_printerr("WrapBox::unmap__impl called\n");
	WrapBoxChild *child;
	unset_flags(GTK_MAPPED);
	for (child = children; child; child = child->next) {
		if (child->widget->is_visible() && child->widget->is_mapped()) {
			gtk_widget_unmap(child->widget->gtkobj());
		}
	}
}

void WrapBox::draw_impl(GdkRectangle *area) {
	//g_printerr("WrapBox::draw_impl called (area: %d,%d,%d,%d)\n", area->x, area->y, area->width, area->height);
	//g_printerr("area: %d, %d, %d, %d\n", area->x, area->y, area->width, area->height);
	WrapBoxChild *child;
	Gdk_Rectangle area_ = Gdk_Rectangle(area);
	GdkRectangle child_area;
	Gdk_Rectangle child_area_ = Gdk_Rectangle(&child_area);
	
	if (is_drawable()) {
		int i = 0;
		for (child = children; child; child = child->next) {
			if (child->widget->is_drawable() && 
						child->widget->intersect(area_, child_area_)) {
				child_area = *(child_area_.gdkobj());
				//g_printerr("WrapBox::draw: Calling child draw %d (area %d,%d,%d,%d)\n", i++, child_area.x, child_area.y, child_area.width, child_area.height);
				child->widget->draw(&child_area);
			}
		}
	}
}

gint WrapBox::expose_event_impl(GdkEventExpose *event) {
	//g_printerr("WrapBox::expose_event_impl called\n");
	WrapBoxChild *child;
	GdkEventExpose child_event = *event;
	
	if (is_drawable()) {
		for (child = children; child; child = child->next) {
			if (child->widget->is_drawable() && child->widget->has_no_window() &&
					child->widget->intersect(Gdk_Rectangle(&event->area), Gdk_Rectangle(&child_event.area))) {
				gtk_widget_event(child->widget->gtkobj(), (GdkEvent*) &child_event);
			}
		}
	}
	return TRUE;
}

GtkType WrapBox::child_type_impl() {
	//g_printerr("WrapBox::child_type_impl called\n");
	return GTK_TYPE_WIDGET;
}

void WrapBox::add(Gtk::Widget& widget) {
	//g_printerr("WrapBox::add called\n");
	pack(widget, false, true, false, true);
}

void WrapBox::remove(Gtk::Widget& widget) {
	//g_printerr("WrapBox::remove called\n");
	
	WrapBoxChild *last = NULL;
	WrapBoxChild *child = children;
	while (child) {
		if (child->widget == &widget) {
			gboolean was_visible = widget.is_visible();
			gtk_widget_unparent(widget.gtkobj());
			if (last) {
				last->next = child->next;
			} else {
				children = child->next;
			}
			delete child;
			n_children--;
			
			if (was_visible) {
				queue_resize();
			}
			break;
		}
		last = child;
		child = last->next;
	}
}

void WrapBox::foreach_(GtkCallback callback, gpointer callback_data) {
	//g_printerr("WrapBox::foreach called\n");
	WrapBoxChild *child = children;
	while (child) {
		Gtk::Widget *widget = child->widget;
		child = child->next;
		callback(widget->gtkobj(), callback_data);
	}
}

WrapBoxChild::WrapBoxChild(Gtk::Widget& widget) {
	//g_printerr("WrapBoxChild::WrapBoxChild called\n");
	this->widget = &widget;
	hexpand = false;
	hfill = false;
	vexpand = false;
	vfill = false;
	forced_break = false;
	next = NULL;
}
