/* $Id: WrapBox.hh,v 1.2 2002/12/29 16:43:40 pklaus Exp $
 *
 * WrapBox.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _WRAPBOX_HH
#  define _WRAPBOX_HH

#include <gtk--.h>
#include <gdk--.h>

#define GTK_JUSTIFY_TOP    GTK_JUSTIFY_LEFT
#define GTK_JUSTIFY_BOTTOM GTK_JUSTIFY_RIGHT

class WrapBoxChild {
	
public:
	Gtk::Widget		*widget;
	guint			hexpand;
	guint			hfill;
	guint			vexpand;
	guint			vfill;
	guint			forced_break;
	
	WrapBoxChild	*next;
	
	WrapBoxChild(Gtk::Widget& widget);
};

class WrapBox : public Gtk::Container {

protected:
	guint		homogeneous;
	guint		justify;
	guint		line_justify;
	guint8		hspacing;
	guint8		vspacing;
	guint16		n_children;
	WrapBoxChild *children;
	gfloat		aspect_ratio; /* 1/256 .. 256 */
	guint		child_limit;

public:

	WrapBox();

	void set_homogeneous(gboolean homogeneous);
	
	void set_hspacing(guint hspacing);
	
	void set_vspacing(guint vspacing);
	
	void set_justify(GtkJustification justify);
	
	void set_line_justify(GtkJustification line_justify);
	
	void set_aspect_ratio(gfloat aspect_ratio);
	
	void pack(Gtk::Widget& child, gboolean hexpand, gboolean hfill,
					gboolean vexpand, gboolean vfill);
	
	void reorder_child(Gtk::Widget& child, gint position);
	
	void query_child_packing(Gtk::Widget& child, gboolean& hexpand, gboolean& hfill,
					gboolean& vexpand, gboolean& vfill);
	
	void query_child_forced_break(Gtk::Widget& child, gboolean& forced_break);
	
	void set_child_packing(Gtk::Widget& child, gboolean hexpand, gboolean hfill,
					gboolean vexpand, gboolean vfill);
	
	void set_child_forced_break(Gtk::Widget& child, gboolean forced_break);
	
	guint* query_line_lengths(guint& n_lines);
	
	virtual GSList *rlist_line_children(WrapBoxChild **child, GtkAllocation& area,
					guint& max_child_size, gboolean& expand_line) = 0;

	void map__impl();
	void unmap__impl();
	void draw_impl(GdkRectangle *p0);
	gint expose_event_impl(GdkEventExpose *p0);
	GtkType child_type_impl();
	void add(Gtk::Widget& widget);
	void remove(Gtk::Widget& widget);
	void foreach_(GtkCallback callback, gpointer callback_data);
};
#endif
