/* $Id: ZoomTool.cc,v 1.3 2004/03/20 00:01:37 pklaus Exp $
 *
 * ZoomTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/zoom.xpm"
#include "../cursors/zoom_in.xbm"
#include "../cursors/zoom_in_mask.xbm"
#include "../cursors/zoom_out.xbm"
#include "../cursors/zoom_out_mask.xbm"
#include "../cursors/end_zoom.xbm"
#include "../cursors/end_zoom_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "ZoomTool.hh"
#include "GDDocument.hh"
#include "BezierPath.hh"
#include "BezierNode.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"

#define doubleMAX(a,b)	(fabs(a) > fabs(b)) ? fabs(a) : fabs(b)

ZoomTool::ZoomTool() {
	pixmapData = zoom_xpm;
	toolName = "ZoomTool";
	moving = false;
	CursorFactory::createCursorFromData(zoomInCursor,
					zoom_in_bits, zoom_in_mask_bits,
					zoom_in_width, zoom_in_height,
					zoom_in_x_hot, zoom_in_y_hot);
	CursorFactory::createCursorFromData(zoomOutCursor,
					zoom_out_bits, zoom_out_mask_bits,
					zoom_out_width, zoom_out_height,
					zoom_out_x_hot, zoom_out_y_hot);
	CursorFactory::createCursorFromData(endZoomCursor,
					end_zoom_bits, end_zoom_mask_bits,
					end_zoom_width, end_zoom_height,
					end_zoom_x_hot, end_zoom_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
}

void ZoomTool::activate() {
	Tool::activate();
}

void ZoomTool::deactivate() {
	Tool::deactivate();
}

gint ZoomTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;
	downButton = ev->button;

	if (downButton == 1) {
		doc = docwin->doc;
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		startx = (gint) ev->x;
		starty = (gint) ev->y;
		oldx = startx;
		oldy = starty;
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint ZoomTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK && ! (ev->state & GDK_MOD1_MASK)) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		setCursor(ev->state, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// square
			gdouble delta = doubleMAX(dx-sx, dy-sy);
			dx = sx + copysign(delta, dx-sx);
			dy = sy + copysign(delta, dy-sy);
		}
		drawRectangle(drawable, doc->context, startx, starty, oldx, oldy);
		gint x = (gint) ((dx - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor);
		gint y = (gint) ((dy - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor);
		startx = (gint) ((sx - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor);
		starty = (gint) ((sy - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor);
		drawRectangle(drawable, doc->context, startx, starty, x, y);
		oldx = x;
		oldy = y;
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint ZoomTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		doc->context.set_function(GDK_SET);
		if (ev->state & GDK_MOD1_MASK) {
			if (docwin->osBuffer.scaleFactor > 0.01) {
				//g_printerr("scale: %f, width %d, height %d, sx %f, sy %f\n", docwin->scaleFactor, docwin->DocWinDrawingArea->width(), docwin->DocWinDrawingArea->height(), sx, sy);
				docwin->osBuffer.scaleFactor /= 2;
				docwin->osBuffer.origin.x = sx - (docwin->DocWinDrawingArea->width() / docwin->osBuffer.scaleFactor / 2);
				docwin->osBuffer.origin.y = sy - (docwin->DocWinDrawingArea->height() / docwin->osBuffer.scaleFactor / 2);
				//g_printerr("scale: %f, origin: %f,%f\n", docwin->scaleFactor, docwin->origin.x, docwin->origin.y);
				docwin->updateScaling();
				docwin->doc->redraw(docwin);
				//g_printerr("scale: %f, origin: %f,%f\n", docwin->scaleFactor, docwin->origin.x, docwin->origin.y);
			}
		} else if (oldx == startx && oldy == starty) {
			if (docwin->osBuffer.scaleFactor < 100) {
				docwin->osBuffer.scaleFactor *= 2;
				docwin->osBuffer.origin.x = sx - (docwin->DocWinDrawingArea->width() / docwin->osBuffer.scaleFactor / 2);
				docwin->osBuffer.origin.y = sy - (docwin->DocWinDrawingArea->height() / docwin->osBuffer.scaleFactor / 2);
				docwin->updateScaling();
				docwin->doc->redraw(docwin);
			}
		} else {
			gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			if (ev->state & GDK_SHIFT_MASK) {
				// square
				gdouble delta = doubleMAX(dx-sx, dy-sy);
				dx = sx + copysign(delta, dx-sx);
				dy = sy + copysign(delta, dy-sy);
			}
			docwin->osBuffer.origin.x = sx;
			docwin->osBuffer.origin.y = sy;
			gdouble scaleX = docwin->DocWinDrawingArea->width() / fabs(dx - sx);
			gdouble scaleY = docwin->DocWinDrawingArea->height() / fabs(dy - sy);
			docwin->osBuffer.scaleFactor = MIN(100, MIN(scaleX, scaleY));
			docwin->updateScaling();
			docwin->doc->redraw(docwin);
		}
		moving = false;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void ZoomTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
	} else {
		if (state & GDK_MOD1_MASK) {
			if (docwin->osBuffer.scaleFactor < 0.01) {
				docwin->DocWinDrawingArea->get_window().set_cursor(endZoomCursor);
			} else {
				docwin->DocWinDrawingArea->get_window().set_cursor(zoomOutCursor);
			}
		} else {
			if (docwin->osBuffer.scaleFactor > 100) {
				docwin->DocWinDrawingArea->get_window().set_cursor(endZoomCursor);
			} else {
				docwin->DocWinDrawingArea->get_window().set_cursor(zoomInCursor);
			}
		}
	}
}
